#!/bin/sh

export TET_ROOT=`pwd`
export PATH=$TET_ROOT/tet/bin:$TET_ROOT/xtest/bin:$PATH

function config_param()
{

DEFAULT=`grep "^$1=" xtest/tetexec.cfg | cut -d= -f2`
printf "$2 ("

if [ -n "$DEFAULT" ]; 
then
  printf "$DEFAULT"
fi

printf ")? "
read cmd

if [ -z "$cmd" ]; then
  cmd=$DEFAULT
fi

# Write to temporary file
echo "$1=$cmd" >> /tmp/newparam.$$

}

function configure()
{
    config_param VSW_OPER "Name of person running tests"
    config_param VSW_ORG  "Organisation"
    config_param VSW_SYS "Test System"
    config_param XT_DISPLAYHOST "Hostname of client for X tests"
    config_param XT_DISPLAY "DISPLAY of the Xvfb (ie., :1)"

    mv $TET_ROOT/xtest/tetexec.cfg $TET_ROOT/xtest/tetexec.cfg.old
    cat $TET_ROOT/xtest/tetexec.cfg.old | $TET_ROOT/tet/bin/editcfg.sh /tmp/newparam.$$ > $TET_ROOT/xtest/tetexec.cfg

    # Install xtest fonts
    DEFAULT_FONT_DIR=/usr/X11R6/lib/X11/fonts
    printf "X11 Font installation directory ($DEFAULT_FONT_DIR)? "
    read cmd
    if [ -z "$cmd" ]; then
        cmd=$DEFAULT_FONT_DIR
    fi
    echo "Enter the root password to install the test fonts"
    su root -c "install -d $DEFAULT_FONT_DIR/xtest && install xtest/fonts/*.pcf $DEFAULT_FONT_DIR/xtest && mkfontdir $DEFAULT_FONT_DIR/xtest"
    
    if [ $? -ne 0 ]; then
        echo Failed to install fonts
        exit 1
    fi

    echo
    echo "The test fonts have been installed. However, you may need"
    echo "to do other things to make the fonts visible. eg"
    echo "- editing the font server search path"
    echo "- restarting the font server"
    echo "- restarting the X server"
    echo
}

# We need to run the tests using the X Virtual Frame Buffer xvfb(1), 
# because using graphics cards gives varying results.
#

# Configure the test suites
if [ ! -f .configured ]; then
    configure
    touch .configured
fi

pkill Xvfb
(Xvfb -screen 0 1024x768x16 :1 -ac +bs -kb -fp /usr/X11R6/lib/X11/fonts/misc/,/usr/X11R6/lib/X11/fonts/xtest/ &)
echo "Starting X Virtual Frame Buffer"; sleep 20
export DISPLAY=:1.0

# Test that test fonts are installed
echo "Testing the test fonts are installed .... "
xlsfonts xtfont0 | grep xtfont0 
if [ $? -ne 0 ]; then
  echo 'Could not find xtfont0. xlsfonts xtfont? should list xtfont0..xtfont6'
  echo 'Aborting tests'
  exit 1
fi

cd xtest && tcc -e -v DISPLAY=:1

pkill Xvfb

#EOF
