/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: btmpbtordr.m,v 1.6 92/06/11 15:55:46 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XBitmapBitOrder(display)
 *   Display	*display;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XBitmapBitOrder	1
char    *TestName = "XBitmapBitOrder";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XBitmapBitOrder function
 */
static Display	*display;


static int 	ValueReturn;

static	char	*msbstr = "MSBFirst";
static	char	*lsbstr = "LSBFirst";
static	char 	errstr[9];

static
char *
sorder(order)
int 	order;
{
	switch (order) {

	case MSBFirst:
		return msbstr;
	case LSBFirst:
		return lsbstr;
	default:
		sprintf(errstr, "%d", order);
		return errstr;
	}
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XBitmapBitOrder-1.(A)
 * A call to XBitmapBitOrder returns the bitorder, either
 * LSBFirst or MSBFirst, for a scanline unit.
 */
static void
t001()
{
int	border;
XImage	*xi;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	border = XBitmapBitOrder(display);	
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	xi = XGetImage(display, DRW(display), 0,0, 1,1, XAllPlanes(), ZPixmap);

	if(border !=  xi->bitmap_bit_order) {
		report("%s() returned %s instead of %s.", TestName, sorder(border), sorder(xi->bitmap_bit_order));
		FAIL;
	} else
		PASS;

	XFree((char *) xi);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
