/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: dfltgcofsc.m,v 1.5 92/06/11 15:57:10 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   GC
 *   XDefaultGCOfScreen(screen)
 *   Screen	*screen;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XDefaultGCOfScreen	1
char    *TestName = "XDefaultGCOfScreen";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XDefaultGCOfScreen function
 */
static Screen	*screen;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	screen = DefaultScreenOfDisplay(Dsp);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XDefaultGCOfScreen-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XDefaultGCOfScreen returns the ID of the default
 * GC of the screen screen.
 */
static void
t001()
{
GC	dgc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(Dsp);
	if (isdeleted())
		return;
	dgc = XDefaultGCOfScreen(screen);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	report("Default GC has ID %lx", (long) dgc);
	CHECK;
	CHECKUNTESTED(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDefaultGCOfScreen-2.(A)
 * The default GC has the same depth as the root window of the
 * screen screen.
 */
static void
t002()
{
XWindowAttributes	atts;
GC			dgc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(Dsp);
	if (isdeleted())
		return;
	dgc = XDefaultGCOfScreen(screen);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	startcall(Dsp);
	XDrawPoint(Dsp, RootWindowOfScreen(screen), dgc, 0, 0);
	endcall(Dsp);


	if(geterr() == BadMatch) {
		report("%s() returned a GC which was not of the correct depth.", TestName);
		FAIL;
	} else
		CHECK;

	if(geterr() != Success) {
		delete("XDrawPoint() failed with a %s error.", errorname(geterr()));
		return;
	} else
		CHECK;


	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
