/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: srvrvndr.m,v 1.5 92/06/11 16:02:32 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   char *
 *   XServerVendor(display)
 *   Display	*display;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XServerVendor	1
char    *TestName = "XServerVendor";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XServerVendor function
 */
static Display	*display;


#include <string.h>
#define SEPS	":"

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XServerVendor-1.(A)
 * A call to XServerVendor returns a pointer to a
 * null-terminated string that identifies the owner of the X
 * server implementation.
 */
static void
t001()
{
char	*sv;
char	*tok;
char	*Pvendor;
Bool	matched = False;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(!config.server_vendor) {
		delete("Parameter XT_SERVER_VENDOR not set.");
		return;
	}

	startcall(display);
	if (isdeleted())
		return;
	sv = XServerVendor(display);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(!sv) {
		report("%s() returned NULL", TestName);
		FAIL;
	}

	Pvendor = (char *)malloc(strlen(config.server_vendor) * sizeof(char));
	if (Pvendor == (char *)NULL) {
		delete("Could not allocate memory for server vendor string");
		return;
	} else
		CHECK;

	strcpy(Pvendor, config.server_vendor);
	while ((tok = strsep(&Pvendor, SEPS)) != NULL)
		if(strcmp(tok, sv) == 0) {
			matched = True;
			break;
		}

	if(matched == False) {
		report("WARNING: %s() returns unknown value for server vendor string", 
						TestName);
		report("Expected value is one of \"%s\"", config.server_vendor);
		report("Observed value \"%s\"", sv);
		FIP;
		return;
	} else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
