/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: dstrywdw.m,v 1.10 92/06/11 16:07:45 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XDestroyWindow(display, w)
 *   Display	*display;
 *   Window	w;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XDestroyWindow	1
char    *TestName = "XDestroyWindow";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XDestroyWindow function
 */
static Display	*display;
static Window	w;



#define	ON_WINDOW	0x0001
#define	ON_PARENT	0x0002

extern	char	*STreeSgl[];
extern	int 	NSTreeSgl;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	w = ErrdefWindow;
}

/*
 * Assertion XDestroyWindow-1.(A)
 * A call to XDestroyWindow destroys the specified window and
 * all of its subwindows.
 */
static void
t001()
{
Window	parent;
struct	buildtree	*btlist;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	parent = defdraw(display, VI_WIN);
	btlist = buildtree(display, parent, STreeSgl, NSTreeSgl);

	/*
	 * Get the window that we are going to destroy and draw a pattern
	 * on it.
	 */
	w = btlist[1].wid;
	pattern(display, w);

	/*
	 * Check that each window exists.
	 * Touch each window with MapWindow, check for no error.
	 */
	for (i = 1; i < NSTreeSgl; i++) {
		CATCH_ERROR(display);
		XMapWindow(display, btlist[i].wid);
		RESTORE_ERROR(display);
		if (GET_ERROR(display) != Success) {
			delete("Error in setting up window tree");
			return;
		} else
			CHECK;
	}

	startcall(display);
	if (isdeleted())
		return;
	XDestroyWindow(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/* 
	 * Check that parent window is now clear, all traces of the window
	 * have gone.
	 */
	if (checkclear(display, parent))
		CHECK;
	else {
		report("Parent window was not clear after destroying its inferior");
		FAIL;
	}

	/*
	 * Check that we now get a BadWindow on the window and all its inferiors.
	 */
	for (i = 1; i < NSTreeSgl; i++) {
		CATCH_ERROR(display);
		XMapWindow(display, btlist[i].wid);
		RESTORE_ERROR(display);
		if (GET_ERROR(display) != BadWindow) {
			report("The window '%s' was still accessible", btlist[i].name);
			FAIL;
		} else
			CHECK;
	}
	CHECKPASS(2*(NSTreeSgl-1)+1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDestroyWindow-2.(A)
 * When a call to XDestroyWindow destroys a window and the
 * window is mapped, then the window is unmapped first and an
 * UnmapNotify event is generated.
 */
static void
t002()
{
XEvent	ev;
XUnmapEvent	good;
XUnmapEvent	*ump;
Window	parent;
struct	buildtree	*btlist;
struct	buildtree	*btp;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a base window and a window with inferiors on the base window. */
	parent = defdraw(display, VI_WIN);
	btlist = buildtree(display, parent, STreeSgl, NSTreeSgl);

	/* Set window that we are to destroy */
	w = btlist[1].wid;

/* Enable events on each window. */
	for (i = 0; i < NSTreeSgl; i++)
		XSelectInput(display, btlist[i].wid, ALLEVENTS);
	XSelectInput(display, parent, ALLEVENTS);

/* Call xname to destroy a window. */
	startcall(display);
	if (isdeleted())
		return;
	XDestroyWindow(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	good.type = UnmapNotify;
	good.serial = 0;
	good.send_event = False;
	good.display = display;
	good.from_configure = False;

/* Check that an UnmapNotify event was generated on the destroyed window */
/* 	and it's the parent window only.  */
	while(XCheckTypedEvent(display, UnmapNotify, &ev)) {
		ump = (XUnmapEvent *)&ev;
		debug(1,"event=0x%x, window=0x%x", ump->event, ump->window);

		btp = btwtobtp(btlist, ump->window);
		if (btp == (struct buildtree *)0) {
			FAIL;
			report("Event received on window that was not part of the test");
			report("Window ID is %x0x", ump->window);
			continue;
		} else
			CHECK;

		if (ump->window == ump->event) {
			/* Event received on window itself */
			if (btp->uflags & ON_WINDOW) {
				report("Repeat UnmapNotify event received on window '%s'",
					btp->name);
				FAIL;
			} else
				CHECK;
			btp->uflags |= ON_WINDOW;
		} else {

			/* Check event was received on the parent */
			if (ump->event == parent) {
				/*
				 * This is the Unmap reported to the parent because it was
				 * selecting SubstructureNotifyMask.
				 */
				if (ump->window != w) {
					report("UnmapNotify event was reported to the parent that on as not on the test window");
					FAIL;
				} else
					CHECK;
			} else if (btp->parent->wid != ump->event) {
				report("UnmapNotify event was reported on a window that was not the parent");
				FAIL;
			} else if (btp->uflags & ON_PARENT) {
				report("Repeat UnmapNotify event received on parent for window '%s'",
					btp->name);
				FAIL;
			} else
				CHECK;
			btp->uflags |= ON_PARENT;
		}

		/*
		 * Event and window have been dealt with - now check the other
		 * fields in the event.
		 */
		good.event = ump->event;
		good.window = ump->window;

		if (checkevent((XEvent*)&good, &ev)) {
			report("Error in event");
			FAIL;
		} else
			CHECK;

	}

	for (i = 1; i < NSTreeSgl; i++) {

		debug(2, "Checking window %s",btlist[i].name);

		if (btlist[i].wid == w) {
			if ((btlist[i].uflags & ON_WINDOW)==0) {
				FAIL;
				report("Unmap event was not received on window '%s'",
					btlist[i].name);
			} else
				CHECK;

			if ((btlist[i].uflags & ON_PARENT)==0) {
				FAIL;
				report("Unmap event was not received on the parent of window '%s'",
					btlist[i].name);
			} else
				CHECK;
		} else {
			if (btlist[i].uflags & ON_WINDOW) { 
				FAIL;
				report("Unmap event was received on window '%s'",
					btlist[i].name);
			} else
				CHECK;

			if (btlist[i].uflags & ON_PARENT) { 
				FAIL;
				report("Unmap event was received on the parent of window '%s'",
					btlist[i].name);
			} else
				CHECK;
		}
	}

	CHECKPASS(2*3 + 2*(NSTreeSgl-1));

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDestroyWindow-3.(A)
 * On a call to XDestroyWindow a DestroyNotify event is
 * generated for each window that is destroyed such that for
 * any given window being destroyed, DestroyNotify is generated
 * on any inferiors of that window before being generated on
 * the given window itself.
 */
static void
t003()
{
XEvent	ev;
XDestroyWindowEvent	good;
XDestroyWindowEvent	*dnp;
Window	parent;
struct	buildtree	*btlist;
struct	buildtree	*btp;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	parent = defdraw(display, VI_WIN);
	btlist = buildtree(display, parent, STreeSgl, NSTreeSgl);

	/* Window to be destroyed */
	w = btlist[1].wid;

	/*
	 * Unmap a window and make sure that everything has happened before we
	 * enable events.
	 */
	btp = btntobtp(btlist, "A1");
	btp->opts |= BT_UNMAP;
	XUnmapWindow(display, btp->wid);
	XSync(display, False);

	for (i = 0; i < NSTreeSgl; i++)
		XSelectInput(display, btlist[i].wid, ALLEVENTS);
	XSelectInput(display, parent, ALLEVENTS);

	startcall(display);
	if (isdeleted())
		return;
	XDestroyWindow(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	good.type = DestroyNotify;
	good.serial = 0;
	good.send_event = False;
	good.display = display;

	/*
	 * This loop continues until there are no more events left.  (In case
	 * there are extra or repeated events).  The loop should be traversed
	 * 2*(NSTreeSgl-1) times.
	 */
	for (i = 0; ; i++) {
		if (XCheckTypedEvent(display, DestroyNotify, &ev) == False)
			break;
		dnp = (XDestroyWindowEvent*)&ev;

		debug(1, "event=0x%x, window=0x%x", dnp->event, dnp->window);

		/*
		 * If window == event then this is the event from StructureNotify,
		 * if they are different then this is the event from the
		 * SubstructureNotify.  Keep track of events received and check
		 * that event is actualy set to the parent of window when they
		 * are different.
		 * Check that a destroy event has not yet been received on the
		 * parent window.
		 * Finally check the other members of the Event.
		 */

		btp = btwtobtp(btlist, dnp->window);
		if (btp == (struct buildtree *)0) {
			report("Event received for a window that was not part of the test");
			report("  Window ID was 0x%x", dnp->window);
			FAIL;
			break;
		}

		debug(1, "DestroyNotify received for window %s", btp->name);

		if (dnp->window == dnp->event) {
			if (btp->uflags & ON_WINDOW) {
				report("Repeat DestroyNotify event received on window '%s'",
					btp->name);
				FAIL;
			} else if (btp->parent->uflags & ON_WINDOW) {
				/* Test ordering */
				report("DestroyNotify event received on parent before child");
				FAIL;
			} else
				CHECK;
			btp->uflags |= ON_WINDOW;
		} else {

			/* Check event was received on the parent */
			if (dnp->event == parent) {
				/*
				 * This is the Unmap reported to the parent because it was
				 * selecting SubstructureNotifyMask.
				 */
				if (dnp->window != w) {
					report("UnmapNotify event was reported on a window that was not the parent");
					FAIL;
				} else
					CHECK;
			} else if (btp->parent->wid != dnp->event) {
				report("DestroyNotify event was reported on a window that was not the parent");
				FAIL;
			} else if (btp->uflags & ON_PARENT) {
				report("DestroyNotify event already received on parent for window '%s'",
					btp->name);
				FAIL;
			} else
				CHECK;
			btp->uflags |= ON_PARENT;
		}

		/* Check other fields in event */
		good.event = dnp->event;
		good.window = dnp->window;

		if (checkevent((XEvent*)&good, &ev)) {
			report("Error in event");
			FAIL;
		} else
			CHECK;

	}

	/* Verify that the window and all subwindows had both notifications */
	for (i = 1; i < NSTreeSgl; i++) {

		if ((btlist[i].uflags & ON_WINDOW) == 0) {
			report("DestroyNotify event was not received on subwindow '%s'",
				btlist[i].name);
			FAIL;
		} else
			CHECK;
			
		if ((btlist[i].uflags & ON_PARENT) == 0) {
			report("DestroyNotify event was not received on the parent of subwindow '%s'",
				btlist[i].name);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS((2*2*(NSTreeSgl-1)) + 2*(NSTreeSgl-1));

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDestroyWindow-4.(A)
 * When the specified window is a root window, then no windows
 * are destroyed.
 */
static void
t004()
{
Window	ourwin;
XWindowAttributes	atts;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * What would happen if the root really was destroyed?!
	 * Just do a couple of simple checks to make sure that nothing
	 * bad has happened.
	 */
	ourwin = defdraw(display, VI_WIN);
	w = DefaultRootWindow(display);

	startcall(display);
	if (isdeleted())
		return;
	XDestroyWindow(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	CATCH_ERROR(display);
	XGetWindowAttributes(display, DRW(display), &atts);
	RESTORE_ERROR(display);
	if (GET_ERROR(display) == BadWindow) {
		report("Root window was destroyed");	/* ! */
		FAIL;
	} else
		CHECK;

	CATCH_ERROR(display);
	XGetWindowAttributes(display, ourwin, &atts);
	RESTORE_ERROR(display);
	if (GET_ERROR(display) == BadWindow) {
		report("A child of the root window was destroyed when attempting to destroy root window");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XDestroyWindow-5.(A)
 * When a call to XDestroyWindow uncovers part of any window
 * that was formerly obscured, then either Expose events are
 * generated or the contents are restored from backing store.
 */
static void
t005()
{
Window	parent;
struct	area	area;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	parent = defdraw(display, VI_WIN);
	setforexpose(display, parent);
	XSelectInput(display, parent, ExposureMask);

	setarea(&area, 25, 35, 17, 46);
	w = crechild(display, parent, &area);

	startcall(display);
	if (isdeleted())
		return;
	XDestroyWindow(display, w);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (exposecheck(display, parent))
		CHECK;
	else {
		report("Neither Expose events or backing store processing");
		report("could correctly restore the window contents.");
		FAIL;
	}

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XDestroyWindow-6.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XDestroyWindow(display, w);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
