/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stwdwbrdr.m,v 1.7 92/06/11 16:10:47 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetWindowBorder(display, w, border_pixel)
 *   Display	*display;
 *   Window	w;
 *   unsigned long	border_pixel;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetWindowBorder	1
char    *TestName = "XSetWindowBorder";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XSetWindowBorder function
 */
static Display	*display;
static Window	w;
static unsigned long	border_pixel;


static Window	parent;
static struct	area	ap;

static void
inittp()
{
	tpstartup();

	ap.x = 50;
	ap.y = 60;
	ap.width = 20;
	ap.height= 20;
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = 0;
	border_pixel = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	w = ErrdefWindow;
}

/*
 * Assertion XSetWindowBorder-1.(A)
 * A call to XSetWindowBorder sets the border of the window to
 * the pixel value specified by border_pixel.
 */
static void
t001()
{
unsigned long	pixel;
Pixmap	pm;
XEvent	event;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	parent = defdraw(display, VI_WIN);

	w = creunmapchild(display, parent, &ap);

	XSetWindowBorderWidth(display, w, 5);
	XSetWindowBorderPixmap(display, w, maketile(display,w));

	XSelectInput(display, w, ExposureMask);	
	XMapWindow(display, w);
	XWindowEvent(display,w,ExposureMask, &event); /* Await exposure */

	pixel = getpixel(display, parent, ap.x, ap.y);
	border_pixel = (pixel == W_FG) ? W_BG:W_FG;
	trace("pixel at ap.x,ap.y: %d . Setting to %d", pixel, border_pixel);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorder(display, w, border_pixel);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XUnmapWindow(display, w);
	XMapWindow(display, w);
	XWindowEvent(display,w,ExposureMask, &event); /* Await exposure */

	if(getpixel(display, parent, ap.x, ap.y) != border_pixel) {
		report("%s did not set the border pixel to %d",
				TestName, border_pixel);
		FAIL;
	} else
		CHECK;

	border_pixel = pixel;
	trace("Now setting border to %d", border_pixel);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorder(display, w, border_pixel);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XUnmapWindow(display, w);
	XMapWindow(display, w);
	XWindowEvent(display,w,ExposureMask, &event); /* Await exposure */

	if(getpixel(display, parent, ap.x, ap.y) != border_pixel) {
		report("%s did not set the border pixel to %d",
				TestName, border_pixel);
		FAIL;
	} else
		CHECK;
	
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBorder-2.(A)
 * When the border pixel value is changed, then the border is
 * repainted.
 */
static void
t002()
{
unsigned long	pixel;
Pixmap	pm;
XEvent	event;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

/* Create windows */
	parent = defdraw(display, VI_WIN);
	w = creunmapchild(display, parent, &ap);
/* Set up child window border for test */
	XSetWindowBorderWidth(display, w, 5);
	XSetWindowBorderPixmap(display, w, maketile(display,w));
/* Ensure the window is mapped and visible */
	XSelectInput(display, w, ExposureMask);	
	XMapWindow(display, w);
	XWindowEvent(display,w,ExposureMask, &event); /* Await exposure */

	pixel = getpixel(display, parent, ap.x, ap.y);
	border_pixel = (pixel == W_FG) ? W_BG:W_FG;
	trace("pixel at ap.x,ap.y: %d . Setting to %d", pixel, border_pixel);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorder(display, w, border_pixel);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	
	if(getpixel(display, parent, ap.x, ap.y) != border_pixel) {
		report("%s did not cause a repaint of the border", TestName);
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWindowBorder-3.(A)
 * The border pixel value is truncated to the depth of the
 * window.
 */
static void
t003()
{
XVisualInfo	*vp;
static int 	pixlist[] = {
	0, 1, 3, 4, 17, 18, 200, 300, 303,
	0x1234, 0x12345, 0x123456, 0x1234567, 0x12345678};
long	pix;
long	borderpix;
int 	i;
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {
		parent = makedrawable(display, vp);

		for (i = 0; i < NELEM(pixlist); i++) {
			pix = pixlist[i];

			border_pixel = pix;

			w = crechild(display, parent, &ap);
			XSetWindowBorderWidth(display, w, 3);

			startcall(display);
			if (isdeleted())
				return;
			XSetWindowBorder(display, w, border_pixel);
			endcall(display);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}

			borderpix = getpixel(display, parent, ap.x, ap.y);
			if (borderpix == (pix & DEPTHMASK(vp->depth)))
				CHECK;
			else {
				report("Border pixel was not truncated (value 0x%x)", pix);
				report("  Was 0x%x, expecting 0x%x", borderpix,
					pix & DEPTHMASK(vp->depth));
				FAIL;
			}
		}
	}

	CHECKPASS(nvinf() * NELEM(pixlist));

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EMat4.mc */
/*
 * Assertion XSetWindowBorder-4.(A)
 * When the window argument is an InputOnly window then a
 * BadMatch error occurs.
 */
/* End of included file error/EMat4.mc */

/* Including from file error/EMat4.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	seterrdef();

	A_WINDOW = iponlywin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorder(display, w, border_pixel);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EMat4.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XSetWindowBorder-5.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	inittp();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetWindowBorder(display, w, border_pixel);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
