/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: cnvrtslctn.m,v 1.5 92/06/11 16:12:31 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XConvertSelection(display, selection, target, property, requestor, test_time)
 *   Display *display;
 *   Atom selection;
 *   Atom target;
 *   Atom property;
 *   Window requestor;
 *   Time  test_time;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XConvertSelection	1
char    *TestName = "XConvertSelection";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_ATOM selection
#define A_WINDOW requestor
#define A_DRAWABLE requestor


/*
 * Arguments to the XConvertSelection function
 */
static Display *display;
static Atom selection;
static Atom target;
static Atom property;
static Window requestor;
static Time  test_time;


#include "Xatom.h"

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	selection = XA_COPYRIGHT;
	target = XA_NOTICE;
	property = XA_INTEGER;
	requestor = defwin(display);
	test_time = CurrentTime;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XConvertSelection-1.(A)
 * When the specified selection has an owner, then a call to
 * XConvertSelection generates a SelectionRequest event to the
 * selection owner, with selection, target, property,
 * requestor, and time arguments passed unchanged as event
 * structure members.
 */
static void
t001()
{
Display *client1, *client2;
Window owner;
int num_ev;
XEvent ev;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a new client. */
	client1 = display;
	client2 = opendisplay();
	if (client2 == (Display *)NULL) {
		delete("Could not create client2");
		return;
	} else
		CHECK;

/* Create a window with a selection which it owns on client2. */
	owner = defwin(client2);
	XSetSelectionOwner(client2, selection, owner, CurrentTime);
	XSync(client2, True);

/* Call xname to convert the selection. */
	startcall(display);
	if (isdeleted())
		return;
	XConvertSelection(display, selection, target, property, requestor, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XSync(client1, False);
	XSync(client2, False);

/* Verify that the correct SelectionNotify event was delivered to client2 */
	num_ev = getevent(client2, &ev);
	if (num_ev != 1) {
		FAIL;
		report("%s did not cause a single SelectionNotify event",
			TestName);
		trace("Received %d events", num_ev);
		while (num_ev > 0) {
			trace("Event %s", eventname(ev.type));
			num_ev = getevent(client2, &ev);
		}
	} else {
		XEvent good;

		good.type = SelectionRequest;
		good.xselectionrequest.type = SelectionRequest;
		good.xselectionrequest.display = client2;
		good.xselectionrequest.owner = owner;
		good.xselectionrequest.requestor = requestor;
		good.xselectionrequest.selection = selection;
		good.xselectionrequest.target = target;
		good.xselectionrequest.property = property; 
		good.xselectionrequest.time	= -1;

		if (checkevent(&good, &ev)) {
			FAIL;
			report("SelectionNotify event was incorrect");
		} else
			CHECK;
	}

/* Verify no events were delivered to client1. */
	num_ev = getevent(client1, &ev);
	if (num_ev != 0) {
		FAIL;
		report("%s generated unexpected events on client1",
			TestName);
		trace("Expected 0 events");
		trace("Received %d events", num_ev);
		do {
			trace("Event: %s", eventname(ev.type));
		} while(getevent(client1, &ev));
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConvertSelection-2.(A)
 * When the specified selection has no owner, then a call to
 * XConvertSelection generates a SelectionNotify event to the
 * requestor window with selection, target and time arguments
 * passed unchanged as event structure members, and with
 * property set to None.
 */
static void
t002()
{
int num_ev;
XEvent ev;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call xname to convert the selection. */
	startcall(display);
	if (isdeleted())
		return;
	XConvertSelection(display, selection, target, property, requestor, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	XSync(display, False);

/* Verify that the correct SelectionNotify event was delivered to display. */
	num_ev = getevent(display, &ev);
	if (num_ev != 1) {
		FAIL;
		report("%s did not cause a single SelectionNotify event",
			TestName);
		trace("Received %d events", num_ev);
		while (num_ev > 0) {
			trace("Event %s", eventname(ev.type));
			num_ev = getevent(display, &ev);
		}
	} else {
		XEvent good;

		good.type = SelectionNotify;
		good.xselection.type = SelectionNotify;
		good.xselection.display = display;
		good.xselection.requestor = requestor;
		good.xselection.selection = selection;
		good.xselection.target = target;
		good.xselection.property = None; /* is passed with no owner */
		good.xselection.time	= -1;

		if (checkevent(&good, &ev)) {
			FAIL;
			report("SelectionNotify event was incorrect");
		} else
			CHECK;
	}

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XConvertSelection-3.(A)
 * The atoms PRIMARY and SECONDARY are predefined selection
 * atoms.
 */
static void
t003()
{
Atom primary, secondary;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Obtain the server representation for the PRIMARY and SECONDARY atoms. */
	primary = XInternAtom(display, "PRIMARY", True);
	secondary = XInternAtom(display, "SECONDARY", True);

/* Verify that the atoms were defined. */
	if (primary == None) {
		FAIL;
		report("PRIMARY was not defined");
	} else
		CHECK;

	if (secondary == None) {
		FAIL;
		report("SECONDARY was not defined");
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XConvertSelection-4.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XConvertSelection(display, selection, target, property, requestor, test_time);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* Including from file error/EAto.mc */
/*
 * Assertion XConvertSelection-5.(A)
 * When an atom argument does not name a valid Atom, then a
 * BadAtom error occurs.
 */
/* End of included file error/EAto.mc */

/* Including from file error/EAto.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_ATOM = ~0L;
	startcall(display);
	if (isdeleted())
		return;
	XConvertSelection(display, selection, target, property, requestor, test_time);
	endcall(display);
	if (geterr() != BadAtom) {
		report("Got %s, Expecting BadAtom", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadAtom)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAto.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
