/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtslctnown.m,v 1.4 92/06/11 16:13:02 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Window
 *   XGetSelectionOwner(display, selection)
 *   Display *display;
 *   Atom selection;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetSelectionOwner	1
char    *TestName = "XGetSelectionOwner";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_ATOM selection


/*
 * Arguments to the XGetSelectionOwner function
 */
static Display *display;
static Atom selection;


#include "Xatom.h"

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	selection = XA_COPYRIGHT;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetSelectionOwner-1.(A)
 * A call to XGetSelectionOwner returns the window ID
 * associated with the window that currently owns the specified
 * selection.
 */
static void
t001()
{
Window owner, ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a window with a selection. */
	owner = defwin(display);
	XSetSelectionOwner(display, selection, owner, CurrentTime);

/* Call xname to obtain the window ID of the selection owner. */
	startcall(display);
	if (isdeleted())
		return;
	ret = XGetSelectionOwner(display, selection);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify the window ID returned was that of the selection owner. */
	if (ret != owner) {
		FAIL;
		report("%s did not return the window ID of the selection owner",
			TestName);
		trace("Expected window ID: %0x", owner);
		trace("Returned window ID: %0x", ret);
	} else
		CHECK;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetSelectionOwner-2.(A)
 * When the selection is not owned by a window, then a call to
 * XGetSelectionOwner returns None.
 */
static void
t002()
{
Window ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Ensure the selection has no owner. */
	XSetSelectionOwner(display, selection, None, CurrentTime);

/* Call xname to obtain selection owner. */
	startcall(display);
	if (isdeleted())
		return;
	ret = XGetSelectionOwner(display, selection);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that xname returned None. */
	if(ret != None) {
		FAIL;
		report("%s returned %0x on a selection with no owner.",
			TestName);
		report("Expecting None.");
	} else
		CHECK;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAto.mc */
/*
 * Assertion XGetSelectionOwner-3.(A)
 * When an atom argument does not name a valid Atom, then a
 * BadAtom error occurs.
 */
/* End of included file error/EAto.mc */

/* Including from file error/EAto.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_ATOM = ~0L;
	startcall(display);
	if (isdeleted())
		return;
	XGetSelectionOwner(display, selection);
	endcall(display);
	if (geterr() != BadAtom) {
		report("Got %s, Expecting BadAtom", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadAtom)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAto.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
