/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: lstprprts.m,v 1.7 92/06/11 16:13:32 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Atom *
 *   XListProperties(display, w, num_prop_return)
 *   Display *display;
 *   Window w;
 *   int *num_prop_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XListProperties	1
char    *TestName = "XListProperties";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XListProperties function
 */
static Display *display;
static Window w;
static int *num_prop_return;


#include "Xatom.h"

static int num_prop;

static Atom xlp_list[] = {
          XA_PRIMARY,
         XA_CUT_BUFFER0,
         XA_RECTANGLE,
         XA_COPYRIGHT,
};
static int xlp_nlist = NELEM(xlp_list);


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = defwin(display);
	num_prop_return = &num_prop;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XListProperties-1.(A)
 * When the specified window w has properties defined, then a
 * call to XListProperties returns a pointer to an array of
 * atom properties that are defined for the specified window w
 * and can be freed with XFree, and returns the number of
 * properties in the array in num_prop_return.
 */
static void
t001()
{
int loop,loop2;
int found;
char *data = "a tested property";
Atom *ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a window with properties. */
	for(loop=0; loop<xlp_nlist; loop++)
		XChangeProperty(display, w, xlp_list[loop], XA_STRING, 8,
			PropModeReplace,(unsigned char *)data, strlen(data));

/* Call xname to obtain the property list for the window. */
	num_prop = -1;
	startcall(display);
	if (isdeleted())
		return;
	ret = XListProperties(display, w, num_prop_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that the number of properties returned was as expected. */
	if (num_prop != xlp_nlist) {
		FAIL;
		report("%s returned an unexpected num_prop_return",
			TestName);
		trace("Expected value: %d", xlp_nlist);
		trace("Returned value: %d", num_prop);
	} else
		CHECK;

	if (ret == (Atom *)NULL) {
		FAIL;
		report("%s returned a NULL pointer.", TestName);
		report("Expecting a pointer to a list of atoms.");
		return;
	} else
		CHECK;
		
	
	found = 0;

/* Verify that the correct properties were returned. */
	for (loop=0; loop<num_prop; loop++) {
		for(loop2=0; loop2<xlp_nlist; loop2++) {
			if( ret[loop] == xlp_list[loop2] ) {
				found++;
				break;
			}
		}
	}

#ifdef TESTING
	found = 0;
#endif

	if (found != num_prop) {
		FAIL;
		report("%s returned unexpected properties", TestName);
		trace("Expected properties");
		for (loop=0; loop<xlp_nlist; loop++)
			trace(" %s", atomname(xlp_list[loop]));
		trace("Returned properties");
		for (loop=0; loop<num_prop; loop++)
			trace(" %s", atomname(ret[loop]));
	} else
		CHECK;

/* Verify the list may be XFree'd. */
	if (num_prop != 0)
	{
		XFree((char*)ret);
		CHECK;
	}

	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XListProperties-2.(A)
 * When the specified window w has no properties defined, then
 * a call to XListProperties returns NULL, and zero in
 * num_prop_return.
 */
static void
t002()
{
Atom *ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a window with no properties. */
/* Call xname to obtain the property list for the window. */
	num_prop = -1;
	startcall(display);
	if (isdeleted())
		return;
	ret = XListProperties(display, w, num_prop_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that a NULL pointer was returned. */
	if (ret != (Atom *)NULL) {
		FAIL;
		report("%s returned an unexpected value", TestName);
		trace("Expected value: NULL pointer");
		trace("Returned value: non-NULL pointer");
	} else
		CHECK;

/* Verify that num_prop_return was zero. */
	if (num_prop != 0) {
		FAIL;
		report("%s returned an unexpected num_prop_return",
			TestName);
		trace("Expected value: 0");
		trace("Returned value: %d", num_prop);
	} else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XListProperties-3.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XListProperties(display, w, num_prop_return);
	endcall(display);
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
