/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: crtgc.m,v 1.14 92/06/11 16:15:35 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   GC
 *   XCreateGC(display, d, valuemask, values)
 *   Display *display;
 *   Drawable d;
 *   unsigned long valuemask;
 *   XGCValues *values;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XCreateGC	1
char    *TestName = "XCreateGC";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_DRAWABLE d


/*
 * Arguments to the XCreateGC function
 */
static Display *display;
static Drawable d;
static unsigned long valuemask;
static XGCValues *values;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	d = 0;
	valuemask = 0L;
	values = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	d = ErrdefDrawable;
}

/*
 * Assertion XCreateGC-1.(A)
 * A call to XCreateGC creates a graphics context and returns
 * a GC which may be used with any destination drawable having
 * the same root and depth as the drawable d.
 */
static void
t001()
{
XVisualInfo	*vp;
Window		win;
int		nvis;
Pixmap		pmap;
GC		gc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(resetvinf(VI_WIN); 	nextvinf(&vp);) {
		d = makewin(display, vp);

		startcall(display);
		if (isdeleted())
			return;
		gc = XCreateGC(display, d, valuemask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		XDrawPoint(display,d,gc,0,0);

		/* 0 is the default foreground pixel value. */
		if( ! checkpixel(display, d, 0, 0, 0) ) {  
			report("For window depth %d, pixel was not set to foreground",
								vp -> depth);
			FAIL;
		} else
			CHECK;
	}

	nvis = nvinf();
	for(resetvinf(VI_PIX); 	nextvinf(&vp);) {
		d = makepixm(display, vp);		

		startcall(display);
		if (isdeleted())
			return;
		gc = XCreateGC(display, d, valuemask, values);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		XDrawPoint(display,d,gc,0,0);

		/* 0 is the default foreground pixel value. */
		if( ! checkpixel(display, d, 0, 0, 0) ) {  
			report("For pixmap depth %d, pixel was not set to foreground",
								vp -> depth);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(nvis + nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-2.(A)
 * A call to XCreateGC initialises the components specified by
 * the valuemask argument in the newly created GC to the values
 * in the values argument.
 */
static void
t002()
{
GC gc;
XWindowAttributes atts;
Status  s;
Pixmap pm;
XGCValues vals;
XVisualInfo *vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Function */
	
	resetvinf(VI_WIN);
	nextvinf(&vp);

	d = makewin(display, vp);
	vals.function = GXxor;
	vals.foreground = W_FG;
	values = &vals;
	valuemask = (GCFunction | GCForeground);
	startcall(display);
	if (isdeleted())
		return;
	gc = XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawPoint(display, d, gc, 0, 0);

	if( ! checkpixel(display, d, 0, 0, W_FG)) {
		delete("Pixel at (0, 0) was not set to foreground.");
		return;
	} else
		CHECK;

	vals.foreground = W_FG ^ W_BG;
	values = &vals;
	valuemask = (GCFunction | GCForeground);
	startcall(display);
	if (isdeleted())
		return;
	gc = XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawPoint(display, d, gc, 0, 0);

	if( ! checkpixel(display, d, 0, 0, W_BG)) {
		report("Pixel at (0, 0) was not set to background.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-3.(A)
 * When the tile pixmap is not set and the fill_style is
 * FillTiled, then a call to XCreateGC creates a GC in which a
 * pixel set to the foreground value of GC is used for tiling.
 */
static void
t003()
{
XVisualInfo	*vp;
XGCValues	vals;
GC		gc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN); 
	nextvinf(&vp);
	d = makewin(display, vp);
	vals.fill_style = FillTiled;
	vals.foreground = W_FG;
	valuemask = GCForeground | GCFillStyle;
	values = &vals;
	startcall(display);
	if (isdeleted())
		return;
	gc = XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XFillRectangle(display, d, gc, 0, 0, W_STDWIDTH, W_STDHEIGHT);

	if( checkarea(display, d, (struct area *) 0, W_FG, 0, CHECK_IN) == False) {
		report("Window was not filled with foreground.");
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XCreateGC-4.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EDra.mc */
/*
 * Assertion XCreateGC-5.(A)
 * When a drawable argument does not name a valid Drawable,
 * then a BadDrawable error occurs.
 */
/* End of included file error/EDra.mc */

static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	d = badwin(Dsp);
	valuemask = 0L;
	values = (XGCValues *)NULL;
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadDrawable) {
		report("Got %s, Expecting BadDrawable", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadDrawable)
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-6.(A)
 * When the font does not name a valid font, and the GCFont
 * bit is set in valuemask, then a BadFont error occurs.
 */
static void
t006()
{
Window w;
Font font;
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	w = DefaultRootWindow(Dsp);
	font = badfont(Dsp);

	vals.font = font;
	d = w;
	valuemask = GCFont;
	values = &vals;

	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadFont) {
		report("Got %s, Expecting BadFont", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadFont) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-7.(A)
 * When the tile does not name a valid pixmap, and the GCTile
 * bit is set in valuemask, then a BadPixmap error occurs.
 */
static void
t007()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.tile = badpixm(display);

	values = &vals;
	d = DRW(display);
	valuemask = GCTile;

	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadPixmap) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-8.(A)
 * When the stipple does not name a valid pixmap, and the
 * GCStipple bit is set in valuemask, then a BadPixmap error
 * occurs.
 */
static void
t008()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.stipple = badpixm(display);

	values = &vals;
	d = DRW(display);
	valuemask = GCStipple;

	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadPixmap) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-9.(A)
 * When the clip-mask does not name a valid pixmap, or None,
 * and the GCClipMask bit is set in valuemask, then a
 * BadPixmap error occurs.
 */
static void
t009()
{
Window w;
Pixmap pm;
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.clip_mask = badpixm(display);

	values = &vals;
	d = DRW(display);
	valuemask = GCClipMask;

	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadPixmap) {
		report("Got %s, Expecting BadPixmap", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadPixmap)
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-10.(C)
 * When the graphics context and the tile pixmap do not have
 * the same depth, and the GCTile bit is set in valuemask,
 * then a BadMatch error occurs.
 */
static void
t010()
{
int		count;
XGCValues 	vals;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	count = 0;
	vals.tile = XCreatePixmap(display, DRW(display), 1,1,1);
	valuemask = GCTile;
	values = &vals;
	for(resetvinf(VI_WIN); nextvinf(&vp);)
		if(vp->depth != 1) {
			trace("Testing a window of depth %d",vp->depth);
			count++;
			d = makewin(display, vp);
			startcall(display);
			if (isdeleted())
				return;
			XCreateGC(display, d, valuemask, values);
			endcall(display);
			if (geterr() != BadMatch) {
				report("Got %s, Expecting BadMatch", errorname(geterr()));
				FAIL;
			}
			if(geterr() == BadMatch)
				CHECK;
			else
				FAIL;
		}

	for(resetvinf(VI_PIX); nextvinf(&vp);)
		if(vp->depth != 1) {
			trace("Testing a window of depth %d",vp->depth);
			count++;
			d = makepixm(display, vp);
			startcall(display);
			if (isdeleted())
				return;
			XCreateGC(display, d, valuemask, values);
			endcall(display);
			if (geterr() != BadMatch) {
				report("Got %s, Expecting BadMatch", errorname(geterr()));
				FAIL;
			}
			if(geterr() == BadMatch)
				CHECK;
			else
				FAIL;
		}


	if(count == 0) {
		tet_result(TET_UNSUPPORTED);
		report("Only depth one drawables are supported.");
		return;
	} else
		CHECKPASS(count);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-11.(A)
 * When the graphics context and the tile pixmap were not
 * created for the same root, and the GCTile bit is set in
 * valuemask, then a BadMatch error occurs.
 */
static void
t011()
{
char    *altroot;
int     scr_num;
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	altroot = tet_getvar("XT_ALT_SCREEN");
	if (altroot == NULL) {
		delete("XT_ALT_SCREEN not set");
		return;
	}
	if (*altroot == 'U') {
		report("Only one screen supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	scr_num = atoi(altroot);
	if (scr_num == DefaultScreen(display)) {
		delete("The Alternate root was the same as the one under test");
		return;
	}
	if (scr_num >= ScreenCount(display)) {
		delete("Screen given in XT_ALT_SCREEN could not be accessed");
		return;
	}

	/*
	 * Create a 1x1 depth 1 pixmap on other screen
	 * and use it to create a gc
	 */
	
	vals.tile =  XCreatePixmap(display, RootWindow(display, scr_num), 1, 1, 1);
	values = &vals;
	valuemask = GCTile;
	d = DRW(display);

	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-12.(C)
 * When the stipple pixmap does not have depth one, and the
 * GCStipple bit is set in valuemask, then a BadMatch error
 * occurs.
 */
static void
t012()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((vals.stipple = nondepth1pixmap(display, DRW(display))) == (Pixmap) 0) {
		tet_result(TET_UNSUPPORTED);
		report("Only depth 1 pixmaps are supported.");
		return;
	} else
		CHECK;

	d = DRW(display);
	valuemask = GCStipple;
	values = &vals;
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}
	
	if(geterr() == BadMatch)
		CHECK;
	else
		FAIL;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-13.(C)
 * When the graphics context and the stipple pixmap were not
 * created for the same root, and the GCStipple bit is set in
 * valuemask, then a BadMatch error occurs.
 */
static void
t013()
{
char    *altroot;
int     scr_num;
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	altroot = tet_getvar("XT_ALT_SCREEN");
	if (altroot == NULL) {
		delete("XT_ALT_SCREEN not set");
		return;
	}
	if (*altroot == 'U') {
		report("Only one screen supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	scr_num = atoi(altroot);
	if (scr_num == DefaultScreen(display)) {
		delete("The Alternate root was the same as the one under test");
		return;
	}
	if (scr_num >= ScreenCount(display)) {
		delete("Screen given in XT_ALT_SCREEN could not be accessed");
		return;
	}

	/*
	 * Create a 1x1 depth 1 pixmap on other screen
	 * and use it to create a gc
	 */
	
	d = DRW(display);
	vals.stipple = XCreatePixmap(display, RootWindow(display, scr_num), 1, 1, 1);
	values = &vals;
	valuemask = GCStipple;
	
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-14.(C)
 * When the clip-mask is set to a pixmap, and the clip-mask
 * does not have depth one, and the GCClipMask bit is set in
 * valuemask, then a BadMatch error occurs.
 */
static void
t014()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if((vals.clip_mask = nondepth1pixmap(display, DRW(display))) == (Pixmap) 0) {
		tet_result(TET_UNSUPPORTED);
		report("Only depth 1 pixmaps are supported.");
		return;
	} else
		CHECK;

	d = DRW(display);
	valuemask = GCClipMask;
	values = &vals;
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadMatch)
		CHECK;
	else
		FAIL;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-15.(C)
 * When the clip-mask is set to a pixmap, and the graphics
 * context and the clip-mask were not created for the same
 * root, and the GCClipMask bit is set in valuemask, then a
 * BadMatch error occurs.
 */
static void
t015()
{
char    *altroot;
int     scr_num;
XGCValues	vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	altroot = tet_getvar("XT_ALT_SCREEN");
	if (altroot == NULL) {
		delete("XT_ALT_SCREEN not set");
		return;
	}
	if (*altroot == 'U') {
		report("Only one screen supported");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	scr_num = atoi(altroot);
	if (scr_num == DefaultScreen(display)) {
		delete("The Alternate root was the same as the one under test");
		return;
	}
	if (scr_num >= ScreenCount(display)) {
		delete("Screen given in XT_ALT_SCREEN could not be accessed");
		return;
	}

	/*
	 * Create a 1x1 depth 1 pixmap on other screen
	 * and use it to create a gc
	 */

	d = DRW(display);
	vals.clip_mask = XCreatePixmap(display, RootWindow(display, scr_num), 1, 1, 1);
	valuemask = GCClipMask;
	values = &vals;

	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadMatch) {
		report("Got %s, Expecting BadMatch", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadMatch)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-16.(A)
 * When the valuemask argument is other than a bitwise OR of
 * none or any of GCFunction, GCPlaneMask, GCForeground,
 * GCBackground, GCLineWidth, GCLineStyle, GCCapStyle,
 * GCJoinStyle, GCFillStyle, GCFillRule, GCTile, GCStipple,
 * GCTileStipXOrigin, GCTileStipYOrigin, GCFont,
 * GCSubwindowMode, GCGraphicsExposures, GCClipXOrigin,
 * GCClipYOrigin, GCClipMask, GCDashOffset, GCDashList, or
 * GCArcMode, then a BadValue error occurs.
 */
static void
t016()
{
Window w;
XGCValues vals;
int i;
unsigned char *p;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	/* first fill XGCValues with ff's and try creating that rubbish */

	i = sizeof(XGCValues);
	p = (unsigned char *)&vals;
	while( i-- > 0 )
		*p++ = 0xff;

	d = DRW(display);
	values = &vals;
	vals.foreground = 1;
	vals.function = ~0;
	valuemask = GCFunction | GCForeground | 
	 ~(GCFunction | GCPlaneMask | GCForeground | GCBackground |
	  GCLineWidth | GCLineStyle | GCCapStyle | GCJoinStyle |
	  GCFillStyle | GCFillRule | GCTile | GCStipple |
	  GCTileStipXOrigin | GCTileStipYOrigin | GCFont | GCSubwindowMode |
	  GCGraphicsExposures | GCClipXOrigin | GCClipYOrigin | GCClipMask |
	  GCDashOffset | GCDashList | GCArcMode);
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadValue)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-17.(A)
 * When the function is other than GXclear, GXand,
 * GXandReverse, GXcopy, GXandInverted, GXnoop, GXxor,
 * GXor, GXnor, GXequiv, GXinvert, GXorReverse,
 * GXcopyInverted, GXorInverted, GXnand or GXset, and the
 * GCFunction bit is set in valuemask, then a BadValue error
 * occurs.
 */
static void
t017()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.function = GXclear + GXand + GXandReverse + GXcopy + 
	  GXandInverted + GXnoop + GXxor + GXor + GXnor + GXequiv + GXinvert + 
	  GXorReverse + GXcopyInverted + GXorInverted + GXnand + GXset + 1;
	values = &vals;
	valuemask = GCFunction;
	d = DRW(display);
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-18.(A)
 * When the line_style is other than LineSolid,
 * LineDoubleDash or LineOnOffDash, and the GCLineStyle bit is
 * set in valuemask, then a BadValue error occurs.
 */
static void
t018()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.line_style = LineSolid + LineDoubleDash + LineOnOffDash + 1;
	values = &vals;
	d = DRW(display);
	valuemask = GCLineStyle;
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-19.(A)
 * When the cap_style is other than CapNotLast, CapButt,
 * CapRound or CapProjecting, and the GCCapStyle bit is set in
 * valuemask, then a BadValue error occurs.
 */
static void
t019()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.cap_style = CapNotLast + CapButt + CapRound + CapProjecting + 1;
	values = &vals;
	valuemask = GCCapStyle;
	d = DRW(display);
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-20.(A)
 * When the join_style is other than JoinMiter, JoinRound or
 * JoinBevel, and the GCJoinStyle bit is set in valuemask,
 * then a BadValue error occurs.
 */
static void
t020()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.join_style = JoinMiter + JoinRound + JoinBevel + 1;
	values = &vals;
	d = DRW(display);
	valuemask = GCJoinStyle;
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-21.(A)
 * When the fill_style is other than FillSolid, FillTiled,
 * FillStippled or FillOpaqueStippled, and the GCFillStyle bit
 * is set in valuemask, then a BadValue error occurs.
 */
static void
t021()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.fill_style = FillSolid + FillTiled + FillStippled + FillOpaqueStippled + 1;
	values = &vals;
	d = DRW(display);
	valuemask = GCFillStyle;
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-22.(A)
 * When the fill_rule is other than EvenOddRule or WindingRule,
 * and the GCFillRule bit is set in valuemask, then a BadValue
 * error occurs.
 */
static void
t022()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.fill_rule = EvenOddRule + WindingRule + 1;
	values = &vals;
	d = DRW(display);
	valuemask = GCFillRule;
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-23.(A)
 * When arc_mode is other than ArcChord or ArcPieSlice, and
 * the GCArcMode bit is set in valuemask, then a BadValue
 * error occurs.
 */
static void
t023()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.arc_mode = ArcChord + ArcPieSlice + 1;
	values = &vals;
	d = DRW(display);
	valuemask = GCArcMode;
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-24.(A)
 * When the subwindow_mode is other than ClipByChildren or
 * IncludeInferiors, and the GCSubwindowMode bit is set in
 * valuemask, then a BadValue error occurs.
 */
static void
t024()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.subwindow_mode = ClipByChildren + IncludeInferiors + 1;
	values = &vals;
	valuemask = GCSubwindowMode;
	d = DRW(display);
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-25.(A)
 * When graphics_exposure is other than True or False, and
 * the GCGraphicsExposures bit is set in valuemask, then a
 * BadValue error occurs.
 */
static void
t025()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.graphics_exposures = (int) False + (int) True + 1;
	values = &vals;
	valuemask = GCGraphicsExposures;
	d = DRW(display);
	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateGC-26.(A)
 * When dashes is set to zero, and the GCDashList bit is set
 * in valuemask, then a BadValue error occurs.
 */
static void
t026()
{
XGCValues vals;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	vals.dashes = (char) 0;
	values = &vals;
	d = DRW(display);
	valuemask = GCDashList;

	startcall(display);
	if (isdeleted())
		return;
	XCreateGC(display, d, valuemask, values);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if( geterr() == BadValue) 
		CHECK;
	else
		FAIL;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	t018, 18,
	t019, 19,
	t020, 20,
	t021, 21,
	t022, 22,
	t023, 23,
	t024, 24,
	t025, 25,
	t026, 26,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
