/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: frclrmp.m,v 1.8 92/06/11 16:15:53 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XFreeColormap(display, colormap)
 *   Display *display;
 *   Colormap colormap;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XFreeColormap	1
char    *TestName = "XFreeColormap";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XFreeColormap function
 */
static Display *display;
static Colormap colormap;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = DefaultColormap(display, DefaultScreen(display));
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XFreeColormap-1.(A)
 * A call to XFreeColormap removes the association between the
 * colormap argument and the colourmap ID, and frees the
 * associated storage.
 */
static void
t001()
{
XVisualInfo *vi;
unsigned long vmask;
XColor col;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals are supported.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vi); ) {
		trace("Attempting XFreeColormap() for class %s", displayclassname(vi->class));
		colormap = XCreateColormap(display, DRW(display), vi->visual, AllocNone);

		startcall(display);
		if (isdeleted())
			return;
		XFreeColormap(display, colormap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		startcall(display);
		XAllocColor(display, colormap, &col);
		endcall(display);

		if( geterr() == Success) {
			report("XAllocColor() succeeded with a freed colormap");
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeColormap-2.(A)
 * When the colormap argument is the default colourmap, then a
 * call to XFreeColormap does not remove the association
 * between the colormap argument and the colourmap ID or free
 * the associated storage.
 */
static void
t002()
{
XColor col;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	col.red = col.green = col.blue = 0;
	colormap = DefaultColormap(display, DefaultScreen(display));
	startcall(display);
	if (isdeleted())
		return;
	XFreeColormap(display, colormap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if(XAllocColor(display, colormap, &col) == False) {
		report("XAllocColor() failed to allocate a colourcell with the default colormap.");
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeColormap-3.(A)
 * When the colormap argument is an installed colourmap, then
 * a call to XFreeColormap uninstalls the colourmap.
 */
static void
t003()
{
int i, len, notfound;
XVisualInfo *vi;
Colormap *maplist;
unsigned long vmask;
XColor col;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals are supported.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vi); ) {
		trace("Attempting XFreeColormap() for class %s", displayclassname(vi->class));
		colormap = XCreateColormap(display, DRW(display), vi->visual, AllocNone);


		XAllocColor(display, colormap, &col);
		XInstallColormap(display, colormap);
		maplist = XListInstalledColormaps(display, DRW(display), &len);
		for(i=0, notfound = 1; i<len  && notfound; i++ )
			if(maplist[i] == colormap) {
				CHECK;
				trace("Found map at position %d of the required list", i);
				notfound = 0;
			}

		XFree((char*)maplist);
		if(notfound) {
			delete("The installed colourmap was not on the required list.");
			return;
		}

		startcall(display);
		if (isdeleted())
			return;
		XFreeColormap(display, colormap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		maplist = XListInstalledColormaps(display, DRW(display), &len);

		for(i=0, notfound = 1 ; i<len && notfound; i++)
			if(maplist[i] == colormap) {
				report("Colormap is still on the required list.");
				FAIL;
				notfound = 0;
			}

		XFree((char*)maplist);

		if(notfound == 1)
			CHECK;
	}

	CHECKPASS(2*nvinf());

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFreeColormap-4.(A)
 * When the specified colourmap is defined as the colourmap
 * for a window, then a call to XFreeColormap changes the
 * colourmap associated with the window to None and generates a
 * ColormapNotify event.
 */
static void
t004()
{
int i, len, notfound;
XVisualInfo *vi;
XWindowAttributes watts;
XEvent ev;
Colormap *maplist;
unsigned long vmask;
XColor col;
Window win;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if( (vmask = visualsupported(display, 0L)) == 0L) {
		delete("No visuals are supported.");
		return;
	}

	for(resetsupvis(vmask); nextsupvis(&vi); ) {
		trace("Attempting XFreeColormap() for class %s", displayclassname(vi->class));
		colormap = XCreateColormap(display, DRW(display), vi->visual, AllocNone);
		XAllocColor(display, colormap, &col);

		win = makewin(display, vi);
		XSetWindowColormap(display, win, colormap);
		XGetWindowAttributes(display, win, &watts);

		XSelectInput(display, win, ColormapChangeMask);

		
		if(watts.colormap != colormap) {
			delete("XSetWindowColormap() did not set the window colormap.");
			return;
		}

		XInstallColormap(display, colormap);

		startcall(display);
		if (isdeleted())
			return;
		XFreeColormap(display, colormap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		XGetWindowAttributes(display, win, &watts);
		if(watts.colormap != None) {
			report("Colormap of window was not set to None.");
			FAIL;
		}

		if(getevent(display, &ev) == 0) {
			report("No Event was generated");
			FAIL;
		} else
			if(ev.type != ColormapNotify) {
				report("Event generated was not ColormapNotify");
				FAIL;
			} else
				CHECK;

	}

	CHECKPASS(nsupvis());

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/ECol.mc */
/*
 * Assertion XFreeColormap-5.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	XFreeColormap(display, colormap);
	endcall(display);
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
