/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stfnt.m,v 1.9 92/06/29 19:02:57 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   char **
 *   XSetFont(display, gc, font)
 *   Display *display;
 *   GC gc;
 *   Font font;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetFont	1
char    *TestName = "XSetFont";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_GC gc
#define A_FONT font


/*
 * Arguments to the XSetFont function
 */
static Display *display;
static GC gc;
static Font font;


#define FONT1 "xtfont0"
#define FONT2 "xtfont1"

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	gc = 0;
	font = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	gc = ErrdefGC;
	font = ErrdefFont;
}

/*
 * Assertion XSetFont-1.(A)
 * A call to XSetFont sets the font component of the specified
 * GC to the value of the font argument.
 */
static void
t001()
{
XVisualInfo	*vp;
XFontStruct	*fs1, *fs2, *rfs;
XCharStruct	*cs1,*cs2,*maxchar;
XGCValues	values;
GContext	gcctxt;
int		xmin, ymax;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	Win = makewin(display, vp);

	if((fs1 = XLoadQueryFont(display, FONT1)) == (XFontStruct *) 0) {
		delete("Failed to load font %s", FONT1);
		return;
	} else
		CHECK;

	if((fs2 = XLoadQueryFont(display, FONT2)) == (XFontStruct *) 0) {
		delete("Failed to load font %s", FONT2);
		return;
	} else
		CHECK;

	cs1 = &(fs1->max_bounds);
	cs2 = &(fs2->max_bounds);

	values.font = fs1->fid;
	values.foreground = W_FG;
	values.background = W_BG;
	values.function = GXxor;

        /*
         * Create the GC with the window with which it is to be used.
         * This is required because there is a possibility that the target
         * window may not match the default visual depth (causing a BadMatch
         * error if the GC is used with it if created with the root).
         */
	gc = XCreateGC(display, Win, GCFont|GCBackground|GCForeground|GCFunction, &values);

	xmin = (int) ((cs1->lbearing < cs2->lbearing) ? cs1->lbearing : cs2->lbearing);
	ymax = (int) ((cs1->ascent > cs2->ascent) ? cs1->ascent : cs2->ascent);
	XDrawString(display, Win, gc, -xmin, ymax ,"a", 1);

	font = fs2->fid;
	startcall(display);
	if (isdeleted())
		return;
	XSetFont(display, gc, font);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XDrawString(display, Win, gc, -xmin, ymax ,"a", 1);

	if( checkarea(display, Win, (struct area *) 0, W_BG, W_BG, CHECK_ALL|CHECK_DIFFER) == True) {
		report("Font was not changed by XSetFont.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XSetFont-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EFon1.mc */
/*
 * Assertion XSetFont-3.(A)
 * When a font argument does not name a valid font, then a
 * BadFont error occurs.
 */
/* End of included file error/EFon1.mc */

/* Including from file error/EFon1.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_FONT = (Font) badfont(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetFont(display, gc, font);
	endcall(display);
	if (geterr() != BadFont) {
		report("Got %s, Expecting BadFont", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadFont)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EFon1.mc */

/* Including from file error/EGC.mc */
/*
 * Assertion XSetFont-4.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetFont(display, gc, font);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
