/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: ldqryfnt.m,v 1.9 92/06/11 17:07:36 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   XFontStruct *
 *   XLoadQueryFont(display, name)
 *   Display	*display;
 *   char	*name;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XLoadQueryFont	1
char    *TestName = "XLoadQueryFont";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XLoadQueryFont function
 */
static Display	*display;
static char	*name;


extern	int 	checkfsp();
extern	struct	fontinfo fontinfo[];
extern	int 	nfontinfo;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	name = "xtfont0";
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XLoadQueryFont-1.(A)
 * When the name argument names an existing font, then a call
 * to XLoadQueryFont loads the font named name, and returns a
 * pointer to an XFontStruct structure which contains
 * information on the font.
 */
static void
t001()
{
XFontStruct	*fsp;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	for (i = 0; i < nfontinfo; i++) {
		name = fontinfo[i].name;
		trace("Loading %s", name);
		startcall(display);
		if (isdeleted())
			return;
		fsp = XLoadQueryFont(display, name);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (checkfsp(fsp, fontinfo[i].fontstruct, *fontinfo[i].string))
			CHECK;
		else
			FAIL;
	}

	CHECKPASS(nfontinfo);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XLoadQueryFont-2.(A)
 * The font ID returned by a call to XLoadQueryFont is usable
 * on any GC created for any screen of the display.
 */
static void
t002()
{
XVisualInfo	*vp;
Font	font;
XFontStruct	*fsp;
Drawable d;
GC	gc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (resetvinf(VI_WIN_PIX); nextvinf(&vp); ) {
		name = fontinfo[1].name;

		startcall(display);
		if (isdeleted())
			return;
		fsp = XLoadQueryFont(display, name);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		font = fsp->fid;

		if (geterr() != Success) {
			/* Already done after calling XLoadFont */
			report("font %s could not be loaded", name);
			report("Check that xtest fonts have been installed");
			FAIL;
		} else {
			d = makewin(display, vp);
			gc = makegc(display, d);
			XSetFont(display, gc, font);
			XDrawString(display, d, gc, 20, 20, "z", 1);
			PIXCHECK(display, d);
			dclear(display, d);
			XFreeFont(display, fsp);
		}
	}
	CHECKPASS(nvinf());


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XLoadQueryFont-3.(A)
 * When the name argument does not name an existing font, then
 * XLoadQueryFont returns NULL.
 */
static void
t003()
{
XFontStruct	*fsp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	name = config.bad_font_name;
	startcall(display);
	if (isdeleted())
		return;
	fsp = XLoadQueryFont(display, name);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (fsp == NULL)
		PASS;
	else {
		report("NULL was not returned when the name was '%s'", name);
		FAIL;
	}
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XLoadQueryFont-4.(A)
 * Upper and lower case characters in the name argument refer
 * to the same font.
 */
static void
t004()
{
XFontStruct	*fsp1;
XFontStruct	*fsp2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	name = "xtfont0";
	startcall(display);
	if (isdeleted())
		return;
	fsp1 = XLoadQueryFont(display, name);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (fsp1 == NULL) {
		delete("Could not load font %s", name);
		return;
	}

	name = "XtFoNt0";
	startcall(display);
	if (isdeleted())
		return;
	fsp2 = XLoadQueryFont(display, name);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (fsp2 == NULL) {
		report("Font could not be loaded with name %s", name);
		FAIL;
		return;
	}

	/*
	 * The checking of the copyright property is inhibited, so that
	 * this test can pass even if the first one doesn't.
	 */
	if (checkfsp(fsp2, fsp1, (char*)0))
		PASS;
	else {
		report("Different XFontStruct descriptions were returned");
		FAIL;
	}
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XLoadQueryFont-5.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
