/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: qrytxtextn.m,v 1.12 92/06/11 17:09:07 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XQueryTextExtents(display, font_ID, string, nchars, direction_return, font_ascent_return, font_descent_return, overall_return)
 *   Display	*display;
 *   XID 	font_ID;
 *   char	*string;
 *   int 	nchars;
 *   int 	*direction_return;
 *   int 	*font_ascent_return;
 *   int 	*font_descent_return;
 *   XCharStruct	*overall_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XQueryTextExtents	1
char    *TestName = "XQueryTextExtents";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XQueryTextExtents function
 */
static Display	*display;
static XID 	font_ID;
static char	*string;
static int 	nchars;
static int 	*direction_return;
static int 	*font_ascent_return;
static int 	*font_descent_return;
static XCharStruct	*overall_return;


static int 	StatusReturn;

static	int 	direction;
static	int 	font_ascent;
static	int 	font_descent;
static	XCharStruct	overall;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	font_ID = 0;
	string = 0;
	nchars = 0;
	direction_return = &direction;
	font_ascent_return = &font_ascent;
	font_descent_return = &font_descent;
	overall_return = &overall;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XQueryTextExtents-1.(A)
 * When the font_ID argument is a valid GContext resource,
 * then a call to XQueryTextExtents returns the bounding box of
 * the specified 8-bit character string, string, as rendered
 * in the font in the corresponding GC's font field.
 */
static void
t001()
{
extern	struct	fontinfo	fontinfo[];
extern	int 	nfontinfo;
int 	i;
char	buf[256];
int 	good_direction;
int 	good_font_ascent;
int 	good_font_descent;
XCharStruct	good_overall;
Drawable	d;
GC		gc;
Font	font;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (i = 0; i < 256; i++)
		buf[i] = i;
	string = buf;
	nchars  = 256;

	d = defdraw(display, VI_WIN_PIX);
	gc = makegc(display, d);
	if (isdeleted())
		return;

	for (i = 0; i < nfontinfo; i++) {
		font = XLoadFont(display, fontinfo[i].name);
		XSetFont(display, gc, font);

		font_ID = XGContextFromGC(gc);
		startcall(display);
		if (isdeleted())
			return;
		StatusReturn = XQueryTextExtents(display, font_ID, string, nchars, direction_return, font_ascent_return, font_descent_return, overall_return);
		endcall(display);
		if (StatusReturn == 0) {
			report("Status returned was %d", StatusReturn);
			FAIL;
		}
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		txtextents(fontinfo[i].fontstruct, (unsigned char *)string,
			nchars, &good_direction, &good_font_ascent, &good_font_descent,
			&good_overall);

		/*
		 * Don't check this because not well enough defined.
		 * Just check that it is one of the allowed values.
		 */
		if (direction != FontLeftToRight && direction != FontRightToLeft) {
			report("Font %s - Direction was %d", fontinfo[i].name);
			FAIL;
		} else
			CHECK;

		if (good_font_ascent != font_ascent) {
			report("Font %s: font ascent was %d, expecting %d",
				fontinfo[i].name, font_ascent, good_font_ascent);
			FAIL;
		} else
			CHECK;
		if (good_font_descent != font_descent) {
			report("Font %s: font descent was %d, expecting %d",
				fontinfo[i].name, font_descent, good_font_descent);
			FAIL;
		} else
			CHECK;
		if (good_overall.lbearing != overall.lbearing) {
			report("Font %s: lbearing was %d, expecting %d",
				fontinfo[i].name, overall.lbearing, good_overall.lbearing);
			FAIL;
		} else
			CHECK;
		if (good_overall.rbearing != overall.rbearing) {
			report("Font %s: rbearing was %d, expecting %d",
				fontinfo[i].name, overall.rbearing, good_overall.rbearing);
			FAIL;
		} else
			CHECK;
		if (good_overall.ascent != overall.ascent) {
			report("Font %s: ascent was %d, expecting %d",
				fontinfo[i].name, overall.ascent, good_overall.ascent);
			FAIL;
		} else
			CHECK;
		if (good_overall.descent != overall.descent) {
			report("Font %s: descent was %d, expecting %d",
				fontinfo[i].name, overall.descent, good_overall.descent);
			FAIL;
		} else
			CHECK;
		if (good_overall.width != overall.width) {
			report("Font %s: width was %d, expecting %d",
				fontinfo[i].name, overall.width, good_overall.width);
			FAIL;
		} else
			CHECK;
	}
	CHECKPASS(8*nfontinfo);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XQueryTextExtents-2.(A)
 * When the font_ID argument is a valid Font resource, then a
 * call to XQueryTextExtents returns the bounding box of the
 * specified 8-bit character string, string, as rendered in
 * the font with font ID font_ID and returns non-zero.
 */
static void
t002()
{
extern	struct	fontinfo	fontinfo[];
extern	int 	nfontinfo;
int 	i;
char	buf[256];
int 	good_direction;
int 	good_font_ascent;
int 	good_font_descent;
XCharStruct	good_overall;
Font	font;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (i = 0; i < 256; i++)
		buf[i] = i;
	string = buf;
	nchars  = 256;

	for (i = 0; i < nfontinfo; i++) {
		font = XLoadFont(display, fontinfo[i].name);
		if (isdeleted()) {
			delete("Could not load font '%s'", fontinfo[i].name);
			return;
		}

		font_ID = font;
		startcall(display);
		if (isdeleted())
			return;
		StatusReturn = XQueryTextExtents(display, font_ID, string, nchars, direction_return, font_ascent_return, font_descent_return, overall_return);
		endcall(display);
		if (StatusReturn == 0) {
			report("Status returned was %d", StatusReturn);
			FAIL;
		}
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		txtextents(fontinfo[i].fontstruct, (unsigned char *)string,
			nchars, &good_direction, &good_font_ascent, &good_font_descent,
			&good_overall);

		/*
		 * Don't check this because not well enough defined.
		 * Just check that it is one of the allowed values.
		 */
		if (direction != FontLeftToRight && direction != FontRightToLeft) {
			report("Font %s - Direction was %d", fontinfo[i].name);
			FAIL;
		} else
			CHECK;

		if (good_font_ascent != font_ascent) {
			report("Font %s: font ascent was %d, expecting %d",
				fontinfo[i].name, font_ascent, good_font_ascent);
			FAIL;
		} else
			CHECK;
		if (good_font_descent != font_descent) {
			report("Font %s: font descent was %d, expecting %d",
				fontinfo[i].name, font_descent, good_font_descent);
			FAIL;
		} else
			CHECK;
		if (good_overall.lbearing != overall.lbearing) {
			report("Font %s: lbearing was %d, expecting %d",
				fontinfo[i].name, overall.lbearing, good_overall.lbearing);
			FAIL;
		} else
			CHECK;
		if (good_overall.rbearing != overall.rbearing) {
			report("Font %s: rbearing was %d, expecting %d",
				fontinfo[i].name, overall.rbearing, good_overall.rbearing);
			FAIL;
		} else
			CHECK;
		if (good_overall.ascent != overall.ascent) {
			report("Font %s: ascent was %d, expecting %d",
				fontinfo[i].name, overall.ascent, good_overall.ascent);
			FAIL;
		} else
			CHECK;
		if (good_overall.descent != overall.descent) {
			report("Font %s: descent was %d, expecting %d",
				fontinfo[i].name, overall.descent, good_overall.descent);
			FAIL;
		} else
			CHECK;
		if (good_overall.width != overall.width) {
			report("Font %s: width was %d, expecting %d",
				fontinfo[i].name, overall.width, good_overall.width);
			FAIL;
		} else
			CHECK;
	}
	CHECKPASS(8*nfontinfo);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XQueryTextExtents-3.(-)
 * The ascent field of overall is set to the maximum of the
 * ascent metrics of all characters in the string.
 */
static void
t003()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XQueryTextExtents-4.(-)
 * The descent field of overall is set to the maximum of the
 * descent metrics of all characters in the string.
 */
static void
t004()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XQueryTextExtents-5.(-)
 * The width field of overall is set to the sum of the
 * character-width metrics of all characters in the string.
 */
static void
t005()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XQueryTextExtents-6.(-)
 * The lbearing field of overall is set to the minimum L of
 * all characters in the string, where for each character L is
 * the left-side-metric plus the sum of the character widths of
 * all preceding characters in the string.
 */
static void
t006()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XQueryTextExtents-7.(-)
 * The rbearing field of overall is set to the maximum R of
 * all characters in the string, where for each character R is
 * the right-side-bearing metric plus the sum of the character
 * widths of all preceding characters in the string.
 */
static void
t007()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XQueryTextExtents-8.(-)
 * The font_ascent_return argument is set to the logical
 * ascent of the font, the font_descent_return argument is set
 * to the logical descent of the font and the direction_return
 * argument is set to either FontLeftToRight or
 * FontRightToLeft.
 */
static void
t008()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XQueryTextExtents-9.(-)
 * When the font has no defined default character, then
 * undefined characters in the string are taken to have all
 * zero metrics.
 */
static void
t009()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XQueryTextExtents-10.(-)
 * Characters with all zero metrics are ignored.
 */
static void
t010()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XQueryTextExtents-11.(-)
 * When the font has no defined default_char, then the
 * undefined characters in the string are also ignored.
 */
static void
t011()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/* Including from file error/EFon2.mc */
/*
 * Assertion XQueryTextExtents-12.(A)
 * When the font argument does not name a valid GContext or
 * font resource, then a BadFont error occurs.
 */
/* End of included file error/EFon2.mc */

static void
t012()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	string = "jfdkjfk";
	nchars = strlen(string);
	font_ID = badfont(display);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XQueryTextExtents(display, font_ID, string, nchars, direction_return, font_ascent_return, font_descent_return, overall_return);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadFont) {
		report("Got %s, Expecting BadFont", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadFont)
		CHECK;
	else {
		report("No BadFont occurred with a bad font");
		FAIL;
	}

	font_ID = XGContextFromGC(badgc(display));	/* XXX */
	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XQueryTextExtents(display, font_ID, string, nchars, direction_return, font_ascent_return, font_descent_return, overall_return);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadFont) {
		report("Got %s, Expecting BadFont", errorname(geterr()));
		FAIL;
	}
	if (geterr() == BadFont)
		CHECK;
	else {
		report("No BadFont occurred with a bad gc");
		FAIL;
	}
	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = fontstartup;
void	(*tet_cleanup)() = fontcleanup;
