/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stfntpth.m,v 1.10 92/06/11 17:09:23 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XSetFontPath(display, directories, ndirs)
 *   Display *display;
 *   char	**directories;
 *   int 	ndirs;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetFontPath	1
char    *TestName = "XSetFontPath";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XSetFontPath function
 */
static Display *display;
static char	**directories;
static int 	ndirs;



/*
 * Startup and cleanup code attempt to save and restore the Font path
 * over the tests. 
 * This should not be needed now because saving and restoring the Font path
 * is done in startup/cleanup - but it is still done here in case that
 * scheme is ever changed.
 */
static	char	**savepath;
static	int 	savenum;
static void
localstartup()
{
	startup();
	if(Dsp) {
		XSetErrorHandler(error_status);
		savepath = XGetFontPath(Dsp, &savenum);
		XSetErrorHandler(unexp_err);
	}
}
static void
localcleanup()
{
	if(Dsp) {
		if (savepath && savenum)
			XSetFontPath(Dsp, savepath, savenum);
		XSync(Dsp, 0);
	}
	cleanup();
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	directories = 0;
	ndirs = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSetFontPath-1.(A)
 * When XSetFontPath is called with the directories argument
 * specifying a list of directories in an operating system
 * dependent format, then the directory search path for font
 * lookup is set to the list of directories in the order
 * specified.
 */
static void
t001()
{
char	*fpathlist;
char	*fpathtmp;
char	*dirlist[MAX_DIRS];
char	*strtok();
char	**checkpath;
int 	nret;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	fpathlist = tet_getvar("XT_FONTPATH_GOOD");
	if (fpathlist == NULL || *fpathlist == '\0') {
		delete("XT_FONTPATH_GOOD not set in config file");
		return;
	}
	fpathtmp = (char *)calloc(strlen(fpathlist)+1, sizeof(char));
	strcpy(fpathtmp, fpathlist);

	for (i = 0; i < MAX_DIRS; i++) {
		dirlist[i] = strtok((i==0)? fpathtmp: (char*)0, SEP);
		if (dirlist[i] == NULL)
			break;
		debug(1, "dirlist entry %d - '%s'", i, dirlist[i]);
	}
	directories = dirlist;
	ndirs = i;

	if (ndirs <= 0) {
		delete("No components in supplied XT_FONTPATH_GOOD");
		return;
	}

	startcall(display);
	if (isdeleted())
		return;
	XSetFontPath(display, directories, ndirs);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	checkpath = XGetFontPath(display, &nret);
	if (nret == ndirs)
		CHECK;
	else {
		report("Different number of directories returned");
		report("  was %d, expecting %d", nret, ndirs);
		FAIL;
	}

	if (ndirs < nret)
		nret = ndirs;
	for (i = 0; i < nret; i++) {
		debug(1, "got back list item '%s'", checkpath[i]);
		if (strcmp(dirlist[i], checkpath[i]) == 0)
			CHECK;
		else {
			report("Font path component did not match what was set");
			report("  was '%s', expecting '%s'", checkpath[i], dirlist[i]);
			FAIL;
		}
	}

	CHECKPASS(1+ndirs);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetFontPath-2.(A)
 * On a call to XSetFontPath, the directory search path for
 * font lookup is set for all clients.
 */
static void
t002()
{
Display	*client2;
char	*fpathlist;
char	*fpathtmp;
char	*dirlist[MAX_DIRS];
char	*strtok();
char	**checkpath;
int 	nret;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	client2 = XOpenDisplay(DisplayString(display));
	if (client2 == NULL || isdeleted()) {
		delete("Could not open second client");
		return;
	}

	fpathlist = tet_getvar("XT_FONTPATH_GOOD");
	if (fpathlist == NULL || *fpathlist == '\0') {
		delete("XT_FONTPATH_GOOD not set in config file");
		return;
	}
	fpathtmp = (char *)calloc(strlen(fpathlist)+1, sizeof(char));
	strcpy(fpathtmp, fpathlist);

	for (i = 0; i < MAX_DIRS; i++) {
		dirlist[i] = strtok((i==0)? fpathtmp: (char*)0, SEP);
		if (dirlist[i] == NULL)
			break;
		debug(1, "dirlist entry %d - '%s'", i, dirlist[i]);
	}
	directories = dirlist;
	ndirs = i;

	if (ndirs <= 0) {
		delete("No components in supplied XT_FONTPATH_GOOD");
		return;
	}

	startcall(display);
	if (isdeleted())
		return;
	XSetFontPath(display, directories, ndirs);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	checkpath = XGetFontPath(client2, &nret);
	if (nret == ndirs)
		CHECK;
	else {
		report("Different number of directories returned");
		report("  was %d, expecting %d", nret, ndirs);
		FAIL;
	}

	if (ndirs < nret)
		nret = ndirs;
	for (i = 0; i < nret; i++) {
		debug(1, "got back list item '%s'", checkpath[i]);
		if (strcmp(dirlist[i], checkpath[i]) == 0)
			CHECK;
		else {
			report("Font path component did not match what was set");
			report("  was '%s', expecting '%s'", checkpath[i], dirlist[i]);
			FAIL;
		}
	}

	CHECKPASS(1+ndirs);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetFontPath-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * On a call to XSetFontPath, the X server flushes all cached
 * information about fonts for which there are currently no
 * explicit resource ID's allocated.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XSetFontPath-4.(C)
 * When the directories argument is invalid for the operating
 * system, then a BadValue error occurs.
 */
static void
t004()
{
char	*fpathlist;
char	*dirlist[MAX_DIRS];
char	*strtok();
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	fpathlist = tet_getvar("XT_FONTPATH_BAD");
	if (fpathlist == NULL || *fpathlist == '\0') {
		delete("XT_FONTPATH_BAD not set in config file");
		return;
	}
	if (strcmp(fpathlist, "UNSUPPORTED") == 0) {
		report("No bad paths are possible");
		tet_result(TET_UNSUPPORTED);
		return;
	}

	for (i = 0; i < MAX_DIRS; i++) {
		dirlist[i] = strtok((i==0)? fpathlist: (char*)0, SEP);
		if (dirlist[i] == NULL)
			break;
	}
	directories = dirlist;
	ndirs = i;

	if (ndirs <= 0) {
		delete("No components in supplied XT_FONTPATH_BAD");
		return;
	}

	startcall(display);
	if (isdeleted())
		return;
	XSetFontPath(display, directories, ndirs);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = localstartup;
void	(*tet_cleanup)() = localcleanup;
