/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: txtextnts.m,v 1.8 92/06/11 17:09:38 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XTextExtents(font_struct, string, nchars, direction_return, font_ascent_return, font_descent_return, overall_return)
 *   XFontStruct	*font_struct;
 *   char	*string;
 *   int 	nchars;
 *   int 	*direction_return;
 *   int 	*font_ascent_return;
 *   int 	*font_descent_return;
 *   XCharStruct	*overall_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XTextExtents	1
char    *TestName = "XTextExtents";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XTextExtents function
 */
static XFontStruct	*font_struct;
static char	*string;
static int 	nchars;
static int 	*direction_return;
static int 	*font_ascent_return;
static int 	*font_descent_return;
static XCharStruct	*overall_return;


static	int 	direction;
static	int 	font_ascent;
static	int 	font_descent;
static	XCharStruct	overall;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	font_struct = 0;
	string = 0;
	nchars = 0;
	direction_return = &direction;
	font_ascent_return = &font_ascent;
	font_descent_return = &font_descent;
	overall_return = &overall;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XTextExtents-1.(A)
 * A call to XTextExtents returns the bounding box of the
 * specified 8-bit character string, string, as rendered in
 * the font referenced by font_struct.
 */
static void
t001()
{
extern	struct	fontinfo	fontinfo[];
extern	int 	nfontinfo;
int 	i;
char	buf[256];
int 	good_direction;
int 	good_font_ascent;
int 	good_font_descent;
XCharStruct	good_overall;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for (i = 0; i < 256; i++)
		buf[i] = i;
	string = buf;
	nchars  = 256;

	for (i = 0; i < nfontinfo; i++) {
		font_struct = fontinfo[i].fontstruct;

		startcall(Dsp);
		if (isdeleted())
			return;
		XTextExtents(font_struct, string, nchars, direction_return, font_ascent_return, font_descent_return, overall_return);
		endcall(Dsp);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		txtextents(font_struct, (unsigned char *)string, nchars,
			&good_direction, &good_font_ascent, &good_font_descent,
			&good_overall);

		/*
		 * Don't check this because not well enough defined.
		 * Just check that it is one of the allowed values.
		 */
		if (direction != FontLeftToRight && direction != FontRightToLeft) {
			report("Font %s - Direction was %d", fontinfo[i].name);
			FAIL;
		} else
			CHECK;

		if (good_font_ascent != font_ascent) {
			report("Font %s: font ascent was %d, expecting %d",
				fontinfo[i].name, font_ascent, good_font_ascent);
			FAIL;
		} else
			CHECK;
		if (good_font_descent != font_descent) {
			report("Font %s: font descent was %d, expecting %d",
				fontinfo[i].name, font_descent, good_font_descent);
			FAIL;
		} else
			CHECK;
		if (good_overall.lbearing != overall.lbearing) {
			report("Font %s: lbearing was %d, expecting %d",
				fontinfo[i].name, overall.lbearing, good_overall.lbearing);
			FAIL;
		} else
			CHECK;
		if (good_overall.rbearing != overall.rbearing) {
			report("Font %s: rbearing was %d, expecting %d",
				fontinfo[i].name, overall.rbearing, good_overall.rbearing);
			FAIL;
		} else
			CHECK;
		if (good_overall.ascent != overall.ascent) {
			report("Font %s: ascent was %d, expecting %d",
				fontinfo[i].name, overall.ascent, good_overall.ascent);
			FAIL;
		} else
			CHECK;
		if (good_overall.descent != overall.descent) {
			report("Font %s: descent was %d, expecting %d",
				fontinfo[i].name, overall.descent, good_overall.descent);
			FAIL;
		} else
			CHECK;
		if (good_overall.width != overall.width) {
			report("Font %s: width was %d, expecting %d",
				fontinfo[i].name, overall.width, good_overall.width);
			FAIL;
		} else
			CHECK;
	}
	CHECKPASS(8*nfontinfo);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XTextExtents-2.(-)
 * The ascent field of overall is set to the maximum of the
 * ascent metrics of all characters in the string.
 */
static void
t002()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XTextExtents-3.(-)
 * The descent field of overall is set to the maximum of the
 * descent metrics of all characters in the string.
 */
static void
t003()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XTextExtents-4.(-)
 * The width field of overall is set to the sum of the
 * character-width metrics of all characters in the string.
 */
static void
t004()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XTextExtents-5.(-)
 * The lbearing field of overall is set to the minimum L of
 * all characters in the string, where for each character L is
 * the left-side-metric plus the sum of the character widths of
 * all preceding characters in the string.
 */
static void
t005()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XTextExtents-6.(-)
 * The rbearing field of overall is set to the maximum R of
 * all characters in the string, where for each character R is
 * the right-side-bearing metric plus the sum of the character
 * widths of all preceding characters in the string.
 */
static void
t006()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XTextExtents-7.(-)
 * The font_ascent_return argument is set to the logical
 * ascent of the font, the font_descent_return argument is set
 * to the logical descent of the font and the direction_return
 * argument is set to either FontLeftToRight or
 * FontRightToLeft.
 */
static void
t007()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XTextExtents-8.(-)
 * When the font has no defined default character, then
 * undefined characters in the string are taken to have all
 * zero metrics.
 */
static void
t008()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XTextExtents-9.(-)
 * Characters with all zero metrics are ignored.
 */
static void
t009()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/*
 * Assertion XTextExtents-10.(-)
 * When the font has no defined default_char, then the
 * undefined characters in the string are also ignored.
 */
static void
t010()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
