/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: allwevnts.m,v 1.7 92/06/11 17:11:03 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XAllowEvents(display, event_mode, test_time)
 *   Display	*display;
 *   int 	event_mode;
 *   Time	test_time;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XAllowEvents	1
char    *TestName = "XAllowEvents";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XAllowEvents function
 */
static Display	*display;
static int 	event_mode;
static Time	test_time;



/*
 * The focus startup routines set the focus to the root window.
 */

/*
 * A window for use as a grab window in the freeze and freezecheck
 * routines.
 */
static	Window	grabwin;

/*
 * Grab and freeze the pointer.
 */
static void
grabfreezepointer(disp, time)
Display	*disp;
Time	time;
{

	grabwin = defwin(disp);

	XGrabPointer(disp, grabwin, False, PointerMotionMask,
		GrabModeSync, GrabModeAsync, None, None, time);
}

/*
 * Return True if the pointer is frozen.  We move the pointer and check
 * the reported new position as returned by warppointer.  If this is the
 * same as the old position then the pointer is frozen.
 */
static	Bool
ispfrozen(disp)
Display	*disp;
{
PointerPlace	*pp;

	(void) warppointer(disp, grabwin, 0, 0);
	pp = warppointer(disp, grabwin, 1, 1);
	if (pp->ox == pp->nx)
		return True;
	else
		return False;
}



/*
 * Returns True if the keyboard is frozen.
 */
static
iskfrozen(display)
Display	*display;
{
XEvent	ev;
Window	win;
int 	res;
int 	key;

	XSync(display, True); /* Flush previous events */
	key = getkeycode(display);

	/*
	 * Try to provoke a keypress on win.
	 */
	win = defwin(display);
	XSelectInput(display, win, KeyPressMask);
	(void) warppointer(display, win, 1, 1);
	keypress(display, key);
	if (XCheckMaskEvent(display, (long)KeyPressMask, &ev))
		res = False;
	else
		res = True;

	return(res);
}


/*
 * Set up for SyncBoth tests grab and freeze both pointer and keyboard.
 */
bothset()
{
	XUngrabPointer(display, CurrentTime);
	XUngrabKeyboard(display, CurrentTime);

	grabwin = defwin(display);
	XSelectInput(display, grabwin, ButtonPressMask|ButtonReleaseMask|KeyPressMask|KeyReleaseMask);
	(void) warppointer(display, grabwin, 5, 5);

	XGrabPointer(display, grabwin, False,
	 	ButtonPressMask|ButtonReleaseMask|PointerMotionMask,
		GrabModeSync, GrabModeAsync, None, None, CurrentTime);
	XGrabKeyboard(display, grabwin, False, GrabModeAsync, GrabModeSync,
		CurrentTime);


	if (!ispfrozen(display)) {
		delete("Could not freeze pointer");
		return;
	}
	/*
	 * Can't check for the keyboard being frozen here since that requires
	 * pressing a key - and that would release the grab.
	 */
}



/* Value list for use in test t029 */
static int 	event_modevallist[] = {
	AsyncPointer,
	SyncPointer,
	AsyncKeyboard,
	SyncKeyboard,
	ReplayPointer,
	ReplayKeyboard,
	AsyncBoth,
	SyncBoth,
};


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	event_mode = AsyncPointer;
	test_time = CurrentTime;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XAllowEvents-1.(A)
 * When the specified time is earlier than the last-grab time
 * of the most recent active grab for the client or later than
 * the current X server time, then a call to XAllowEvents has
 * no effect.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/* get time from the server */
	test_time = gettime(display);
	grabfreezepointer(display, test_time);

	test_time -= 100;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display))
		CHECK;
	else {
		report("Events allowed when time was earlier than last-grab time");
		FAIL;
	}

	/*
	 * Get current time again and add several minutes to get a time in the
	 * future.
	 */
	test_time = gettime(display);
	test_time += ((config.speedfactor+1) * 1000000);
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display))
		CHECK;
	else {
		report("Events allowed when time was later than current server time");
		FAIL;
	}

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-2.(A)
 * When the event_mode argument is AsyncPointer and the
 * pointer is frozen by the client, then pointer event
 * processing is resumed.
 */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	grabfreezepointer(display, test_time);

	event_mode = AsyncPointer;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display)) {
		report("Pointer was not released after AsyncPointer");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-3.(A)
 * When the event_mode argument is AsyncPointer and the
 * pointer is frozen twice by the client on behalf of two
 * separate grabs, then a call to XAllowEvents thaws for both
 * grabs.
 */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	grabfreezepointer(display, test_time);
	XGrabKeyboard(display, grabwin, False, GrabModeSync, GrabModeAsync,
		CurrentTime);

	if (isdeleted())
		return;

	event_mode = AsyncPointer;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display)) {
		report("Pointer was not released from double grab after AsyncPointer");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-4.(A)
 * When the event_mode argument is AsyncPointer and the
 * pointer is not frozen by the client, then a call to
 * XAllowEvents has no effect.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	event_mode = AsyncPointer;
	grabwin = defwin(display);

	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (!ispfrozen(display))
		PASS;
	else {
		report("Pointer was frozen after AsyncPointer");
		FAIL;
	}
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-5.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is SyncPointer and the pointer
 * is frozen and actively grabbed by the client, then pointer
 * event processing is resumed normally until the next
 * ButtonPress or ButtonRelease event is reported to the
 * client, at which time the pointer again appears to freeze,
 * unless the reported event causes the pointer grab to be
 * released.
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	grabfreezepointer(display, test_time);
	XChangeActivePointerGrab(display, ButtonPressMask | PointerMotionMask,
				 None, CurrentTime);
	if (ispfrozen(display))
		CHECK;
	else {
		delete("Could not freeze pointer");
		return;
	}

	event_mode = SyncPointer;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display)) {
		report("Pointer was not released after SyncPointer");
		FAIL;
	} else
		CHECK;

	if (noext(1) || nbuttons() <= 1) {
		CHECKUNTESTED(2);
		return;
	}

	/* If extension we can go on */
	(void)warppointer(display, grabwin, 1, 2);
	buttonpress(display, Button1);

	if (ispfrozen(display))
		CHECK;
	else {
		report("Pointer was not re-frozen by a button press");
		FAIL;
	}

	CHECKPASS(3);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-6.(A)
 * When the event_mode argument is SyncPointer and the pointer
 * is not frozen by the client or the pointer is not grabbed by
 * the client, then a call to XAllowEvents has no effect.
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	event_mode = SyncPointer;
	grabwin = defwin(display);

	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (!ispfrozen(display))
		PASS;
	else {
		report("Pointer was frozen after SyncPointer with no initial freeze");
		FAIL;
	}
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-7.(A)
 * When the event_mode argument is SyncPointer and the pointer
 * is frozen twice by the client on behalf of two separate
 * grabs, then a call to XAllowEvents thaws for both grabs.
 */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	grabfreezepointer(display, test_time);
	XGrabKeyboard(display, grabwin, False, GrabModeSync, GrabModeAsync,
		CurrentTime);

	if (isdeleted())
		return;

	event_mode = SyncPointer;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display)) {
		report("Pointer was not released after SyncPointer");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-8.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is ReplayPointer and the
 * pointer is actively grabbed by the client and is frozen as
 * the result of an event having been sent to the client, then
 * the pointer grab is released and the event is completely
 * reprocessed as though any passive grabs at or above the grab
 * window of the grab just released were not present.
 */
static void
t008()
{
XEvent  ev;
XButtonPressedEvent    *bpp;
Window  chwin;
struct  area    area;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	event_mode = ReplayPointer;
	if (noext(1)) {
		startcall(display);
		if (isdeleted())
			return;
		XAllowEvents(display, event_mode, test_time);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		untested("There is no reliable test method, but a touch test was performed");
		return;
	} else
		CHECK;

	/*
	 * Set up a pointer freeze as a result of a button press.
	 */
	grabwin = defwin(display);
	setarea(&area, 50, 50, 5, 5);
	chwin = crechild(display, grabwin, &area);

	/* XSelectInput(display, grabwin, ButtonPress); */
	XGrabButton(display, Button1, 0, grabwin,
		False, PointerMotionMask, GrabModeSync, GrabModeAsync,
		None, None);
	XGrabButton(display, Button1, 0, chwin,
		False, PointerMotionMask, GrabModeSync, GrabModeAsync,
		None, None);

	/*
	 * Activate the grab.
	 */
	XSync(display, True);	/* Discard any events */
	(void) warppointer(display, chwin, 1, 1);
	buttonpress(display, Button1);

	/*
	 * Check that the grab was activated and that it occurs on the parent
	 * window.
	 */
	if (XCheckMaskEvent(display, ButtonPressMask, &ev)) {
		bpp = (XButtonPressedEvent*)&ev;
		if (bpp->window == grabwin)
			CHECK;
		else if (bpp->window == chwin) {
			delete("Child window had the grab");
			return;
		} else {
			delete("Could not get grab on parent window");
			return;
		}
	} else {
		delete("Did not get a button event when trying to activate grab");
		return;
	}

	/* Do the ReplayPointer */
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/*
	 * The effect should be as if the button were pressed again
	 * but without the passive grab on the parent window.  So this
	 * time the child should pick up the grab.
	 */
	if (XCheckMaskEvent(display, ButtonPressMask, &ev)) {
		bpp = (XButtonPressedEvent*)&ev;
		if (bpp->window == chwin)
			CHECK;
		else if (bpp->window == grabwin) {
			report("Parent window had the grab after a ReplayPointer");
			FAIL;
		} else {
			report("After ReplayPointer the grab on the child did not activate");
			FAIL;
		}
	} else {
		report("Did not get a button event when trying to activate grab");
		FAIL;
	}

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-9.(A)
 * When the event_mode argument is ReplayPointer and the
 * pointer is not frozen as a result of an event or the pointer
 * is not grabbed by the client, then a call to XAllowEvents
 * has no effect.
 */
static void
t009()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	event_mode = ReplayPointer;
	grabwin = defwin(display);

	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (!ispfrozen(display))
		PASS;
	else {
		report("Pointer was frozen after ReplayPointer");
		FAIL;
	}
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-10.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is ReplayPointer and the
 * pointer is frozen twice by the client on behalf of two
 * separate grabs, then a call to XAllowEvents thaws for both
 * grabs.
 */
static void
t010()
{
int 	key;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(1))
		return;

	grabwin = defwin(display);
	(void) warppointer(display, grabwin, 1, 1);

	key = getkeycode(display);
	XGrabKey(display, key, 0, grabwin, False, GrabModeSync, GrabModeAsync);
	XGrabButton(display, Button1, 0, grabwin,
		False, PointerMotionMask, GrabModeSync, GrabModeAsync,
		None, None);
	buttonpress(display, Button1);
	keypress(display, key);

	event_mode = ReplayPointer;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display)) {
		report("Pointer was not released after ReplayPointer");
		report("  and the pointer was frozen by two grabs.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-11.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is AsyncKeyboard and the
 * keyboard is frozen by the client, then keyboard event
 * processing is resumed.
 */
static void
t011()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0)) {
		event_mode = AsyncKeyboard;
		startcall(display);
		if (isdeleted())
			return;
		XAllowEvents(display, event_mode, test_time);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		untested("There is no reliable test method, but a touch test was performed");
		return;
	} else
		CHECK;

	grabwin = defwin(display);
	XGrabKeyboard(display, grabwin, False, GrabModeAsync, GrabModeSync,
		CurrentTime);
	if (iskfrozen(display))
		CHECK;
	else {
		delete("Could not freeze keyboard");
		return;
	}

	event_mode = AsyncKeyboard;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (iskfrozen(display)) {
		report("Keyboard was not released by AsyncKeyboard");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-12.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is AsyncKeyboard and the
 * keyboard is frozen twice by the client on behalf of two
 * separate grabs, then a call to XAllowEvents thaws for both.
 */
static void
t012()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	grabwin = defwin(display);
	XGrabPointer(display, grabwin, False, PointerMotionMask,
		GrabModeAsync, GrabModeSync, None, None, test_time);
	XGrabKeyboard(display, grabwin, False, GrabModeAsync, GrabModeSync,
		CurrentTime);

	if (iskfrozen(display))
		CHECK;
	else {
		delete("Could not not set up keyboard grab");
		return;
	}

	event_mode = AsyncKeyboard;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (iskfrozen(display)) {
		report("Keyboard was not released by AsyncKeyboard");
		report("  when it was frozen on behalf of two separate grabs.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-13.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is AsyncKeyboard and the
 * keyboard is not frozen by the client, then a call to
 * XAllowEvents has no effect.
 */
static void
t013()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	event_mode = AsyncKeyboard;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (iskfrozen(display)) {
		report("Keyboard was frozen by AsyncKeyboard");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-14.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is SyncKeyboard and the
 * keyboard is frozen and actively grabbed by the client, then
 * keyboard event processing is resumed normally until the next
 * KeyPress or KeyRelease event is reported to the client, at
 * which time the keyboard again appears to freeze unless the
 * reported event causes the keyboard grab to be released.
 */
static void
t014()
{
int 	key;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	event_mode = SyncKeyboard;
	if (noext(0)) {
		startcall(display);
		if (isdeleted())
			return;
		XAllowEvents(display, event_mode, test_time);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		untested("There is no reliable test method, but a touch test was performed");
		return;
	} else
		CHECK;

	grabwin = defwin(display);
	XSelectInput(display, grabwin, KeyPressMask);
	XGrabKeyboard(display, grabwin, False, GrabModeAsync, GrabModeSync,
		CurrentTime);

	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (iskfrozen(display)) {
		report("Keyboard remained frozen after a SyncKeyboard");
		FAIL;
	} else
		CHECK;

	key = getkeycode(display);
	keypress(display, key);

	if (iskfrozen(display))
		CHECK;
	else {
		report("Keyboard was not frozen by a keypress after a SyncKeyboard");
		FAIL;
	}

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-15.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is SyncKeyboard and the
 * keyboard is not frozen by the client or the keyboard is not
 * grabbed by the client, then a call to XAllowEvents has no
 * effect.
 */
static void
t015()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	event_mode = SyncKeyboard;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (iskfrozen(display)) {
		report("Keyboard was frozen by SyncKeyboard though there was no freeze in effect");
		FAIL;
	} else
		CHECK;

	keypress(display, getkeycode(display));
	if (iskfrozen(display)) {
		report("Keyboard was frozen by SyncKeyboard and a key press");
		report("  though there was no freeze in effect");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-16.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is SyncKeyboard and the
 * keyboard is frozen twice by the client on behalf of two
 * separate grabs, then a call to XAllowEvents thaws for both
 * grabs.
 */
static void
t016()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	grabwin = defwin(display);
	XGrabPointer(display, grabwin, False, PointerMotionMask,
		GrabModeAsync, GrabModeSync, None, None, test_time);
	XGrabKeyboard(display, grabwin, False, GrabModeAsync, GrabModeSync,
		CurrentTime);

	event_mode = SyncKeyboard;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (iskfrozen(display)) {
		report("Keyboard remained frozen after a SyncKeyboard with");
		report("  the keyboard frozen on behalf of two separate grabs");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-17.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is ReplayKeyboard and the
 * keyboard is actively grabbed by the client and is frozen as
 * the result of an event having been sent to the client, then
 * the keyboard grab is released and the event is completely
 * reprocessed as though any passive grabs at or above the grab
 * window of the grab just released were not present.
 */
static void
t017()
{
XEvent	ev;
XKeyPressedEvent	*kpp;
Window	win;
Window	chwin;
struct	area	area;
int 	key;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	event_mode = ReplayKeyboard;

	if (noext(0)) {
		startcall(display);
		if (isdeleted())
			return;
		XAllowEvents(display, event_mode, test_time);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		untested("There is no reliable test method, but a touch test was performed");
		return;
	} else
		CHECK;

	win = defwin(display);
	setarea(&area, 50, 50, 5, 5);
	chwin = crechild(display, win, &area);
	XSelectInput(display, win, KeyPressMask);
	XSelectInput(display, chwin, KeyPressMask);

	key = getkeycode(display);
	XGrabKey(display, key, 0, win, False, GrabModeAsync, GrabModeSync);
	XGrabKey(display, key, 0, chwin, False, GrabModeAsync, GrabModeSync);

	(void) warppointer(display, chwin, 1, 1);

	XSync(display, True);
	/* This should activate the grab */
	keypress(display, key);

	if (XCheckMaskEvent(display, KeyPressMask, &ev)) {
		kpp = (XKeyPressedEvent*)&ev;
		if (kpp->window == win)
			CHECK;
		else if (kpp->window == chwin) {
			/*
			 * This could also be because owner_events=False is being
			 * ignored.
			 */
			delete("Child window had the grab, rather than parent");
			return;
		} else {
			delete("Parent did not get the grab");
			return;
		}
	} else {
		delete("Did not get KeyPress event");
		return;
	}
	XSync(display, True);	/* Discard any other events */

	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/*
	 * When the event is replayed then we should now get the event on
	 * the child window.
	 */
	if (XCheckMaskEvent(display, KeyPressMask, &ev)) {
		kpp = (XKeyPressedEvent*)&ev;
		if (kpp->window == chwin)
			CHECK;
		else if (kpp->window == win) {
			report("Parent window had the grab after ReplayPointer");
			report("  expecting that child window would get grab");
			FAIL;
		} else {
			report("Child did not get the grab after ReplayPointer");
			FAIL;
		}
	} else {
		report("Did not get KeyPress event after ReplayPointer");
		FAIL;
	}

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-18.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is ReplayKeyboard and the
 * keyboard is not frozen as a result of an event or the
 * keyboard is not grabbed by the client, then a call to
 * XAllowEvents has no effect.
 */
static void
t018()
{
XEvent	ev;
XKeyPressedEvent	*kpp;
Window	win;
Window	chwin;
struct	area	area;
int 	key;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	event_mode = ReplayKeyboard;

	if (noext(0)) {
		startcall(display);
		if (isdeleted())
			return;
		XAllowEvents(display, event_mode, test_time);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		untested("There is no reliable test method, but a touch test was performed");
		return;
	} else
		CHECK;

	win = defwin(display);
	setarea(&area, 50, 50, 5, 5);
	chwin = crechild(display, win, &area);

	key = getkeycode(display);
	XGrabKey(display, key, 0, win, False, GrabModeAsync, GrabModeAsync);
	XGrabKey(display, key, 0, chwin, False, GrabModeAsync, GrabModeAsync);

	(void) warppointer(display, chwin, 1, 1);

	XSync(display, True);
	/* This should activate the grab */
	keypress(display, key);

	if (XCheckMaskEvent(display, KeyPressMask, &ev)) {
		kpp = (XKeyPressedEvent*)&ev;
		if (kpp->window == win)
			CHECK;
		else if (kpp->window == chwin) {
			delete("Child window had the grab");
			return;
		} else {
			delete("Parent did not get the grab");
			return;
		}
	} else {
		delete("Did not get KeyPress event");
		return;
	}
	XSync(display, True);	/* Discard any other events */

	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/*
	 * Should be no event after the ReplayKeyboard.
	 */
	if (XCheckMaskEvent(display, KeyPressMask, &ev)) {
		report("ReplayKeyboard had an effect when the keyboard was not frozen.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-19.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is ReplayKeyboard and the
 * keyboard is frozen twice by the client on behalf of two
 * separate grabs, then a call to XAllowEvents thaws for both
 * grabs.
 */
static void
t019()
{
int	key;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(1))
		return;

	grabwin = defwin(display);
	(void) warppointer(display, grabwin, 1, 1);

	XGrabButton(display, Button1, 0, grabwin,
		False, PointerMotionMask, GrabModeSync, GrabModeAsync,
		None,  None);

	key = getkeycode(display);
	XGrabKey(display, key, 0, grabwin, False, GrabModeSync, GrabModeAsync);

	keypress(display, key);
	buttonpress(display, Button1);

	event_mode = ReplayKeyboard;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (iskfrozen(display)) {
		report("Keyboard was not unfrozen by ReplayKeyboard when frozen");
		report("  on behalf of two separate grabs.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(1);

	restoredevstate();

	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-20.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is SyncBoth and both pointer
 * and keyboard are frozen by the client, then event processing
 * for both devices continues normally until the next
 * ButtonPress, ButtonRelease, KeyPress, or KeyRelease event
 * is reported to the client for a grabbed device at which time
 * both devices again appear to freeze, unless the reported
 * event causes the grab to be released.
 */
static void
t020()
{
int 	key;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	event_mode = SyncBoth;

	if (noext(1)) {
		startcall(display);
		if (isdeleted())
			return;
		XAllowEvents(display, event_mode, test_time);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		untested("There is no reliable test method, but a touch test was performed");
		return;
	} else
		CHECK;


	bothset();
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display)) {
		report("SyncBoth did not release pointer and keyboard");
		FAIL;
	} else
		CHECK;

	/* 1. Button press */
	buttonpress(display, Button1);
	if (ispfrozen(display))
		CHECK;
	else {
		report("Pointer was not re-frozen by a button press after SyncBoth");
		FAIL;
	}
	if (iskfrozen(display))
		CHECK;
	else {
		report("Keyboard was not re-frozen by a button press after SyncBoth");
		FAIL;
	}

	/* Allow events again for next part */
	bothset();
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (ispfrozen(display)) {
		report("SyncBoth did not release pointer and keyboard");
		FAIL;
	} else
		CHECK;

	/* 2. Button release */
	buttonrel(display, Button1);
	if (ispfrozen(display))
		CHECK;
	else {
		report("Pointer was not re-frozen by a button release after SyncBoth");
		FAIL;
	}
	if (iskfrozen(display))
		CHECK;
	else {
		report("Keyboard was not re-frozen by a button release after SyncBoth");
		FAIL;
	}

	/* Allow events again for next part */
	bothset();
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (ispfrozen(display)) {
		report("SyncBoth did not release pointer and keyboard");
		FAIL;
	} else
		CHECK;

	/* 3. Press key. */
	key = getkeycode(display);
	keypress(display, key);
	if (ispfrozen(display))
		CHECK;
	else {
		report("Pointer was not re-frozen by a key press after SyncBoth");
		FAIL;
	}
	if (iskfrozen(display))
		CHECK;
	else {
		report("Keyboard was not re-frozen by a key press after SyncBoth");
		FAIL;
	}

	/* Allow events again for next part */
	bothset();
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (ispfrozen(display)) {
		report("SyncBoth did not release pointer and keyboard");
		FAIL;
	} else
		CHECK;

	/* 4. Key release. */
	keyrel(display, key);
	if (ispfrozen(display))
		CHECK;
	else {
		report("Pointer was not re-frozen by a key release after SyncBoth");
		FAIL;
	}
	if (iskfrozen(display))
		CHECK;
	else {
		report("Keyboard was not re-frozen by a key release after SyncBoth");
		FAIL;
	}

	CHECKPASS(13);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-21.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is SyncBoth and an event has
 * caused the grab on one device to be released and a
 * subsequent event is reported for the other device that does
 * not cause the grab to be released, then both devices are
 * again frozen.
 */
static void
t021()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(1))
		return;

	grabwin = defwin(display);
	XGrabKeyboard(display, grabwin, False, GrabModeAsync, GrabModeSync,
		CurrentTime);
	XGrabButton(display, Button1, 0, grabwin,
		False, PointerMotionMask, GrabModeSync, GrabModeAsync,
		None, None);

	(void) warppointer(display, grabwin, 1, 1);
	buttonpress(display, Button1);

	if (ispfrozen(display))
		CHECK;
	else {
		delete("Could not freeze pointer and keyboard");
		return;
	}

	event_mode = SyncBoth;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	/*
	 * Release pointer grab.
	 */
	buttonrel(display, Button1);
	if (ispfrozen(display)) {
		report("Pointer remained frozen after releasing button");
		FAIL;
	} else
		CHECK;

	keypress(display, getkeycode(display));
	if (ispfrozen(display))
		CHECK;
	else {
		report("Pointer was not re-frozen by an event from the keyboard after the pointer grab was released.");
		FAIL;
	}
	if (iskfrozen(display))
		CHECK;
	else {
		report("Keyboard was not re-frozen by an event from the keyboard after the pointer grab was released");
		FAIL;
	}

	CHECKPASS(4);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-22.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is SyncBoth and the pointer or
 * keyboard is frozen twice by the client on behalf of two
 * separate grabs, then a call to XAllowEvents thaws for both
 * grabs.
 */
static void
t022()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	grabwin = defwin(display);
	XGrabPointer(display, grabwin, False, PointerMotionMask,
		GrabModeSync, GrabModeSync, None, None, test_time);
	XGrabKeyboard(display, grabwin, False, GrabModeSync, GrabModeSync,
		CurrentTime);

	event_mode = SyncBoth;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display)) {
		report("Pointer was not thawed by SyncBoth when pointer was frozen");
		report("  on behalf of two grabs");
		FAIL;
	} else
		CHECK;
	if (iskfrozen(display)) {
		report("Keyboard was not thawed by SyncBoth when pointer was frozen");
		report("  on behalf of two grabs");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-23.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is SyncBoth and either the
 * keyboard or pointer is not frozen by the client or neither
 * are grabbed by the client, then a call to XAllowEvents has
 * no effect.
 */
static void
t023()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	grabwin = defwin(display);

	event_mode = SyncBoth;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	buttonpress(display, Button1);
	if (ispfrozen(display)) {
		report("Pointer was frozen by button press after SyncBoth");
		report("  even though there were no grabs active");
		FAIL;
	} else
		CHECK;
	if (iskfrozen(display)) {
		report("Keyboard was frozen by button press after SyncBoth");
		report("  even though there were no grabs active");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-24.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is AsyncBoth and the pointer
 * and the keyboard are frozen by the client, then event
 * processing for both devices is resumed normally.
 */
static void
t024()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	event_mode = AsyncBoth;

	if (noext(0)) {
		startcall(display);
		if (isdeleted())
			return;
		XAllowEvents(display, event_mode, test_time);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		untested("There is no reliable test method, but a touch test was performed");
		return;
	} else
		CHECK;

	bothset();
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display)) {
		report("Pointer remained frozen after AsyncBoth");
		FAIL;
	} else
		CHECK;
	if (iskfrozen(display)) {
		report("Keyboard remained frozen after AsyncBoth");
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-25.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is AsyncBoth and either the
 * keyboard or the pointer is not frozen by the client, then a
 * call to XAllowEvents has no effect.
 */
static void
t025()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	grabfreezepointer(display, CurrentTime);

	event_mode = AsyncBoth;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display))
		CHECK;
	else {
		report("Pointer was released by AsyncBoth, although keyboard was not frozen");
		FAIL;
	}
	CHECKPASS(1);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-26.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is AsyncBoth and the pointer
 * or keyboard is frozen twice by the client on behalf of two
 * separate grabs, then a call to XAllowEvents thaws for both
 * grabs.
 */
static void
t026()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	grabwin = defwin(display);
	XGrabPointer(display, grabwin, False, PointerMotionMask,
		GrabModeSync, GrabModeSync, None, None, test_time);
	XGrabKeyboard(display, grabwin, False, GrabModeSync, GrabModeSync,
		CurrentTime);

	event_mode = AsyncBoth;
	startcall(display);
	if (isdeleted())
		return;
	XAllowEvents(display, event_mode, test_time);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (ispfrozen(display)) {
		report("Pointer remained frozen after AsyncBoth");
		report("  when it was frozen twice");
		FAIL;
	} else
		CHECK;
	if (iskfrozen(display)) {
		report("Keyboard remained frozen after AsyncBoth");
		report("  when it was frozen twice");
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-27.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When the event_mode argument is AsyncPointer, SyncPointer,
 * or ReplayPointer, then a call to XAllowEvents has no effect
 * on the processing of keyboard events.
 */
static void
t027()
{
static	int 	modes[] = {
	AsyncPointer, SyncPointer, ReplayPointer};
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0))
		return;

	grabwin = defwin(display);
	XGrabKeyboard(display, grabwin, False, GrabModeAsync, GrabModeSync,
		CurrentTime);

	for (i = 0; i < NELEM(modes); i++) {
		event_mode = modes[i];
		startcall(display);
		if (isdeleted())
			return;
		XAllowEvents(display, event_mode, test_time);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (iskfrozen(display))
			CHECK;
		else {
			report("Keyboard was released when event_mode was %s",
				alloweventmodename(modes[i]));
			FAIL;
		}
	}

	CHECKPASS(NELEM(modes));
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-28.(A)
 * When the event_mode argument is AsyncKeyboard,
 * SyncKeyboard, or ReplayKeyboard, then a call to
 * XAllowEvents has no effect on the processing of pointer
 * events.
 */
static void
t028()
{
static	int 	modes[] = {
	AsyncKeyboard, SyncKeyboard, ReplayKeyboard};
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	grabfreezepointer(display, test_time);

	for (i = 0; i < NELEM(modes); i++) {
		event_mode = modes[i];
		startcall(display);
		if (isdeleted())
			return;
		XAllowEvents(display, event_mode, test_time);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		if (ispfrozen(display))
			CHECK;
		else {
			report("Pointer was released when event_mode was %s",
				alloweventmodename(modes[i]));
			FAIL;
		}
	}

	CHECKPASS(NELEM(modes));
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XAllowEvents-29.(A)
 * When the value of event_mode is other than AsyncPointer,
 * SyncPointer, AsyncKeyboard, SyncKeyboard, ReplayPointer,
 * ReplayKeyboard, AsyncBoth or SyncBoth, then a BadValue
 * error occurs.
 */
/* Including from file error/EVal.mc */
static void
t029()
{
#undef	VALUE_ARG
#define	VALUE_ARG event_mode
#undef	VALUE_LIST
#define	VALUE_LIST event_modevallist
#undef NOTMEMTYPE
#define NOTMEMTYPE 
#undef	NOTMEMBER
#define	NOTMEMBER notmember
int 	i;
int 	n;
NOTMEMTYPE
long	vals[NM_LEN];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	n = NOTMEMBER(VALUE_LIST, NELEM(VALUE_LIST), vals);

	for (i = 0; i < n; i++) {

		debug(1, "Trying arg of %d", vals[i]);

		VALUE_ARG = vals[i];
		startcall(display);
		if (isdeleted())
			return;
		XAllowEvents(display, event_mode, test_time);
		endcall(display);
		if (geterr() != BadValue) {
			report("Got %s, Expecting BadValue", errorname(geterr()));
			FAIL;
		}

		if (geterr() == BadValue)
			CHECK;
		else {
			trace("Value of %d did not give BadValue", vals[i]);
			FAIL;
		}
	}

	CHECKPASS(n);
	restoredevstate();
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EVal.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	t010, 10,
	t011, 11,
	t012, 12,
	t013, 13,
	t014, 14,
	t015, 15,
	t016, 16,
	t017, 17,
	t018, 18,
	t019, 19,
	t020, 20,
	t021, 21,
	t022, 22,
	t023, 23,
	t024, 24,
	t025, 25,
	t026, 26,
	t027, 27,
	t028, 28,
	t029, 29,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = focusstartup;
void	(*tet_cleanup)() = focuscleanup;
