/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: chngkybrdm.m,v 1.6 92/06/11 17:11:50 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XChangeKeyboardMapping(display, first_keycode, keysyms_per_keycode, keysyms, num_codes)
 *   Display	*display;
 *   int 	first_keycode;
 *   int 	keysyms_per_keycode;
 *   KeySym	*keysyms;
 *   int 	num_codes;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XChangeKeyboardMapping	1
char    *TestName = "XChangeKeyboardMapping";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XChangeKeyboardMapping function
 */
static Display	*display;
static int 	first_keycode;
static int 	keysyms_per_keycode;
static KeySym	*keysyms;
static int 	num_codes;



#define	MAXKPK	5	/* Maximum keysyms_per_keyocde we will use */
#define	MAXCODES 255	/* Max number of keycodes we will use */

static	int 	First;
static	int 	ncodes;
static	int 	kpk;	/* Keysyms per keycode */
static	KeySym	*oldkeym;

/*
 * Can't really assume that there are any particular keysym names defined so
 * we use our own arbitrary values.
 */
#define	XT_KSYM1	0x5678
#define	XT_KSYM2	0x9328

static	KeySym	Keys[MAXKPK*MAXCODES] = {
	XT_KSYM1, XT_KSYM2};

/*
 * Set startup and cleanup functions to save and restore
 * the original keyboard map.
 */
static void
savekeymap()
{
int 	last;

	startup();

	if(Dsp) {
		XDisplayKeycodes(Dsp, &First, &last);
		ncodes = last-First;

		oldkeym = XGetKeyboardMapping(Dsp, First, ncodes, &kpk);
	}
}

static void
cleankeymap()
{
	if(Dsp) {
		XChangeKeyboardMapping(Dsp, First, kpk, oldkeym, ncodes);
		XFree((char*) oldkeym);
	}

	cleanup();
}



int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	first_keycode = First;
	keysyms_per_keycode = 1;
	keysyms = Keys;
	num_codes = 2;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XChangeKeyboardMapping-1.(A)
 * A call to XChangeKeyboardMapping associates
 * keysyms_per_keycode KeySyms for each of the num_codes
 * KeyCodes starting with first_keycode, with the KeySyms
 * being taken from the array keysyms.
 */
static void
t001()
{
int 	i, j;
int 	newkpk;
KeySym	*newmap;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	first_keycode = First+3;
	keysyms_per_keycode = MAXKPK;
	num_codes = 6;

	for (i = 0; i < keysyms_per_keycode*num_codes; i++)
		keysyms[i] = XT_KSYM1;

	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardMapping(display, first_keycode, keysyms_per_keycode, keysyms, num_codes);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	newmap = XGetKeyboardMapping(display, first_keycode, num_codes, &newkpk);

	for (i = 0; i < num_codes; i++) {
		for (j = 0; j < keysyms_per_keycode; j++) {
			if (keysyms[i*keysyms_per_keycode+j] == newmap[i*newkpk+j])
				CHECK;
			else {
				report("Keysym for keycode %d was 0x%x, expecting 0x%x",
					first_keycode+i,
					newmap[i*newkpk+j], keysyms[i*keysyms_per_keycode+j]);
				FAIL;
				break;	/* probably pointless to continue */
			}
		}
	}

	CHECKPASS(keysyms_per_keycode*num_codes);

	XFree((char*)newmap);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardMapping-2.(A)
 * The KeySyms for KeyCodes outside the specified range remain
 * unchanged.
 */
static void
t002()
{
KeySym	*oldmap;
int 	oldkpk;
KeySym	*newmap;
int 	newkpk;
int 	i, j;
int 	oldind, newind;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	oldmap = XGetKeyboardMapping(display, First, ncodes, &oldkpk);

	first_keycode = First+2;
	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardMapping(display, first_keycode, keysyms_per_keycode, keysyms, num_codes);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	newmap = XGetKeyboardMapping(display, First, ncodes, &newkpk);

	for (i = 0; i < ncodes; i++) {

		/* Skip the ones that were changed. */
		if (i + First >= first_keycode && i + First < first_keycode+num_codes)
			continue;

		/*
		 * The call may have altered the number of keysyms per keycode,
		 * so must only check that the previously set position have
		 * not changed.  Expansion should have been filled with NoSymbol.
		 */
		for (j = 0; j < oldkpk; j++) {
			oldind = i*oldkpk + j;
			newind = i*newkpk + j;
			if (oldmap[oldind] == newmap[newind])
				CHECK;
			else {
				report("Keysym outside of range altered");
				report("  keycode %d was 0x%x, expecting 0x%x",
					First+i, newmap[newind], oldmap[oldind]);
				FAIL;
			}
		}
	}

	/* Number of codes that should not have been changed is ncodes-num_codes */
	CHECKPASS((ncodes - num_codes)*oldkpk);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardMapping-3.(A)
 * A call to XChangeKeyboardMapping generates a MappingNotify
 * event.
 */
static void
t003()
{
XEvent	ev;
XMappingEvent	good;
int 	n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	first_keycode = First + 2;
	num_codes = 3;
	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardMapping(display, first_keycode, keysyms_per_keycode, keysyms, num_codes);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	defsetevent(good, display, MappingNotify);
	good.window = None;	/* unused */
	good.request = MappingKeyboard;
	good.first_keycode = first_keycode;
	good.count = num_codes;

	n = getevent(display, &ev);
	if (n == 0) {
		report("Expecting one MappingEvent");
		FAIL;
		return;
	} else
		CHECK;

	if (checkevent((XEvent*)&good, &ev))
		FAIL;
	else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardMapping-4.(A)
 * It is legal for the KeySym NoSymbol to appear anywhere in
 * the KeySym list for a particular KeyCode.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	Keys[0] = NoSymbol;
	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardMapping(display, first_keycode, keysyms_per_keycode, keysyms, num_codes);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (geterr() == Success)
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardMapping-5.(A)
 * When first_keycode is less than the value of min_keycode
 * returned by XDisplayKeycodes, then a BadValue error occurs.
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	first_keycode = First-1;
	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardMapping(display, first_keycode, keysyms_per_keycode, keysyms, num_codes);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XChangeKeyboardMapping-6.(A)
 * When first_keycode+num_codes-1 is greater than the
 * value of max_keycode returned by XDisplayKeycodes, then a
 * BadValue error occurs.
 */
static void
t006()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	num_codes = ncodes;
	first_keycode = First+10;

	startcall(display);
	if (isdeleted())
		return;
	XChangeKeyboardMapping(display, first_keycode, keysyms_per_keycode, keysyms, num_codes);
	endcall(display);
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XChangeKeyboardMapping-7.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t007()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = savekeymap;
void	(*tet_cleanup)() = cleankeymap;
