/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtkybrdcnt.m,v 1.3 92/06/11 17:13:50 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XGetKeyboardControl(display, values_return)
 *   Display	*display;
 *   XKeyboardState	*values_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetKeyboardControl	1
char    *TestName = "XGetKeyboardControl";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XGetKeyboardControl function
 */
static Display	*display;
static XKeyboardState	*values_return;



static XKeyboardState	Ksvals;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	values_return = &Ksvals;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetKeyboardControl-1.(A)
 * A call to XGetKeyboardControl returns the current control
 * values for the keyboard to values_return.
 */
static void
t001()
{
XKeyboardControl	kset;
XKeyboardState	oldkb;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	/*
	 * Actually first we save the original values.
	 */
	values_return = &oldkb;
	startcall(display);
	if (isdeleted())
		return;
	XGetKeyboardControl(display, values_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	kset.key_click_percent = 21;
	kset.bell_percent = 12;
	kset.bell_pitch = 402;
	kset.bell_duration = 222;

	XChangeKeyboardControl(display, KBKeyClickPercent|KBBellPercent|KBBellPitch|KBBellDuration, &kset);

	if (isdeleted())
		return;

	values_return = &Ksvals;
	startcall(display);
	if (isdeleted())
		return;
	XGetKeyboardControl(display, values_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (values_return->key_click_percent == kset.key_click_percent)
		CHECK;
	else {
		report("Value of key_click_percent was %d, expecting %d",
			values_return->key_click_percent, kset.key_click_percent);
		FAIL;
	}
	if (values_return->bell_percent == kset.bell_percent)
		CHECK;
	else {
		report("Value of bell_percent was %d, expecting %d",
			values_return->bell_percent, kset.bell_percent);
		FAIL;
	}
	if (values_return->bell_pitch == kset.bell_pitch)
		CHECK;
	else {
		report("Value of bell_pitch was %d, expecting %d",
			values_return->bell_pitch, kset.bell_pitch);
		FAIL;
	}
	if (values_return->bell_duration == kset.bell_duration)
		CHECK;
	else {
		report("Value of bell_duration was %d, expecting %d",
			values_return->bell_duration, kset.bell_duration);
		FAIL;
	}

	CHECKPASS(4);

	/*
	 * Attempt to restore the original values.
	 */
	kset.key_click_percent = oldkb.key_click_percent;
	kset.bell_percent = oldkb.bell_percent;
	kset.bell_pitch = oldkb.bell_pitch;
	kset.bell_duration = oldkb.bell_duration;

	XChangeKeyboardControl(display, KBKeyClickPercent|KBBellPercent|KBBellPitch|KBBellDuration, &kset);

	XSync(display, False);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
