/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtmdfrmppn.m,v 1.8 92/06/11 17:14:08 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   XModifierKeymap *
 *   XGetModifierMapping(display)
 *   Display	*display;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetModifierMapping	1
char    *TestName = "XGetModifierMapping";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XGetModifierMapping function
 */
static Display	*display;


static int pgrabbed(display, win)
Display *display;
Window win;
{
	PointerPlace	*p;
	XEvent		ev;
	int		result;

	XSync(display, True);
	XSelectInput(display, win, PointerMotionMask);
	p = warppointer(display, win, 0,0);
	(void) warppointer(display, win, 10,10);
	(void) warppointer(display, p->oroot, p->ox, p->oy);
	result = (getevent(display, &ev) == 0);
	XSync(display, True);
	return result;
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetModifierMapping-1.(A)
 * A call to XGetModifierMapping returns a pointer to a newly
 * created XModifierKeymap structure that contains the KeyCodes
 * being used as modifiers and the structure can be freed with
 * XFreeModifiermap.
 */
static void
t001()
{
XModifierKeymap	*mkmp;
extern struct	valname	S_modifier[];
extern int 	NS_modifier;
int 	mod;
int 	set;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	mkmp = XGetModifierMapping(display);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	for (mod = 0; mod < NS_modifier; mod++) {
		trace("modifier %s:", S_modifier[mod].name);
		for (set = 0; set < mkmp->max_keypermod; set++) {
			trace("  0x%x", mkmp->modifiermap[set+mod*mkmp->max_keypermod]);
		}
	}

	XFreeModifiermap(mkmp);

        CHECK;  /* Merely check and record that we reach this point. */
        CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetModifierMapping-2.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When only zero values appear in the set for any modifier,
 * then that modifier is disabled.
 */
static void
t002()
{
Window win;
Display *client2;
int minkc, maxkc;
XModifierKeymap *map;
XModifierKeymap *savemap;
int k;
int i;
int row;
int non_zero = 0;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(1))
		return;
	else
		CHECK;
	win = defwin(display);
	client2 = opendisplay();
	XDisplayKeycodes(display, &minkc, &maxkc);
	startcall(display);
	if (isdeleted())
		return;
	savemap = XGetModifierMapping(display);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	startcall(display);
	if (isdeleted())
		return;
	map = XGetModifierMapping(display);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (isdeleted() || geterr() != Success || !map || !savemap) {
		delete("Could not get initial modifier key map.");
		return;
	} else
		CHECK;
	row = ShiftMapIndex * map->max_keypermod;
	for (i=0; i<map->max_keypermod; i++) {
	    if (map->modifiermap[ row+i ]) {
		trace("Shift had keycode %d.", map->modifiermap[ row+i ]);
		non_zero++;
		map->modifiermap[ row+i ] = 0;
	    }
	}
	trace("Shift had %d keycodes.", non_zero);
	if (XSetModifierMapping(display, map) != MappingSuccess || isdeleted()) {
		delete("Could not set new mapping with all zeroes for Shift.");
		XSetModifierMapping(display, savemap);
		XFreeModifiermap(map);
		XFreeModifiermap(savemap);
		return;
	} else
		CHECK;
	XGrabButton(display, AnyButton, ShiftMask, win, False, 0, GrabModeAsync, GrabModeAsync, None, None);
	if (isdeleted()) {
		delete("Could not set passive button grab.");
		XSetModifierMapping(display, savemap);
		XFreeModifiermap(map);
		XFreeModifiermap(savemap);
		return;
	} else
		CHECK;
	if (pgrabbed(client2, win)) {
		delete("Passive button grab erroneously triggered.");
		XSetModifierMapping(display, savemap);
		XFreeModifiermap(map);
		XFreeModifiermap(savemap);
		return;
	} else
		CHECK;
	(void) warppointer(display, win, 2,2); /* pgrabbed restores pointer */
	for(k=minkc; k <= maxkc; k++) {
		keypress(display, k);
		buttonpress(display, Button1);
		if (pgrabbed(client2, win)) {
			report("Despite Shift being disabled keycode %d acted like Shift modifier.", k);
			FAIL;
		} else
			CHECK;
		relalldev();
	}

	XUngrabButton(display, AnyButton, AnyModifier, win);
	XSetModifierMapping(display, savemap);
	XFreeModifiermap(map);
	XFreeModifiermap(savemap);

	CHECKPASS(5 + (maxkc - minkc + 1));
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
