/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtpntrmppn.m,v 1.7 92/06/11 17:14:23 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XGetPointerMapping(display, map_return, nmap)
 *   Display	*display;
 *   unsigned char	*map_return;
 *   int 	nmap;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetPointerMapping	1
char    *TestName = "XGetPointerMapping";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XGetPointerMapping function
 */
static Display	*display;
static unsigned char	*map_return;
static int 	nmap;


static int 	ValueReturn;


/*
 * The number of buttons should be between 1 and 5 -- allow more space
 * in the array than this.
 */
#define	MAPSIZE	20
static	unsigned char	Map[MAPSIZE];


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	map_return = Map;
	nmap = MAPSIZE;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetPointerMapping-1.(B)
 *   Reason: There is no known reliable test method for this assertion
 * A call to XGetPointerMapping returns in map_return[i] the
 * logical button numbers of the physical buttons i+1.
 */
static void
t001()
{
int 	nbuttons;
int	i;
Window	win;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	nbuttons = XGetPointerMapping(display, map_return, nmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	trace("Number of buttons reported as %d", nbuttons);

	if (noext(nbuttons)) {
		report("There is no reliable test method, but a touch test was performed");
		UNTESTED;
		return;
	} else
		CHECK;
	win = defwin(display);
	if (isdeleted()) {
		delete("Failed to create ButtonPress window.");
		return;
	} else
		CHECK;
	XSelectInput(display, win, ButtonPressMask);
	XSync(display, True);
	if (isdeleted()) {
		delete("Failed to select for ButtonPress. May indicate competing clients (e.g. window managers).");
		return;
	} else
		CHECK;
	(void) warppointer(display, win, 2, 2);
	for (i=0; i<nbuttons; i++) {
		XEvent ev;

		XSync(display, True); /* clear out event Q */
		_startcall(display); /* install and reset error handlers */
		buttonpress(display, (unsigned int)i+1);
		if (!XCheckWindowEvent(display, win, ButtonPressMask, &ev)) {
			delete("No event received for button %d.", i+1);
			relalldev();
			return;
		} else
			CHECK;
		relalldev();
		_endcall(display); /* back to catching unexpected ones */
		if (ev.xbutton.button != map_return[i]) {
			report("Mapping for button %d is %d not %d.",
				i+1, (int)ev.xbutton.button, (int)map_return[i]);
			FAIL;
		} else
			CHECK;
		trace("Mapping for button %d is %d, expecting %d.",
				i+1, (int)ev.xbutton.button, (int)map_return[i]);
	}
	CHECKPASS(3+2*nbuttons);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetPointerMapping-2.(B)
 *   Reason: There is no known reliable test method for this assertion
 * A call to XGetPointerMapping returns the number of physical
 * buttons actually on the pointer.
 */
static void
t002()
{
int	nbuttons;
int	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	nbuttons = XGetPointerMapping(display, map_return, nmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (nbuttons < 1 || nbuttons > 5) {
		report("Protocol limit of 1..5 buttons exceeded (%d).", nbuttons);
		FAIL;
	} else
		CHECK;

	if (noext(nbuttons)) {
		report("There is no reliable test method, but a touch test was performed");
		UNTESTED;
		return;
	} else
		CHECK;

	for(i=0; i<5; i++) {
		_startcall(display); /* install and reset error handlers */
		buttonpress(display, (unsigned int)i+1);
		if (geterr() != ((i<nbuttons)?Success:BadValue)) {
			report("Apparently button %d could%s be pressed.",
				i+1, (i<nbuttons)?"n't":"");
			FAIL;
		} else
			CHECK;
		relalldev();
		_endcall(display); /* back to catching unexpected ones */
	}
	CHECKPASS(2+5);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetPointerMapping-3.(A)
 * When the nmap argument is less than the number of elements
 * in the pointer mapping, then only the first nmap elements
 * are returned in map_return.
 */
static void
t003()
{
int 	nbuttons;
int 	i;
#define	TEST_VAL	((unsigned char)255)
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	startcall(display);
	if (isdeleted())
		return;
	nbuttons = XGetPointerMapping(display, map_return, nmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	for (i = 0; i < MAPSIZE; i++)
		Map[i] = TEST_VAL;

	/*
	 * Its not clear that asking for 0 elements is really sensible so we
	 * only do it when there is only one button.
	 */
	if (nbuttons == 1)
		nmap = 0;
	else
		nmap = 1;

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XGetPointerMapping(display, map_return, nmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	for (i = nmap; i < MAPSIZE; i++) {
		if (Map[i] != TEST_VAL) {
			report("An element beyond the first nmap was returned");
			report(" element %d was %u, expecting %u", i, (unsigned)Map[i],
				(unsigned)TEST_VAL);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(MAPSIZE-nmap);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
