/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtscrnsvr.m,v 1.4 92/06/11 17:14:32 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XGetScreenSaver(display, timeout_return, interval_return, prefer_blanking_return, allow_exposures_return)
 *   Display	*display;
 *   int 	*timeout_return;
 *   int 	*interval_return;
 *   int 	*prefer_blanking_return;
 *   int 	*allow_exposures_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetScreenSaver	1
char    *TestName = "XGetScreenSaver";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XGetScreenSaver function
 */
static Display	*display;
static int 	*timeout_return;
static int 	*interval_return;
static int 	*prefer_blanking_return;
static int 	*allow_exposures_return;



static	int 	tr;
static	int 	ir;
static	int 	pbr;
static	int 	aer;

static	int 	origt;
static	int 	origi;
static	int 	origpb;
static	int 	origae;

static void
savesaver()
{
	startup();
	if(Dsp)
		XGetScreenSaver(Dsp, &origt, &origi, &origpb, &origae);
}

static void
resetsaver()
{
	if(Dsp)
		XSetScreenSaver(Dsp, origt, origi, origpb, origae);
	cleanup();
}


#define	TOUT	71
#define	INTERVAL	57
#define	BLANKING	PreferBlanking
#define	EXPOSURES	AllowExposures

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	timeout_return = &tr;
	interval_return = &ir;
	prefer_blanking_return = &pbr;
	allow_exposures_return = &aer;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetScreenSaver-1.(A)
 * A call to XGetScreenSaver returns the current screen saver
 * values.
 */
static void
t001()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	XSetScreenSaver(display, TOUT, INTERVAL, BLANKING, EXPOSURES);

	startcall(display);
	if (isdeleted())
		return;
	XGetScreenSaver(display, timeout_return, interval_return, prefer_blanking_return, allow_exposures_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (*timeout_return == TOUT)
		CHECK;
	else {
		report("timeout_return was %d, expecting %d", *timeout_return, TOUT);
		FAIL;
	}
	if (*interval_return == INTERVAL)
		CHECK;
	else {
		report("interval_return was %d, expecting %d",
			*interval_return, INTERVAL);
		FAIL;
	}
	if (*prefer_blanking_return == BLANKING)
		CHECK;
	else {
		report("prefer_blanking_return was %d, expecting %d",
			*prefer_blanking_return, BLANKING);
		FAIL;
	}
	if (*allow_exposures_return == EXPOSURES)
		CHECK;
	else {
		report("allow_exposures_return was %d, expecting %d",
			*allow_exposures_return, EXPOSURES);
		FAIL;
	}

	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = savesaver;
void	(*tet_cleanup)() = resetsaver;
