/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: nwmdfrmp.m,v 1.4 92/06/11 17:15:18 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   XModifierKeymap *
 *   XNewModifiermap(max_keys_per_mod)
 *   int 	max_keys_per_mod;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XNewModifiermap	1
char    *TestName = "XNewModifiermap";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XNewModifiermap function
 */
static int 	max_keys_per_mod;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	max_keys_per_mod = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XNewModifiermap-1.(A)
 * A call to XNewModifiermap returns a pointer to a
 * XModifierKeymap structure, that can be freed with
 * XFreeModifiermap, with max_keys_per_mod entries
 * preallocated for each of the 8 modifiers.
 */
static void
t001()
{
XModifierKeymap	*mkmp;
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	max_keys_per_mod = 12;

	startcall(Dsp);
	if (isdeleted())
		return;
	mkmp = XNewModifiermap(max_keys_per_mod);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (mkmp->max_keypermod == max_keys_per_mod)
		CHECK;
	else {
		report("max_keypermod was %d, expecting %d", mkmp->max_keypermod,
			max_keys_per_mod);
		FAIL;
	}

	/*
	 * Access each member of the array.  If this fails (with a signal)
	 * then the TET will delete the test.  This is not exactly right
	 * because it should really fail. Anyway the problem still has to
	 * be fixed to get a PASS so there is no real problem.
	 */
	for (i = 0; i < 8*mkmp->max_keypermod; i++) {
		mkmp->modifiermap[i] = NoSymbol;
	}

	XFreeModifiermap(mkmp);

	max_keys_per_mod = 0;

	startcall(Dsp);
	if (isdeleted())
		return;
	mkmp = XNewModifiermap(max_keys_per_mod);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (mkmp->max_keypermod == 0)
		CHECK;
	else {
		report("max_keypermod was %d, not zero", mkmp->max_keypermod);
		FAIL;
	}
	XFreeModifiermap(mkmp);

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
