/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: qrykymp.m,v 1.7 92/06/11 17:15:26 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XQueryKeymap(display, keys_return)
 *   Display	*display;
 *   char	*keys_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XQueryKeymap	1
char    *TestName = "XQueryKeymap";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XQueryKeymap function
 */
static Display	*display;
static char	*keys_return;



static	char	Keys[32];


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	keys_return = Keys;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XQueryKeymap-1.(B)
 *   Reason: There is no known reliable test method for this assertion
 * A call to XQueryKeymap returns a bit vector with bits set
 * to 1 where the corresponding key is currently in the logical
 * down state and all other bits set to 0.
 */
static void
t001()
{
int i;
char *p,*q;
int fcount;
char pressed[32];
int minkc, maxkc;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if (noext(0)) {
		startcall(display);
		if (isdeleted())
			return;
		XQueryKeymap(display, keys_return);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		report("There is no reliable test method, but a touch test was performed");

		UNTESTED;
		return;
	} else
		CHECK;

	/* expect all zero map. If not can't start. Should we fail? */
	startcall(display);
	if (isdeleted())
		return;
	XQueryKeymap(display, keys_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	for(fcount=i=0,p=keys_return; i<32; i++,p++)
		if (*p)
			fcount++;

	if (fcount) {
		delete("%d keys appear to be pressed down.", fcount);
		return;
	} else
		CHECK;

	XDisplayKeycodes(display, &minkc, &maxkc);

	for(p=pressed,i=0; i<32; i++,p++)
		*p = (char) 0;

	for(i=minkc; i < minkc+5 && i <= maxkc; i++) {
		keypress(display, i);
		pressed[i/8] |= (char) (0x1 << (i % 8));
	}

	/* check map now has only those set */
	startcall(display);
	if (isdeleted())
		return;
	XQueryKeymap(display, keys_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	for(p=pressed,q=keys_return,fcount=i=0; i<32; i++,p++,q++)
		if (*p != *q) {
			char diffs = (unsigned)*p ^ (unsigned)*q;
			int di;

			fcount++;
			for(di=0; di<8; di++) {
				char mask = (char) (0x1 << di);

				if (diffs & mask)
					report("Key %d%s pressed.", 8*i + di,
						(*p & mask) ? "" : " not");
			}
		}

	if (fcount)
		FAIL;
	else
		CHECK;

	relalldev(); /* now all released */

	/* check map now all zero. */
	startcall(display);
	if (isdeleted())
		return;
	XQueryKeymap(display, keys_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	for(fcount=i=0,p=keys_return; i<32; i++,p++)
		if (*p)
			fcount++;

	if (fcount) {
		report("%d keys appear to be pressed down.", fcount);
		FAIL;
	} else
		CHECK;

	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
