/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stpntrmppn.m,v 1.9 92/06/11 17:16:46 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XSetPointerMapping(display, map, nmap)
 *   Display	*display;
 *   unsigned char	*map;
 *   int 	nmap;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetPointerMapping	1
char    *TestName = "XSetPointerMapping";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XSetPointerMapping function
 */
static Display	*display;
static unsigned char	*map;
static int 	nmap;


static int 	ValueReturn;


/* Maximum button number allowed. */
#define	MAXBUTTON	5

/*
 * MAPSIZE must be at least one greater than the maximum number of buttons
 * allowed.  We use a much larger value.
 */
#define	MAPSIZE	32
static	unsigned char	Map[MAPSIZE];

static	int 	numbuttons;

/*
 * Set up the number of buttons.  Also set the nmap value to numbuttons (
 * the test may later override this).
 */
static	void
getnbutton()
{
	numbuttons = XGetPointerMapping(Dsp, Map, MAPSIZE);
	nmap = numbuttons;
	if (isdeleted())
		delete("XGetPointerMapping failed");
}

/*
 * Save and restore the old map.
 */
static unsigned char	oldmap[MAPSIZE];
static void
savemap()
{
	startup();
	if(Dsp)
		numbuttons = XGetPointerMapping(Dsp, oldmap, MAPSIZE);
}

static void
restoremap()
{
	if(Dsp)
		(void) XSetPointerMapping(Dsp, oldmap, numbuttons);
	cleanup();
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	map = Map;
	nmap = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSetPointerMapping-1.(A)
 * A successful call to XSetPointerMapping sets the pointer
 * mapping for the physical buttons to the nmap logical button
 * numbers specified in the array map and returns
 * MappingSuccess.
 */
static void
t001()
{
unsigned char	newmap[MAPSIZE];
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();
	getnbutton();

	/*
	 * Cycle the current mapping around.
	 */
	for (i = 0; i < numbuttons; i++) {
		map[i] = map[i] + 1;
		if (map[i] > MAXBUTTON)
			map[i] = 1;
	}

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XSetPointerMapping(display, map, nmap);
	endcall(display);
	if (ValueReturn != MappingSuccess) {
		report("Returned value was %d, expecting MappingSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	(void) XGetPointerMapping(display, newmap, MAPSIZE);
	if (isdeleted()) {
		delete("Could not get pointer mapping");
		return;
	}

	for (i = 0; i < numbuttons; i++) {
		if (map[i] == newmap[i])
			CHECK;
		else {
			report("Mapping not set correctly in position %d", i);
			report("  was %u, expecting %u", (unsigned)newmap[i], (unsigned)map[i]);
			FAIL;
		}
	}

	CHECKPASS(numbuttons);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetPointerMapping-2.(A)
 * When a call to XSetPointerMapping is successful, then a
 * MappingNotify event is generated.
 */
static void
t002()
{
XEvent	ev;
XMappingEvent	good;
int 	n;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();
	getnbutton();

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XSetPointerMapping(display, map, nmap);
	endcall(display);
	if (ValueReturn != MappingSuccess) {
		report("Returned value was %d, expecting MappingSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	n = getevent(display, &ev);
	if (n == 0 || ev.type != MappingNotify) {
		report("Expecting a MappingNotify event, received %s", n? eventname(ev.type): "no event");
		FAIL;
	} else
		CHECK;

	defsetevent(good, display, MappingNotify);
	good.window = None;	/* Not used */
	good.request = MappingPointer;
	if (checkevent((XEvent*)&good, &ev))
		FAIL;
	else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetPointerMapping-3.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When an element of map is zero, then the corresponding
 * physical button is disabled.
 */
static void
t003()
{
int	i;
unsigned char zmap[5];
Window	win;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();
	getnbutton();

	if (noext(numbuttons))
		return;
	else
		CHECK;

	if (numbuttons<1 || numbuttons>5) {
		report("Protocol limit of 1..5 buttons exceeded (%d).", numbuttons);
		return;
	} else
		CHECK;

	for(i=0; i<numbuttons; i++)
		zmap[i] = map[i];
	win = defwin(display);
	(void) warppointer(display, win, 2,2);
	XSelectInput(display, win, ButtonPressMask);

	for(i=0; i<numbuttons; i++) {
		unsigned char mapsave = zmap[i];
		int ret;
		XEvent ev;

		zmap[i] = 0; /* disable button i+1 */
		map = zmap;
		startcall(display);
		if (isdeleted())
			return;
		ret = XSetPointerMapping(display, map, nmap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		if (ret != MappingSuccess) {
			delete("Couldn't set zero entry for button %d.", i+1);
			return;
		} else
			CHECK;

		XSync(display, True); /* discard event queue */
		_startcall(display);	/* set error handler etc. */
		buttonpress(display, (unsigned int)i+1);
		relalldev();
		_endcall(display);
		if (geterr() != Success) {
			delete("Couldn't simulate pressing button %d.", i+1);
			return;
		} else
			CHECK;
		if (XCheckWindowEvent(display, win, ButtonPressMask, &ev)) {
			report("Got event after pressing disabled button %d.", i+1);
			FAIL;
		} else
			CHECK;
		zmap[i] = mapsave; /* restore button i+1 */
	}
	CHECKPASS(2+numbuttons*3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetPointerMapping-4.(A)
 * Elements of the map array are not restricted in value by
 * the number of physical buttons.
 */
static void
t004()
{
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();
	getnbutton();

	for (i = 0; i < numbuttons; i++)
		map[i] = i;

	map[0] = numbuttons+1;

	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XSetPointerMapping(display, map, nmap);
	endcall(display);
	if (ValueReturn != MappingSuccess) {
		report("Returned value was %d, expecting MappingSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if (geterr() == Success)
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetPointerMapping-5.(B)
 *   Reason: There is no known reliable test method for this assertion
 * When any of the buttons to be altered are logically in the
 * down state, then a call to XSetPointerMapping returns
 * MappingBusy, and the mapping is not changed.
 */
static void
t005()
{
int	i;
int	nb;
unsigned char firstmap[MAPSIZE];
unsigned char newmap[MAPSIZE];
int	ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();
	getnbutton();

	if (noext(numbuttons))
		return;
	else
		CHECK;

	if (numbuttons<1 || numbuttons>5) {
		delete("Protocol limit of 1..5 buttons exceeded (%d).", numbuttons);
		return;
	} else
		CHECK;
	for(i=0; i<numbuttons; i++)
		firstmap[i] = map[i] = (unsigned char)i+1;
	startcall(display);
	if (isdeleted())
		return;
	ret = XSetPointerMapping(display, map, nmap);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	if (isdeleted() || ret != MappingSuccess) {
		delete("Couldn't set up monotonic map to start with.");
		return;
	} else
		CHECK;
	for(i=0; i<numbuttons; i++) {
		int j;
		unsigned char tmp;

		/* cycle map by one, until map[i] is not i=1 */
		do {
			for(tmp=map[0], j=0; j<(numbuttons-1); j++)
				map[j] = map[j+1];
			map[numbuttons - 1] = tmp;
		} while (map[i] == (unsigned char)i+1);

		_startcall(display);	/* set error handler etc. */
		buttonpress(display, (unsigned int)i+1);
		if (geterr() != Success) {
			delete("Couldn't simulate pressing button %d.", i+1);
			relalldev();
			_endcall(display);
			return;
		} else
			CHECK;
		_endcall(display);
		startcall(display);
		if (isdeleted())
			return;
		ret = XSetPointerMapping(display, map, nmap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}
		_startcall(display);	/* set error handler etc. */
		relalldev();
		_endcall(display);
		if (ret != MappingBusy) {
			report("Expecting MappingBusy with button %d, got %s (%d).",
				i+1, (ret==MappingSuccess)?"MappingSuccess":"<unknown>", ret);
			FAIL;
		} else
			CHECK;
		nb = XGetPointerMapping(display, newmap, numbuttons);
		if (isdeleted()) {
			delete("Couldn't get current pointer map for comparison.");
			return;
		} else
			CHECK;
		if (nb != numbuttons) {
			report("Button numbers changed from %d to %d.", numbuttons, nb);
			FAIL;
		} else {
			for(j=0; j<numbuttons; j++)
				if (firstmap[j] != newmap[j]) {
					report("Maps differ for button %d, was %d now %d.", j+1, firstmap[j], newmap[j]);
					FAIL;
				} else
					CHECK;
		}
	}
	CHECKPASS(3+numbuttons*(3+numbuttons*1));
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetPointerMapping-6.(A)
 * When nmap is not the same as the length that
 * XGetPointerMapping would return, then a BadValue error
 * occurs.
 */
static void
t006()
{
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();
	getnbutton();

	nmap = numbuttons + 2;
	for (i = 0; i < nmap; i++)
		map[i] = i;	/* MAPSIZE is large enough to allow this */
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XSetPointerMapping(display, map, nmap);
	endcall(display);
	if (ValueReturn != MappingSuccess) {
		report("Returned value was %d, expecting MappingSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	else
		FAIL;	/* done already */
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetPointerMapping-7.(C)
 * If there is more than one button: When two elements of map
 * have the same non-zero value, then a BadValue error occurs.
 */
static void
t007()
{
int 	i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();
	getnbutton();

	if (numbuttons < 2) {
		unsupported("There are less than two buttons");
		return;
	}

	for (i = 0; i < nmap; i++)
		map[i] = i;

	map[0] = map[1];
	startcall(display);
	if (isdeleted())
		return;
	ValueReturn = XSetPointerMapping(display, map, nmap);
	endcall(display);
	if (ValueReturn != MappingSuccess) {
		report("Returned value was %d, expecting MappingSuccess", ValueReturn);
		FAIL;
	}
	if (geterr() != BadValue) {
		report("Got %s, Expecting BadValue", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadValue)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = savemap;
void	(*tet_cleanup)() = restoremap;
