/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: unnstllclr.m,v 1.4 92/06/11 17:17:46 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XUninstallColormap(display, colormap)
 *   Display	*display;
 *   Colormap	colormap;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XUninstallColormap	1
char    *TestName = "XUninstallColormap";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_COLORMAP colormap


/*
 * Arguments to the XUninstallColormap function
 */
static Display	*display;
static Colormap	colormap;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	colormap = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	colormap = ErrdefColormap;
}

/*
 * Assertion XUninstallColormap-1.(B)
 *   Reason: There is no known reliable test method for this assertion
 * A call to XUninstallColormap removes the specified
 * colourmap from the required list for its associated screen.
 */
static void
t001()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XUninstallColormap-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the specified colourmap is uninstalled by a call to
 * XUninstallColormap, then a ColormapNotify event is
 * generated on each window that has that colourmap.
 */
static void
t002()
{
XVisualInfo	*vp;
Window	win;
XEvent	ev;
XColormapEvent	good;
XColormapEvent	*cmp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	defsetevent(good, display, ColormapNotify);
	good.new = False;
	good.state = ColormapUninstalled;

	for (resetvinf(VI_WIN); nextvinf(&vp); ) {

		win = makewin(display, vp);
		colormap = makecolmap(display, vp->visual, AllocNone);
		XSetWindowColormap(display, win, colormap);
		XSelectInput(display, win, ColormapChangeMask);
		XInstallColormap(display, colormap);

		startcall(display);
		if (isdeleted())
			return;
		XUninstallColormap(display, colormap);
		endcall(display);
		if (geterr() != Success) {
			report("Got %s, Expecting Success", errorname(geterr()));
			FAIL;
		}

		while (XCheckWindowEvent(display, win, ColormapChangeMask, &ev)) {
			cmp = (XColormapEvent*)&ev;
			if (cmp->colormap != colormap)
				continue;

			good.window = win;
			good.colormap = colormap;
			if (checkevent((XEvent*)&good, &ev))
				FAIL;
			else
				CHECK;

		}
	}

	if (fail == 0 && pass == nvinf())
		PASS;
	else if (fail == 0 && pass == 0)
		untested("The assertion could not be tested since no event was sent");
	else if (fail == 0)
		untested("The assertion could only be tested for some visual types");

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XUninstallColormap-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When another colourmap is installed or uninstalled as a
 * side effect of a call to XUninstallColormap, then a
 * ColormapNotify event is generated on each window that has
 * that colourmap.
 */
static void
t003()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XUninstallColormap-4.(B)
 *   Reason: There is no known reliable test method for this assertion
 * No other colourmaps are removed from the required list by a
 * call to XUninstallColormap.
 */
static void
t004()
{
	report("There is no known reliable test method for this assertion");
	UNTESTED;
}

/* Including from file error/ECol.mc */
/*
 * Assertion XUninstallColormap-5.(A)
 * When a colourmap argument does not name a valid colourmap,
 * then a BadColor error occurs.
 */
/* End of included file error/ECol.mc */

/* Including from file error/ECol.mc */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_COLORMAP = badcolormap(A_DISPLAY, DRW(A_DISPLAY));
	startcall(display);
	if (isdeleted())
		return;
	XUninstallColormap(display, colormap);
	endcall(display);
	if (geterr() != BadColor) {
		report("Got %s, Expecting BadColor", errorname(geterr()));
		FAIL;
	}
	if(geterr() == BadColor)
		PASS;
	else 
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/ECol.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
