/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: kyrls.m,v 1.13 92/07/06 19:59:05 rws Exp $
 * 
 */
/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_KeyRelease	1
char    *TestName = "KeyRelease";



#define EVENT KeyRelease
#define EVENTMASK KeyReleaseMask

static void
presskey(Dpy, w, kc)
Display		*Dpy;
Window		w;
int		kc;
{
	/*
	 * Note that we do not call XSetInputFocus() since 2 tests require
	 * the event window to be an ancestor of the source window.
	 */
	keypress(Dpy, kc);
	XSync(Dsp, True);
	keyrel(Dsp, kc);
}

static	int	keyc = 0;

static unsigned int
keycode()
{
int	tmp;

	if(keyc == 0) {
		XDisplayKeycodes(Dsp, &keyc, &tmp);
        	if (keyc < 8)
	                keyc = 8;
	}
	return(keyc);
}    

static unsigned int
keystate(keyc)
int keyc;
{
XModifierKeymap *map;
unsigned int state;
int i, j;

	map = XGetModifierMapping(Dsp);
	if (!map)
		return(0);
	state = 0;
	for (i = 0; i < 8; i++) {
		for (j = 0; j < map->max_keypermod; j++) {
			if (map->modifiermap[(i * map->max_keypermod) + j] ==
			    keyc)
				state |= (ShiftMask << i);
		}
	}
	XFreeModifiermap(map);
	return(state);
}

int 	tet_thistest;

/*
 * Assertion KeyRelease-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * When any key is released, then a KeyRelease event is
 * generated.
 */
static void
t001()
{
Display		*display = Dsp;
Window		w;
XEvent		event_return;
XKeyEvent	good;
PointerPlace	*ptr;
int		kc;
int 	pass = 0, fail = 0;

	tpstartup();

	

	/* If extended testing is enabled: */
	if(noext(0))
		return;

		/* Create a window. */
	w = defwin(display);

		/* Select for xname events. */
	XSelectInput(display, w, EVENTMASK);

		/* Simulate a xname event on the window. */
	ptr = warppointer(display, w, 0,0);
	kc = keycode();
	presskey(display, w, kc);

		/* Verify that a xname event was delivered. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	} else {

		CHECK;

		good = event_return.xkey;
		good.type = EVENT;
		good.send_event = False;
		good.display = display;
		good.window = w;
		good.root = DRW(display);
		good.subwindow = None;
		good.x = 0;
		good.y = 0;
		good.x_root = ptr->nx;
		good.y_root = ptr->ny;
		good.state = keystate(kc);
		good.keycode = kc;
		good.same_screen = True;

		/* Verify that event member fields are correctly set. */
		if (checkevent((XEvent *) &good, &event_return)) {
			report("Unexpected values in delivered event");
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion KeyRelease-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a KeyRelease event is generated, then all clients
 * having set KeyReleaseMask event mask bits on the event
 * window are delivered a KeyRelease event.
 */
static void
t002()
{
Display			*display = Dsp;
Display			*client2;
Display			*client3;
Window			w;
XEvent			event_return;
XKeyReleasedEvent	good;
PointerPlace		*ptr;
int	 		kc;
int 	pass = 0, fail = 0;

	tpstartup();

		/* If extended testing is required: */
	if(config.extensions) {
	
		if(noext(0))
			return;

			/* Create a second client. */
		client2 = opendisplay();
			/* Create a third client. */
		client3 = opendisplay();

			/* Create a window. */
		w = defwin(display);

			/* Select for xname events on all clients. */
		XSelectInput(display, w, EVENTMASK);
		XSelectInput(client2, w, EVENTMASK);
		XSelectInput(client3, w, EVENTMASK);
	
			/* Simulate a xname event on the window. */
		ptr = warppointer(display, w, 0,0);
		kc = keycode();

		XSync(client2, True);
		XSync(client3, True);

		presskey(display, w, kc);

		XSync(client2, False);
		XSync(client3, False);
	
		good = event_return.xkey;
		good.type = EVENT;
		good.send_event = False;
		good.display = display;
		good.window = w;
		good.root = DRW(display);
		good.subwindow = None;
		good.x = 0;
		good.y = 0;
		good.x_root = ptr->nx;
		good.y_root = ptr->ny;
		good.state = keystate(kc);
		good.keycode = kc;
		good.same_screen = True;
	
			/* Verify that the event was delivered to all clients. */
			/* Verify that event member fields are correctly set. */
		if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
			report("Expected %s event, got none", eventname(EVENT));
			FAIL;
		} else {
			CHECK;
			if (checkevent((XEvent *) &good, &event_return)) {
				report("Unexpected values in delivered event");
				FAIL;
			} else
				CHECK;
		}	
/* client 2 */
		if (!XCheckTypedWindowEvent(client2, w, EVENT, &event_return)) {
			report("Expected %s event, got none", eventname(EVENT));
			FAIL;
		} else {
			CHECK;

			good.display = client2;
		
			if (checkevent((XEvent *) &good, &event_return)) {
				report("Unexpected values in delivered event");
				FAIL;
			} else
				CHECK;
		}
/* client 3 */

		if (!XCheckTypedWindowEvent(client3, w, EVENT, &event_return)) {
			report("Expected %s event, got none", eventname(EVENT));
			FAIL;
		} else {
			CHECK;

			good.display = client3;

			if (checkevent((XEvent *) &good, &event_return)) {
				report("Unexpected values in delivered event");
				FAIL;
			} else
				CHECK;
		}

		CHECKPASS(6);

	} else {

		/* Otherwise : */
			/* Create a window. */
		w = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
			/* Select for xname events. */
		XSelectInput(display, w, EVENTMASK);
		XSync(display, True);
			/* Verify no error was generated. */
		if (geterr() != Success) {
			delete("Error setting up for test.");
			return;
		}
		else
			CHECK;
	
		CHECKUNTESTED(1);
	}

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion KeyRelease-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a KeyRelease event is generated, then clients not
 * having set KeyReleaseMask event mask bits on the event
 * window are not delivered a KeyRelease event.
 */
static void
t003()
{
Display			*client2;
XEvent			ev;
Window			w;
int			count;
int			kc;
int 	pass = 0, fail = 0;

	tpstartup();

	/* If extended testing is required: */
	if(noext(0))
		return;

		/* Create a window. */
	w = defwin(Dsp);

		/* Create a second client. */
	client2 = opendisplay();

		/* Select for various events other that xname for the first client on the window. */
	kc = keycode();
 	XSelectInput(Dsp, w, KeyPressMask|ButtonPressMask|ButtonReleaseMask);

		/* Select for xname events for the second client on the window. */
	XSelectInput(client2, w, EVENTMASK);

		/* Simulate a xname event on the window. */
	(void) warppointer(Dsp, w, 0, 0);
	XSync(Dsp, True);
	XSync(client2, True);

	presskey(Dsp, w, kc);

	XSync(client2, False);

		/* Verify that a xname event was generated for the second client. */
	if(XCheckWindowEvent(client2, w, EVENTMASK, &ev) == False) {
		report("No %s event was generated.", eventname(EVENT));
		FAIL;
	} else
		CHECK;

		/* Verify no events were generated for the first client. */
	count = XPending(Dsp);
	if (count != 0) {
		report("Got %d unexpected events.", count);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion KeyRelease-4.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a KeyRelease event is generated and no client has
 * selected KeyReleaseMask on the source window, then the event
 * propagates, with propagation stopping at the root window of
 * the screen or at the first window with KeyReleaseMask in its
 * do-not-propagate mask, from the source window to the first
 * ancestor window for which some client has selected for
 * KeyRelease events.
 */
static void
t004()
{
XEvent			ev;
Window			w;
Window			w1;
Window			w2;
XSetWindowAttributes	atts;
int			kc;
int 	pass = 0, fail = 0;

	tpstartup();


	/* If extended testing is required: */
	if(noext(0))
		return;

		/* Create a window. */
	w = defwin(Dsp);
		/* Create a child of that window. */
	w1 = crechild(Dsp, w, (struct area *) NULL);
		/* Create a grandchild of the window. */
	w2 = crechild(Dsp, w1, (struct area *) NULL);

		/* Select xname events on the root window of the screen. */
	XSelectInput(Dsp, DRW(Dsp), EVENTMASK);	

		/* Simulate a xname event on the grandchild. */
	(void) warppointer(Dsp, w2, 1,1);
	kc = keycode();
	presskey(Dsp, w2, kc);
	
		/* Verify that a xname event was generated on the root. */
	if (XCheckWindowEvent(Dsp, DRW(Dsp), EVENTMASK, &ev) == False) {
		report("Expected event (%s) not received.", eventname(EVENT));
		FAIL;
	} else
		CHECK;

	XSelectInput(Dsp, DRW(Dsp), NoEventMask);	
		/* Select xname events on the grandparent window. */
	XSelectInput(Dsp, w, EVENTMASK);	

		/* Set the do_not_propagate mask on the child to xname events. */
	atts.do_not_propagate_mask = EVENTMASK;
	XChangeWindowAttributes(Dsp, w1, CWDontPropagate, &atts);
	
		/* Simulate a xname event on the grandchild. */
	presskey(Dsp, w2, kc);	

		/* Verify no xname event was generated on the grandchild. */
	if (XCheckWindowEvent(Dsp, w2, EVENTMASK, &ev) != False) {
		report("Unexpected event (%s) received.", eventname(EVENT));
		FAIL;
	} else
		CHECK;

		/* Verify no xname event was generated on the child. */
	if (XCheckWindowEvent(Dsp, w1, EVENTMASK, &ev) != False) {
		report("Unexpected event (%s) received.", eventname(EVENT));
		FAIL;
	} else
		CHECK;

		/* Verify no xname event was generated on the parent. */
	if (XCheckWindowEvent(Dsp, w, EVENTMASK, &ev) != False) {
		report("Unexpected event (%s) received.", eventname(EVENT));
		FAIL;
	} else
		CHECK;


		/* Select xname events on the child window. */
	XSelectInput(Dsp, w1, EVENTMASK);	
		/* Set the do_not_propagate mask of the grandparent to xname events. */
	XChangeWindowAttributes(Dsp, w, CWDontPropagate, &atts);

		/* Set the do_not_propagate mask of the child to NoEventMask. */
	atts.do_not_propagate_mask = NoEventMask;
	XChangeWindowAttributes(Dsp, w1, CWDontPropagate, &atts);

		/* Simulate a xname event on the grandchild. */
	presskey(Dsp, w2, kc);

		/* Verify that no xname event was generated on the grandchild. */
	if (XCheckWindowEvent(Dsp, w2, EVENTMASK, &ev) != False) {
		report("Unexpected event (%s) received.", eventname(EVENT));
		FAIL;
	} else
		CHECK;

		/* Verify that no xname event was generated on the parent. */
	if (XCheckWindowEvent(Dsp, w, EVENTMASK, &ev) != False) {
		report("Unexpected event (%s) received.", eventname(EVENT));
		FAIL;
	} else
		CHECK;

		/* Verify that a xname event was generated on the child. */
	if (XCheckWindowEvent(Dsp, w1, EVENTMASK, &ev) == False) {
		report("KeyRelease event was not delivered to selecting child window.");
		FAIL;
	} else
		CHECK;

	CHECKPASS(7);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion KeyRelease-5.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a KeyRelease event is delivered and the source window
 * is an inferior of the event window and the source window is
 * a child of the event window, then subwindow is set to the
 * source window.
 */
static void
t005()
{
XEvent		ev;
Window		w;
Window		w1;
int		kc;
int 	pass = 0, fail = 0;

	tpstartup();


	/* If extended testing is required: */
	if(noext(0))
		return;

		/* Create a window. */
	w = defwin(Dsp);
		/* Create a child of the window */
	w1 = crechild(Dsp, w, (struct area *) NULL);
		/* Select xname events on the parent. */
	XSelectInput(Dsp, w, EVENTMASK);	
	(void) warppointer(Dsp, w1, 1,1);
	
		/* Simulate a xname event on the child. */
	kc=keycode();
	presskey(Dsp, w1, kc);

		/* Verify that a xname event was generated on the parent. */
	if (XCheckWindowEvent(Dsp, w, EVENTMASK, &ev) == False) {
		report("Expected event (%s) not received.", eventname(EVENT));
		FAIL;
	} else {
		CHECK;
		/* Verify that the subwindow component was set to the child. */
		if(ev.xkey.subwindow != w1) {
			report("The subwindow component of the %s name was not set correctly.", eventname(EVENT));
			FAIL;
		} else 
			CHECK;
	}
	
	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion KeyRelease-6.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a KeyRelease event is delivered and the source window
 * is an inferior of the event window and the source window is
 * not a child of the event window, then subwindow is set to
 * the child of the event window that is an ancestor of the
 * source window.
 */
static void
t006()
{
Window		w;
Window		w1;
Window		w2;
Window		w3;
XEvent		ev;
struct area	area;
int		kc;
int 	pass = 0, fail = 0;

	tpstartup();

	/* If extended testing is required : */
	if (noext(0))
		return;
		
		/* Create a window. */	
	w = defwin(Dsp);
	setarea(&area, 10, 10, 30, 30);
		/* Create a child. */
	w1 = crechild(Dsp, w, &area);
		/* Create a grandchild. */
	w2 = crechild(Dsp, w1, &area);
		/* Create a great-grandchild. */
	w3 = crechild(Dsp, w2, &area);
	(void) warppointer(Dsp, w3, 1,1);

		/* Select xname events on the great-grandparent. */
	XSelectInput(Dsp, w, EVENTMASK);

		/* Simulate a xname event on the great-grandchild. */
	kc = keycode();
	presskey(Dsp, w3, kc);
	
		/* Verify that a xname event was generated on the great-grandparent. */
	if (XCheckWindowEvent(Dsp, w, EVENTMASK, &ev) == False) {
		report("Expected event (%s) not received.", eventname(EVENT));
		FAIL;
	} else {
		CHECK;
		/* Verify that subwindow component of the event was the child. */
		if(ev.xkey.subwindow != w1) {
			report("The subwindow component of the %s event was not set correctly.", eventname(EVENT));
			FAIL;
		} else 
			CHECK;
	}

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion KeyRelease-7.(B)
 *   Reason: There is no known portable test method for this assertion
 * When a KeyRelease event is delivered and the source window
 * is not an inferior of the event window, then subwindow is
 * set to None.
 */
static void
t007()
{
Window		w;
Window		w2;
XEvent		ev;
int		gr;
int		kc;
int 	pass = 0, fail = 0;

	tpstartup();

	/* If extended testing is required: */
	if (noext(0))
		return;

		/* Create a window. */
        w = defwin(Dsp);
	(void) warppointer(Dsp, w, 1,1);

		/* Select xname events on the window. */
	XSelectInput(Dsp, w, EVENTMASK);

		/* Simulate a xname event on the window. */
	kc=keycode();
	presskey(Dsp, w, kc);
	
		/* Verify that a xname event was generated on the window. */
	if (XCheckWindowEvent(Dsp, w, EVENTMASK, &ev) == False) {
		report("Expected event (%s) not received.", eventname(EVENT));
		FAIL;
	} else {
		CHECK;
		/* Verify that the subwindow component was set to None. */
		if(ev.xkey.subwindow != None) {
			report("The subwindow component of the %s event was not set correctly.", eventname(EVENT));
			FAIL;
		} else 
			CHECK;
	}

		/* Create a second window. */
	w2 = defwin(Dsp);

		/* Grab the keyboard for the first window with owner_events set to False. */
	if( (gr=XGrabKeyboard(Dsp, w, False, GrabModeAsync, GrabModeAsync, CurrentTime)) != GrabSuccess) {
		delete("XGrabKeyboard() returned %s instead of GrabSuccess.", grabreplyname(gr));
		return;
	} else
		CHECK;

		/* Simulate a xname event on the second window. */
	(void) warppointer(Dsp, w2, 1,1);
	presskey(Dsp, w2, kc);
	
		/* Verify that a xname event was generated on the grabbing window. */
	if (XCheckWindowEvent(Dsp, w, EVENTMASK, &ev) == False) {
		report("Expected event (%s) not received.", eventname(EVENT));
		FAIL;
	} else {
		CHECK;
		/* Verify that the subwindow component was set to None. */
		if(ev.xkey.subwindow != None) {
			report("The subwindow component of the %s event was not set correctly.", eventname(EVENT));
			FAIL;
		} else 
			CHECK;
	}

	XUngrabKeyboard(Dsp, CurrentTime);

	CHECKPASS(5);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion KeyRelease-8.(D)
 *   Reason: There is no known portable test method for this assertion
 * If multiple screens are supported: When a KeyRelease event
 * is delivered and the event and root windows are not on the
 * same screen, then x and y are set to zero.
 */
static void
t008()
{
Window		w;
Window		w2;
XEvent		ev;
int		gr;
int		kc;
int 	pass = 0, fail = 0;

	tpstartup();


	/* If multiple screens are supported: */
	if (config.alt_screen == -1) {
		unsupported("Multiple screens not supported.");
		return;
	} else
		CHECK;

		/* If extended testing is required: */
	if(noext(0))
		return;
		
			/* Create a window on the default screen. */
        w = defwin(Dsp);
	
			/* Select xname events on the window. */
	(void) warppointer(Dsp, w, 1,1);
	XSelectInput(Dsp, w, EVENTMASK);
	
			/* Create a window on the alternative screen. */
	w2 = defdraw(Dsp, VI_ALT_WIN);
	
			/* Grab the keyboard for the first window. */
	if( (gr=XGrabKeyboard(Dsp, w, False, GrabModeAsync, GrabModeAsync, CurrentTime)) != GrabSuccess) {
		delete("XGrabKeyboard() returned %s instead of GrabSuccess.", grabreplyname(gr));
		return;
	} else
		CHECK;
	
	kc=keycode();
			/* Simulate a xname event on the alternate window. */
	(void) warppointer(Dsp, w2, 1,1);
	presskey(Dsp, w2, kc);
	
			/* Verify that a xname event was generated on the grabbing window. */
	if (XCheckWindowEvent(Dsp, w, EVENTMASK, &ev) == False) {
		report("Expected event (%s) not received.", eventname(EVENT));
		FAIL;
	} else {
		CHECK;
			/* Verify that the x and y components were set to zero. */
		if(ev.xkey.x != 0 || ev.xkey.y != 0) {
			report("The x (value %d) and y (value %d) components of the %s event were not set to zero.", ev.xkey.x, ev.xkey.y, eventname(EVENT));
			FAIL;
		} else 
			CHECK;
	}
	
	XUngrabKeyboard(Dsp, CurrentTime);
	
	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion KeyRelease-9.(D)
 *   Reason: There is no known portable test method for this assertion
 * If multiple screens are supported: When a KeyRelease event
 * is delivered and the event and root windows are not on the
 * same screen, then same_screen is set to False.
 */
static void
t009()
{
Window		w;
Window		w2;
XEvent		ev;
int		gr;
int		kc;
int 	pass = 0, fail = 0;

	tpstartup();

	/* If multiple screens are supported: */
	if (config.alt_screen == -1) {
		unsupported("Multiple screens not supported.");
		return;
	} else
		CHECK;

		/* If extended testing is required: */
	if(noext(0))
		return;

			/* Create a window on the default screen. */
        w = defwin(Dsp);
	
	(void) warppointer(Dsp, w, 1,1);
			/* Select xname events on the window. */
	XSelectInput(Dsp, w, EVENTMASK);
	
			/* Create a window on the alternative screen. */
	w2 = defdraw(Dsp, VI_ALT_WIN);
	
			/* Grab the keyboard for the first window. */
	if( (gr=XGrabKeyboard(Dsp, w, False, GrabModeAsync, GrabModeAsync, CurrentTime)) != GrabSuccess) {
		delete("XGrabKeyboard() returned %s instead of GrabSuccess.", grabreplyname(gr));
		return;
	} else
		CHECK;
	
	kc=keycode();
			/* Simulate a xname event on the alternate window. */
	(void) warppointer(Dsp, w2, 1,1);
	presskey(Dsp, w2, kc);
	
			/* Verify that a xname event was generated on the grabbing window. */
	if (XCheckWindowEvent(Dsp, w, EVENTMASK, &ev) == False) {
		report("Expected event (%s) not received.", eventname(EVENT));
		FAIL;
	} else {
		CHECK;
			/* Verify that the same_screen component was False. */
		if(ev.xkey.same_screen != False) {
			report("The same_screen component of the %s event was not set to False.", eventname(EVENT));
			FAIL;
		} else 
			CHECK;
	}
	
	XUngrabKeyboard(Dsp, CurrentTime);
	
	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = focusstartup;
void	(*tet_cleanup)() = focuscleanup;
