/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: mpntfy.m,v 1.8 92/06/11 17:22:54 rws Exp $
 * 
 */
/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_MapNotify	1
char    *TestName = "MapNotify";



#define	EVENT		MapNotify
#define	MASK		StructureNotifyMask
#define	MASKP		SubstructureNotifyMask

int 	tet_thistest;

/*
 * Assertion MapNotify-1.(A)
 * When a window's state changes from unmapped to mapped as a
 * result of save-set processing, then a MapNotify event is
 * generated.
 */
static void
t001()
{
Display	*display = Dsp;
Display	*client2;
Window	child;
Window	parent;
XEvent	event_return;
XMapEvent good;
struct area area;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create client2. */
	if (config.display == (char *) NULL) {
		delete("config.display not set");
		return;
	}
	else
		CHECK;
	client2 = XOpenDisplay(config.display);
	if (client2 == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
/* Create parent window with client2. */
	/* must avoid resource registration! */
	parent = XCreateSimpleWindow(display, DRW(display), 1, 1, W_STDWIDTH, W_STDHEIGHT, 1, 0L, 0L);
	XSync(client2, True);
/* Create child window as inferior to parent. */
	area.x = 1;
	area.y = 1;
	area.width = W_STDWIDTH/2;
	area.height = W_STDHEIGHT/2;
	child = mkwinchild(display, (XVisualInfo *) NULL, &area, False, parent, 1);
	XSync(display, True);
/* Add child window to client2's save-set. */
	XAddToSaveSet(client2, child);
/* Set client2's shut down mode to DestroyAll. */
	XSetCloseDownMode(client2, DestroyAll);
/* Select for MapNotify events on child. */
	XSelectInput(display, child, MASK);
/* Call XCloseDisplay with client2. */
	XSync(display, True);
	XSync(client2, True);
	XCloseDisplay(client2);

/* Allow time for the client to close. */
	reset_delay();

	XSync(display, False);
/* Verify that a MapNotify event was delivered. */
	if (!XCheckTypedWindowEvent(display, child, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
/* Verify that event member fields are correctly set. */
	good = event_return.xmap;
	good.type = EVENT;
	good.event = child;
	good.window = child;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event");
		FAIL;
	}
	else
		CHECK;

	CHECKPASS(4);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion MapNotify-2.(A)
 * When a MapNotify event is generated, then all clients
 * having set StructureNotifyMask event mask bits on the mapped
 * window are delivered a MapNotify event.
 */
static void
t002()
{
Display	*display = Dsp;
Display	*client2;
Display	*client3;
Window	w;
int	count;
XEvent	event_return;
XMapEvent good;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create clients client2 and client3. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
	if ((client3 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client3.");
		return;
	}
	else
		CHECK;
/* Create window. */
	w = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
/* Select for MapNotify events using StructureNotifyMask. */
	XSelectInput(display, w, MASK);
/* Select for MapNotify events using StructureNotifyMask with client2. */
	XSelectInput(client2, w, MASK);
/* Select for no events with client3. */
	XSelectInput(client3, w, NoEventMask);
/* Generate MapNotify event. */
	XSync(display, True);
	XSync(client2, True);
	XSync(client3, True);
	XMapWindow(display, w);
	XSync(display, False);
	XSync(client2, False);
	XSync(client3, False);
/* Verify that a MapNotify event was delivered. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(display, w, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xmap;
	good.type = EVENT;
	good.event = w;
	good.window = w;
	good.override_redirect = config.debug_override_redirect;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event");
		FAIL;
	}
	else
		CHECK;
/* Verify that a MapNotify event was delivered to client2. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(client2, w, EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xmap;
	good.type = EVENT;
	good.event = w;
	good.window = w;
	good.override_redirect = config.debug_override_redirect;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event for client2");
		FAIL;
	}
	else
		CHECK;
/* Verify that no events were delivered to client3. */
	count = XPending(client3);
	if (count != 0) {
		report("Got %d events, expected %d for client3", count, 0);
		FAIL;
		return;
	}
	else
		CHECK;
	
	CHECKPASS(7);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion MapNotify-3.(A)
 * When a MapNotify event is generated, then all clients
 * having set SubstructureNotifyMask event mask bits on the
 * parent of the mapped window are delivered a MapNotify event.
 */
static void
t003()
{
Display	*display = Dsp;
Display	*client2;
Display	*client3;
Window	w;
int	count;
XEvent	event_return;
XMapEvent good;
int 	pass = 0, fail = 0;

	tpstartup();

/* Create clients client2 and client3. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
	if ((client3 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client3.");
		return;
	}
	else
		CHECK;
/* Create window. */
	w = mkwin(display, (XVisualInfo *) NULL, (struct area *) NULL, False);
/* Select for MapNotify events using SubstructureNotifyMask. */
	XSelectInput(display, DRW(display), MASKP);
/* Select for MapNotify events using SubstructureNotifyMask with client2. */
	XSelectInput(client2, DRW(display), MASKP);
/* Select for no events with client3. */
	XSelectInput(client3, DRW(display), NoEventMask);
/* Generate MapNotify event. */
	XSync(display, True);
	XSync(client2, True);
	XSync(client3, True);
	XMapWindow(display, w);
	XSync(display, False);
	XSync(client2, False);
	XSync(client3, False);
/* Verify that a MapNotify event was delivered. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(display, DRW(display), EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xmap;
	good.type = EVENT;
	good.event = DRW(display);
	good.window = w;
	good.override_redirect = config.debug_override_redirect;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event");
		FAIL;
	}
	else
		CHECK;
/* Verify that a MapNotify event was delivered to client2. */
/* Verify that event member fields are correctly set. */
	if (!XCheckTypedWindowEvent(client2, DRW(display), EVENT, &event_return)) {
		report("Expected %s event, got none", eventname(EVENT));
		FAIL;
	}
	else
		CHECK;
	good = event_return.xmap;
	good.type = EVENT;
	good.event = DRW(display);
	good.window = w;
	good.override_redirect = config.debug_override_redirect;
	if (checkevent((XEvent *) &good, &event_return)) {
		report("Unexpected values in delivered event for client2");
		FAIL;
	}
	else
		CHECK;
/* Verify that no events were delivered to client3. */
	count = XPending(client3);
	if (count != 0) {
		report("Got %d events, expected %d for client3", count, 0);
		FAIL;
		return;
	}
	else
		CHECK;
	
	CHECKPASS(7);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion MapNotify-4.(-)
 * When a MapNotify event is generated, then clients not
 * having set StructureNotifyMask event mask bits on the mapped
 * window and also not having set SubstructureNotifyMask event
 * mask bits on the parent of the mapped window are not
 * delivered a MapNotify event.
 */
static void
t004()
{
	report("The assertion is descriptive or is tested elsewhere.");
	tet_result(TET_NOTINUSE);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
