/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: synchrnz.m,v 1.6 92/06/11 17:26:07 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int ((*)())()
 *   XSynchronize(display, onoff)
 *   Display *display;
 *   Bool onoff;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSynchronize	1
char    *TestName = "XSynchronize";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XSynchronize function
 */
static Display *display;
static Bool onoff;


static int 	ValueReturn;

static int
afterfunction(display)
Display *display;
{
	return((int) display);
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	onoff = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSynchronize-1.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XSynchronize with onoff set to True turns on
 * synchronous behavior.
 */
static void
t001()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XSynchronize-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * A call to XSynchronize with onoff set to False turns off
 * synchronous behavior.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/*
 * Assertion XSynchronize-3.(A)
 * A call to XSynchronize with onoff set to True sets the
 * after function to a non-NULL value.
 */
static void
t003()
{
int	(*proc)();
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call XSynchronize with onoff set to True. */
	onoff = True;
	startcall(display);
	if (isdeleted())
		return;
	(void) XSynchronize(display, onoff);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Call XSetAfterFunction to get value of old after function. */
	proc = XSetAfterFunction(display, afterfunction);
/* Verify that XSetAfterFunction returned non-NULL. */
	if (proc == (int (*)()) NULL) {
		report("Returned NULL, expected non-NULL.");
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSynchronize-4.(A)
 * A call to XSynchronize with onoff set to False sets the
 * after function to NULL.
 */
static void
t004()
{
int	(*proc)();
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call XSynchronize with onoff set to False. */
	onoff = False;
	startcall(display);
	if (isdeleted())
		return;
	(void) XSynchronize(display, onoff);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Call XSetAfterFunction to get value of old after function. */
	proc = XSetAfterFunction(display, afterfunction);
/* Verify that XSetAfterFunction returned NULL. */
	if (proc != (int (*)()) NULL) {
		report("Returned non-NULL, expected NULL.");
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(1);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSynchronize-5.(A)
 * A call to XSynchronize returns the previous after function.
 */
static void
t005()
{
int	(*proc)();
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call XSetAfterFunction to set after function to afterfunction. */
	(void) XSetAfterFunction(display, afterfunction);
/* Call XSynchronize with onoff set to False. */
	onoff = False;
	startcall(display);
	if (isdeleted())
		return;
	proc = XSynchronize(display, onoff);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify that XSynchronize returned afterfunction. */
	if (proc != afterfunction) {
		report("Did not return previous after function.");
		FAIL;
	}
	else
		CHECK;
/* Call XSetAfterFunction to set after function to afterfunction. */
	(void) XSetAfterFunction(display, afterfunction);
/* Call XSynchronize with onoff set to True. */
	onoff = True;
	startcall(display);
	if (isdeleted())
		return;
	proc = XSynchronize(display, onoff);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
/* Verify that XSynchronize returned afterfunction. */
	if (proc != afterfunction) {
		report("Did not return previous after function.");
		FAIL;
	}
	else
		CHECK;
	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
