/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gticnnm.m,v 1.7 92/06/11 17:27:59 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XGetIconName(display, w, icon_name_return)
 *   Display	*display;
 *   Window	w;
 *   char	**icon_name_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetIconName	1
char    *TestName = "XGetIconName";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XGetIconName function
 */
static Display	*display;
static Window	w;
static char	**icon_name_return;


static int 	StatusReturn;

#include	"Xatom.h"
char	*iconname = "XtestIC";

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	icon_name_return = &iconname;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetIconName-1.(A)
 * When the WM_ICON_NAME property for the window w has been
 * set, and has format 8 and type STRING, then a call to
 * XGetIconName returns that name, which can be freed with
 * XFree, in the icon_name_return argument and returns
 * non-zero.
 */
static void
t001()
{
Window	win;
char	*iconname = "Xtest Icon Name";
char	*iconnameret;
Status	status;
char	**list_return;
int	count_return;
XTextProperty	tp, rtp;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);

	XSetIconName(display, win, iconname);

	w = win;
	icon_name_return = &iconnameret;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetIconName(display, w, icon_name_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == False) {
		report("XGetIconName() returned False.");
		FAIL;
	} else
		CHECK;

	if(strcmp(iconname, (iconnameret != NULL) ? iconnameret : "NULL") != 0) {
		report("XGetIconName() returned \"%s\" instead of \"%s\"", iconnameret, iconname);
		FAIL;
	} else
		CHECK;

	if(iconnameret != NULL) 
		XFree(iconnameret);

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetIconName-2.(A)
 * When the WM_ICON_NAME property for the window w has not
 * been set, or has format other than 8 or type other than
 * STRING, then a call to XGetIconName sets the string in the
 * icon_name_return argument to NULL and returns zero.
 */
static void
t002()
{
char	*iconnameret = "XTestUninit";
char	*name = "XTestIconName";
Status	status;
char	**list_return;
int	count_return;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);

/* Property unset */

	icon_name_return = &iconnameret;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetIconName(display, w, icon_name_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("XGetIconName() did not return False when the WM_ICON_NAME property was unset.");
		FAIL;
	} else
		CHECK;

	if( iconnameret != NULL) {
		report("XGetIconName() returned \"%s\" instead of NULL when the WM_ICON_NAME was unset.", iconnameret);
		FAIL;
	} else
		CHECK;

	w = makewin(display, vp);

/* format 16 */
	XChangeProperty(display, w, XA_WM_ICON_NAME, XA_STRING, 16, PropModeReplace, (unsigned char *) name, strlen(name));

	startcall(display);
	if (isdeleted())
		return;
	status = XGetIconName(display, w, icon_name_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("XGetIconName() did not return False when the WM_ICON_NAME property format was 16.");
		FAIL;
	} else
		CHECK;

	if( iconnameret != NULL) {
		report("XGetIconName() returned \"%s\" instead of NULL when the WM_ICON_NAME property format was 16.", iconnameret);
		FAIL;
	} else
		CHECK;

	w = makewin(display, vp);
/* type XA_ATOM */
	XChangeProperty(display, w, XA_WM_ICON_NAME, XA_ATOM, 8, PropModeReplace, (unsigned char *) name, strlen(name));
	startcall(display);
	if (isdeleted())
		return;
	status = XGetIconName(display, w, icon_name_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("XGetIconName() did not return False when the WM_ICON_NAME property type was ATOM.");
		FAIL;
	} else
		CHECK;

	if( iconnameret != NULL) {
		report("XGetIconName() returned \"%s\" instead of NULL when the WM_ICON_NAME property type was ATOM", iconnameret);
		FAIL;
	} else
		CHECK;

	CHECKPASS(6);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XGetIconName-3.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XGetIconName(display, w, icon_name_return);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
