/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gttrnsntfr.m,v 1.7 92/06/11 17:28:21 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XGetTransientForHint(display, w, prop_window_return)
 *   Display	*display;
 *   Window	w;
 *   Window	*prop_window_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetTransientForHint	1
char    *TestName = "XGetTransientForHint";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XGetTransientForHint function
 */
static Display	*display;
static Window	w;
static Window	*prop_window_return;


static int 	StatusReturn;

#include "Xatom.h"
Window	propwindow;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	prop_window_return = &propwindow;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetTransientForHint-1.(A)
 * When the WM_TRANSIENT_FOR property is set for the window w,
 * and is format 32, length 1 element and of type WINDOW, then
 * a call to XGetTransientForHint returns the property value in
 * the prop_window_return argument and returns non-zero.
 */
static void
t001()
{
Status		status;
Window		win, twin;
Window		rwin = (Window) -1;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);
	twin = makewin(display, vp);

	XSetTransientForHint(display, win, twin);

	w = win;
	prop_window_return = &rwin;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetTransientForHint(display, w, prop_window_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == False) {
		report("XGetTransientForHint() returned False.");
		FAIL;
	} else
		CHECK;

	if(rwin != twin) {
		report("The value of the WM_TRANSIENT_FOR property was %lx instead of %lx.", (long) rwin, (long) twin);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetTransientForHint-2.(A)
 * When the WM_TRANSIENT_FOR property is not set for the
 * window w, or is format other than 32, length < 1 element or
 * of type other than WINDOW, then a call to
 * XGetTransientForHint returns zero.
 */
static void
t002()
{
Status		status;
XVisualInfo	*vp;
Window		pwin;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vp);

	prop_window_return = &pwin;
	pwin = makewin(display, vp);	
	w = makewin(display, vp);

/* Property unset */

	startcall(display);
	if (isdeleted())
		return;
	status = XGetTransientForHint(display, w, prop_window_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("%s() did not return False when the WM_TRANSIENT_FOR property was not set.", TestName);
		FAIL;
	} else
		CHECK;

	w = makewin(display, vp);

/* Format 16 */
	XChangeProperty(display, w, XA_WM_TRANSIENT_FOR, XA_WINDOW, 16, PropModeReplace, (unsigned char *) &pwin, 1);

	startcall(display);
	if (isdeleted())
		return;
	status = XGetTransientForHint(display, w, prop_window_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("%s() did not return False when the WM_TRANSIENT_FOR property format was 16.", TestName);
		FAIL;
	} else
		CHECK;

	w = makewin(display, vp);

/*  Type STRING */
	XChangeProperty(display, w, XA_WM_TRANSIENT_FOR, XA_STRING, 32, PropModeReplace, (unsigned char *) &pwin, 1);

	startcall(display);
	if (isdeleted())
		return;
	status = XGetTransientForHint(display, w, prop_window_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("%s() did not return False when the WM_TRANSIENT_FOR property type was STRING.", TestName);
		FAIL;
	} else
		CHECK;

	w = makewin(display, vp);

/* No elements = 0 */
	XChangeProperty(display, w, XA_WM_TRANSIENT_FOR, XA_WINDOW, 32, PropModeReplace, (unsigned char *) &pwin, 0);

	startcall(display);
	if (isdeleted())
		return;
	status = XGetTransientForHint(display, w, prop_window_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("%s() did not return False when the WM_TRANSIENT_FOR property size  was 0.", TestName);
		FAIL;
	} else
		CHECK;

	CHECKPASS(4);


	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EWin.mc */
/*
 * Assertion XGetTransientForHint-3.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XGetTransientForHint(display, w, prop_window_return);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
