/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gttxtprprt.m,v 1.10 92/06/11 17:28:28 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XGetTextProperty(display, w, text_prop_return, property)
 *   Display		*display;
 *   Window		w;
 *   XTextProperty	*text_prop_return;
 *   Atom		property;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetTextProperty	1
char    *TestName = "XGetTextProperty";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w
#define A_ATOM property


/*
 * Arguments to the XGetTextProperty function
 */
static Display		*display;
static Window		w;
static XTextProperty	*text_prop_return;
static Atom		property;


static int 	StatusReturn;

#include	"Xatom.h"
static XTextProperty	textprop = { (unsigned char *) 0, XA_STRING, 8, (unsigned long) 0 };

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	text_prop_return = &textprop;
	property = XA_WM_NAME;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetTextProperty-1.(A)
 * When the property specified by the property argument exists
 * on the window specified by the w argument, then a call to
 * XGetTextProperty stores the data, which can be freed with
 * XFree, in the value field, the type of the data in the
 * encoding field, the format of the data in the format field,
 * and the number of items of data in the nitems field of the
 * XTextProperty structure named by the text_prop_return
 * argument and returns non-zero.
 */
static void
t001()
{
Window	win;
char	*str1 = "Xtest test string1";
char	*str2 = "Xtest test string2";
char	*str[2];
Status	status;
char	**list_return;
int	count_return;
XTextProperty	tp, rtp;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	str[0] = str1;
	str[1] = str2;

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);

	if(XStringListToTextProperty(str, 2, &tp) == False) {
		delete("XStringListToTextProperty() Failed.");
		return;
	} else 
		CHECK;

	XSetTextProperty(display, win, &tp, property);

	text_prop_return = &rtp;
	w = win;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetTextProperty(display, w, text_prop_return, property);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == False) {
		report("XGetTextProperty() returned False.");
		FAIL;
	} else
		CHECK;

	if(rtp.encoding != XA_STRING) {
		report("Property was not of type STRING");
		FAIL;
	} else
		CHECK;

	if(rtp.format != 8) {
		report("Format was %d instead of 8", rtp.format);
		FAIL;
	} else
		CHECK;

	if(rtp.nitems != strlen(str1) + strlen(str2) + 1) {
		report("The nitems component was %lu instead of %d", rtp.nitems,1 +  strlen(str1) + strlen(str2) );
		FAIL;
	} else
		CHECK;

	if(XTextPropertyToStringList( &rtp, &list_return, &count_return) == False) {
		delete("XTextPropertyToStringList() returned False.");
		return;
	} else
		CHECK;

	if (count_return != 2) {
		delete("count_return was %d instead of 2.", count_return);
		return;
	} else
		CHECK;

	if( (strncmp(str1, list_return[0], strlen(str1)) != 0) ||
		(strncmp(str2, list_return[1], strlen(str2)) != 0) ) {

		report("Value strings were:");
		report("\"%s\" and \"%s\"", list_return[0], list_return[1]);
		report("Instead of:");
		report("\"%s\" and \"%s\"", str1, str2);
		FAIL;
	} else
		CHECK;

	XFree((char*)tp.value);
	XFree((char*)rtp.value);
	XFreeStringList(list_return);
	CHECKPASS(8);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetTextProperty-2.(A)
 * When the property specified by the property argument does
 * not exist on the window specified by the w argument, then a
 * call to XGetTextProperty sets the value field to NULL, the
 * encoding field to None, the format field to 0 and the nitems
 * field to 0 of the XTextProperty structure named by the
 * text_prop_return argument and returns zero.
 */
static void
t002()
{
Window	win;
Status	status;
char	**list_return;
int	count_return;
XTextProperty	tp;
XVisualInfo	*vp;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	tp.value = (unsigned char *) "XTuninit";
	tp.format = 1;
	tp.encoding = (Atom) 1;
	tp.nitems = (unsigned long) 1;

	resetvinf(VI_WIN);
	nextvinf(&vp);
	win = makewin(display, vp);

	text_prop_return = &tp;
	w = win;
	startcall(display);
	if (isdeleted())
		return;
	status = XGetTextProperty(display, w, text_prop_return, property);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status != False) {
		report("XGetTextProperty() did not return False.");
		FAIL;
	} else
		CHECK;

	if(tp.encoding != None) {
		report("Property was not of type None");
		FAIL;
	} else
		CHECK;
	
	if(tp.format != 0) {
		report("The format component was %d instead of 0", tp.format);
		FAIL;
	} else
		CHECK;

	if(tp.nitems != 0) {
		report("The nitems component was %lu instead of 0", tp.nitems);
		FAIL;
	} else
		CHECK;

	if(tp.value != (unsigned char *) NULL) {
		report("The value component was not NULL."); 
		FAIL;
	} else
		CHECK;

	CHECKPASS(5);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAto.mc */
/*
 * Assertion XGetTextProperty-3.(A)
 * When an atom argument does not name a valid Atom, then a
 * BadAtom error occurs.
 */
/* End of included file error/EAto.mc */

/* Including from file error/EAto.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	A_ATOM = ~0L;
	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XGetTextProperty(display, w, text_prop_return, property);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadAtom) {
		report("Got %s, Expecting BadAtom", errorname(geterr()));
		FAIL;
	}

	if(geterr() == BadAtom)
		PASS;
	else
		FAIL;
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAto.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XGetTextProperty-4.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XGetTextProperty(display, w, text_prop_return, property);
	endcall(display);
	if (StatusReturn != 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
