/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: stwmprtcls.m,v 1.6 92/06/11 17:32:09 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Status
 *   XSetWMProtocols(display, w, protocols, count)
 *   Display	*display;
 *   Window	w;
 *   Atom	*protocols;
 *   int 	count;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetWMProtocols	1
char    *TestName = "XSetWMProtocols";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XSetWMProtocols function
 */
static Display	*display;
static Window	w;
static Atom	*protocols;
static int 	count;


static int 	StatusReturn;

#include	"Xatom.h"
Atom	prots;

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = DRW(Dsp);
	protocols = &prots;
	count = 1;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XSetWMProtocols-1.(A)
 * A call to XSetWMProtocols sets the WM_PROTOCOLS property
 * for the window w, to be of type ATOM, format 32 and to
 * have value set to the list of count atoms specified by the
 * protocols argument and returns non-zero.
 */
static void
t001()
{
Status		status;
XVisualInfo	*vp;
unsigned long	leftover, nitems, len;
int		actual_format;
Atom		actual_type;
int		nats = 5;
Atom		xa_wm_protocols;
Atom		prots[5], *rprots = (Atom *) NULL;
Atom		at, *atp;
int		i;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	for(i=0, at = XA_LAST_PREDEFINED; i<nats; i++)
		prots[i] = (int) --at;

	resetvinf(VI_WIN);
	nextvinf(&vp);
	w = makewin(display, vp);

	protocols = prots;
	count = nats;
	startcall(display);
	if (isdeleted())
		return;
	status = XSetWMProtocols(display, w, protocols, count);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(status == 0) {
		report("%s() returned zero.", TestName);
		FAIL;
	} else
		CHECK;

	if((xa_wm_protocols = XInternAtom(display, "WM_PROTOCOLS", True)) == None) {
		delete("The \"WM_PROTOCOLS\" string was not interned.");
		return;
	} else
		CHECK;


	if (XGetWindowProperty(display, w, xa_wm_protocols,
                            0L, (long) nats, False,
                            AnyPropertyType, &actual_type, &actual_format,
                            &nitems, &leftover, (unsigned char **) &rprots) != Success) {
		delete("XGetWindowProperty() did not return Success.");
		return;
	} else
		CHECK;

	if(leftover != 0) {
		report("The leftover elements numbered %lu instead of 0", leftover);
		FAIL;
	} else
		CHECK;

	if(actual_format != 32) {
		report("The format of the WM_PROTOCOLS property was %lu instead of 32", actual_format);
		FAIL;
	} else
		CHECK;

	if(actual_type != XA_ATOM) {
		report("The type of the WM_PROTOCOLS property was %lu instead of ATOM (%lu)", actual_type, (long) XA_ATOM);
		FAIL;
	} else
		CHECK;

	if( rprots == (Atom *) NULL) {
		report("No value was set for the WM_PROTOCOLS property.");
		FAIL;
	} else {

		CHECK;		
		if( nitems != nats) {
			report("The WM_PROTOCOLS property comprised %ul elements instead of %d", nitems, nats);
			FAIL;
		} else {

			if(actual_format == 32) {

				CHECK;
				for(i = 0, atp = rprots; i<nats; i++, atp++)
					if( *atp != prots[i]) {
						report("Element %d of the WM_PROTOCOLS value was %lu instead of %lu", i+1, (long) *atp, (long) prots[i]);
						FAIL;
					} else
						CHECK;
			}
		}
		XFree((char*)rprots);
	}

	CHECKPASS(8 + nats);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XSetWMProtocols-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the atom name (lqWM_PROTOCOLS(rq cannot be interned,
 * then a call to XSetWMProtocols returns zero.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* Including from file error/EAll.mc */
/*
 * Assertion XSetWMProtocols-3.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* Including from file error/EWin.mc */
/*
 * Assertion XSetWMProtocols-4.(A)
 * When a window argument does not name a valid Window, then a
 * BadWindow error occurs.
 */
/* End of included file error/EWin.mc */

/* Including from file error/EWin.mc */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();

	A_WINDOW = badwin(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	StatusReturn = XSetWMProtocols(display, w, protocols, count);
	endcall(display);
	if (StatusReturn == 0) {
		report("Status returned was %d", StatusReturn);
		FAIL;
	}
	if (geterr() != BadWindow) {
		report("Got %s, Expecting BadWindow", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadWindow)
		PASS;
	else
		FAIL;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EWin.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
