/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission. MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose. It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: wmgmtry.m,v 1.5 92/06/11 17:32:31 rws Exp $
 * 

 */
/*
 * SYNOPSIS:
 *   int
 *   XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return)
 *   Display		*display;
 *   int		screen;
 *   char		*user_geom;
 *   char		*def_geom;
 *   unsigned int	bwidth;
 *   XSizeHints	*hints;
 *   int		*x_return;
 *   int		*y_return;
 *   int		*width_return;
 *   int		*height_return;
 *   int		*gravity_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XWMGeometry	1
char    *TestName = "XWMGeometry";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XWMGeometry function
 */
static Display		*display;
static int		screen;
static char		*user_geom;
static char		*def_geom;
static unsigned int	bwidth;
static XSizeHints	*hints;
static int		*x_return;
static int		*y_return;
static int		*width_return;
static int		*height_return;
static int		*gravity_return;


static int 	ValueReturn;

#include	"Xatom.h"
int		dint;
XSizeHints	size_hints;

XSizeHints	szhints = {	0L,
				0,0,     
				0,0,    
				38,	29,    	/* min width, min height */
				400,	300,	/* max width, max height */
				17,	18,  	/* width inc, height inc */
				{0,0},  
				{0,0},  
				20,	10,	/* base width, base height */
				SouthWestGravity};

XSizeHints	szhints1 = { PAllHints, /* Deliberate lack of PBaseSize and PWinGravity */
				0,0,     
				0,0,    
				20,	30,    	/* min width, min height */
				400,	300,	/* max width, max height */
				17,	18,  	/* width inc, height inc */
				{0,0},  
				{0,0},  
				-1,	-2,	/* base width, base height */
				-1};

XSizeHints	szhints2 = {	0L,
				0,0,     
				0,0,    
				0,0,
				0,0,
				0,0,
				{0,0},  
				{0,0},  
				0,0,
				0};



int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	screen = DefaultScreen(Dsp);
	user_geom = NULL;
	def_geom = NULL;
	bwidth = 1;
	hints = &size_hints;
	x_return = &dint;
	y_return = &dint;
	width_return = &dint;
	height_return = &dint;
	gravity_return = &dint;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XWMGeometry-1.(A)
 * A call to XWMGeometry combines the geometry information
 * given in the user_geom, def_geom and hints arguments and
 * returns in x_return the x offset, in y_return the y offset,
 * in width_return the width ( = hints->base_width+
 * width*hints->width_inc), in height_return the height ( =
 * hints->base_height+ height*hints->height_inc) and in
 * gravity_return the gravity information specified by the
 * geometry arguments.
 */
static void
t001()
{
int	rval;
int	xr, yr, wr, hr, gr;
int	x,  y,  w,  h,  g;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	bwidth = 10;
	szhints.flags= PAllHints|PBaseSize|PWinGravity;
	hints = &szhints;
	x_return = &xr;
	y_return = &yr;
	width_return = &wr;
	height_return = &hr;
	gravity_return = &gr;

	user_geom = "10x12+30+40";
	def_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	w = szhints.base_width  + (10 * szhints.width_inc);
	h = szhints.base_height + (12 * szhints.height_inc);
	x = 30;
	y = 40;

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(w != wr) {
		report("Returned width was %d instead of %d", wr, w);
		FAIL;
	} else
		CHECK;

	if(h != hr) {
		report("Returned height was %d instead of %d", hr, h);
		FAIL;
	} else
		CHECK;

	if(x != xr) {
		report("Returned x-coordinate was %d instead of %d", xr, x);
		FAIL;
	} else
		CHECK;

	if(y != yr) {
		report("Returned y-coordinate was %d instead of %d", yr, y);
		FAIL;
	} else
		CHECK;

	user_geom = NULL;
	def_geom =  "10x12+30+40";
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	w = szhints.base_width  + (10 * szhints.width_inc);
	h = szhints.base_height + (12 * szhints.height_inc);

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(w != wr) {
		report("Returned width was %d instead of %d", wr, w);
		FAIL;
	} else
		CHECK;

	if(h != hr) {
		report("Returned height was %d instead of %d", hr, h);
		FAIL;
	} else
		CHECK;

	if(x != xr) {
		report("Returned x-coordinate was %d instead of %d", xr, x);
		FAIL;
	} else
		CHECK;

	if(y != yr) {
		report("Returned y-coordinate was %d instead of %d", yr, y);
		FAIL;
	} else
		CHECK;

/* When no x,y is specified by user_geom, use the def_geom */
	user_geom = "10x12";
	def_geom = "2x3+30+40";
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	w = szhints.base_width  + (10 * szhints.width_inc);
	h = szhints.base_height + (12 * szhints.height_inc);

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(w != wr) {
		report("Returned width was %d instead of %d", wr, w);
		FAIL;
	} else
		CHECK;

	if(h != hr) {
		report("Returned height was %d instead of %d", hr, h);
		FAIL;
	} else
		CHECK;

	if(x != xr) {
		report("Returned x-coordinate was %d instead of %d", xr, x);
		FAIL;
	} else
		CHECK;

	if(y != yr) {
		report("Returned y-coordinate was %d instead of %d", yr, y);
		FAIL;
	} else
		CHECK;

/* Ensure that the minimums hold */
	user_geom = "1x1";
	def_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	w = szhints.min_width;
	h = szhints.min_height;

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(w != wr) {
		report("Returned width was %d instead of %d", wr, w);
		FAIL;
	} else
		CHECK;

	if(h != hr) {
		report("Returned height was %d instead of %d", hr, h);
		FAIL;
	} else
		CHECK;

/* Ensure that the maximums hold */
	user_geom = "40x30";
	def_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	w = szhints.max_width;
	h = szhints.max_height;

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(w != wr) {
		report("Returned width was %d instead of %d", wr, w);
		FAIL;
	} else
		CHECK;

	if(h != hr) {
		report("Returned height was %d instead of %d", hr, h);
		FAIL;
	} else
		CHECK;

/* Check that border, width and height is taken into account with negative x & y */
	user_geom = "10x12-1-2";
	def_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	w = szhints.base_width  + (10 * szhints.width_inc);
	h = szhints.base_height + (12 * szhints.height_inc);
	x = DisplayWidth(display, screen) + (-1) - w - 2 * bwidth;
	y = DisplayHeight(display, screen) + (-2) - h - 2 * bwidth;


	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(x != xr) {
		report("Returned x-coordinate was %d instead of %d", xr, x);
		FAIL;
	} else
		CHECK;

	if(y != yr) {
		report("Returned y-coordinate was %d instead of %d", yr, y);
		FAIL;
	} else
		CHECK;

	CHECKPASS(18);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWMGeometry-2.(A)
 * The value returned in the gravity_return information is one
 * of NorthWestGravity, NorthGravity, NorthEastGravity,
 * WestGravity, CenterGravity, EastGravity, SouthWestGravity,
 * SouthGravity, or SouthEastGravity.
 */
static void
t002()
{
int	rval;
int	xr, yr, wr, hr, gr;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	user_geom = "10x20-0-0";
	def_geom = NULL;
	bwidth = 1;
	szhints.flags= PAllHints|PBaseSize|PWinGravity;
	hints = &szhints;
	x_return = &xr;
	y_return = &yr;
	width_return = &wr;
	height_return = &hr;
	gravity_return = &gr;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if( (gr != NorthWestGravity) && (gr != NorthGravity) &&(gr != NorthEastGravity) &&
	    (gr != WestGravity) && (gr != CenterGravity) && (gr != EastGravity) &&
	    (gr != SouthWestGravity) && (gr != SouthGravity) && (gr != SouthEastGravity)) {
		report("Gravity value returned was invalid : %d", gr);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWMGeometry-3.(A)
 * A call to XWMGeometry returns a mask which indicates which
 * information came from the user_geom argument and whether the
 * position is relative to the right and bottom edges which is
 * the OR of none or any of XValue, YValue, WidthValue,
 * HeightValue, XNegative, or YNegative.
 */
static void
t003()
{
int	rval;
int	xr, yr, wr, hr, gr;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	user_geom = "40x30-9-9";
	def_geom = NULL;
	bwidth = 1;
	szhints.flags= PAllHints|PBaseSize|PWinGravity;
	hints = &szhints;
	x_return = &xr;
	y_return = &yr;
	width_return = &wr;
	height_return = &hr;
	gravity_return = &gr;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(rval != (XValue | YValue | WidthValue |HeightValue | XNegative | YNegative)) {
		report("Return value was 0x%x, instead of", rval);
		report("(XValue | YValue | XWidthValue | XHeightValue | XNegative | YNegative)");
		report("with user_geom = %s", user_geom);
		FAIL;
	} else
		CHECK;

	user_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(rval != NoValue) {
		report("Return value was 0x%x, instead of NoValue with user_geom = %s", rval, user_geom);
		FAIL;
	} else
		CHECK;

	user_geom = "-9+9";
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(rval != ( XValue | YValue | XNegative)) {
		report("Return value was 0x%x, instead of", rval);
		report("(XWidthValue | XHeightValue | XNegative)");
		report("with user_geom = %s", user_geom);
		FAIL;
	} else
		CHECK;

	user_geom = "5x5";
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(rval != (WidthValue | HeightValue)) {
		report("Return value was 0x%x, instead of", rval);
		report("(XValue | YValue)");
		report("with user_geom = %s", user_geom);
		FAIL;
	} else
		CHECK;

	user_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(rval != 0) {
		report("Return value was 0x%x, instead of 0", rval);
		report("with user_geom = %s", user_geom);
		FAIL;
	} else
		CHECK;

	CHECKPASS(5);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWMGeometry-4.(A)
 * When the base_width component of the XSizeHints structure
 * named by the hints argument is not set, then the min_width
 * component is used.
 */
static void
t004()
{
int	rval;
int	xr, yr, wr, hr, gr;
int	w;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	bwidth = 10;
	hints = &szhints1;
	x_return = &xr;
	y_return = &yr;
	width_return = &wr;
	height_return = &hr;
	gravity_return = &gr;

	user_geom = "10x12";
	def_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	w = szhints1.min_width  + (10 * szhints1.width_inc);

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(w != wr) {
		report("Returned width was %d instead of %d", wr, w);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWMGeometry-5.(A)
 * When the base_height component of the XSizeHints structure
 * named by the hints argument is not set, then the min_height
 * component is used.
 */
static void
t005()
{
int	rval;
int	xr, yr, wr, hr, gr;
int	h;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	bwidth = 10;
	hints = &szhints1;
	x_return = &xr;
	y_return = &yr;
	width_return = &wr;
	height_return = &hr;
	gravity_return = &gr;

	user_geom = "10x12";
	def_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	h = szhints1.min_height  + (12 * szhints1.height_inc);

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(h != hr) {
		report("Returned height was %d instead of %d", hr, h);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWMGeometry-6.(A)
 * When the min_width component is not set in the XSizeHints
 * structure named by the hints argument, then the base_width
 * component is used.
 */
static void
t006()
{
int	rval;
int	xr, yr, wr, hr, gr;
int	w;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	bwidth = 10;

	szhints2.flags = PBaseSize | PResizeInc;
	szhints2.min_width = 21;
	szhints2.min_height = 31;
	szhints2.width_inc = -1;
	szhints2.height_inc = -1;
	szhints2.base_height = 20;
	szhints2.base_width = 30;

	hints = &szhints2;
	x_return = &xr;
	y_return = &yr;
	width_return = &wr;
	height_return = &hr;
	gravity_return = &gr;

	user_geom = "1x1";
	def_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	w = szhints2.base_width;

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(w != wr) {
		report("Returned width was %d instead of %d", wr, w);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWMGeometry-7.(A)
 * When the min_height component is not set in the XSizeHints
 * structure named by the hints argument, then the base_height
 * component is used.
 */
static void
t007()
{
int	rval;
int	xr, yr, wr, hr, gr;
int	h;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	bwidth = 10;

	szhints2.flags = PBaseSize | PResizeInc;
	szhints2.min_width = 21;
	szhints2.min_height = 31;
	szhints2.width_inc = -1;
	szhints2.height_inc = -1;
	szhints2.base_height = 20;
	szhints2.base_width = 30;

	hints = &szhints2;
	x_return = &xr;
	y_return = &yr;
	width_return = &wr;
	height_return = &hr;
	gravity_return = &gr;

	user_geom = "1x1";
	def_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	h = szhints2.base_height;

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(h != hr) {
		report("Returned height was %d instead of %d", hr, h);
		FAIL;
	} else
		PASS;


	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWMGeometry-8.(A)
 * When neither the base_width nor min_width components of the
 * XSizeHints structure named by the hints argument is set,
 * then the value 0 is used.
 */
static void
t008()
{
int	rval;
int	xr, yr, wr, hr, gr;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	bwidth = 10;

	szhints2.flags = PResizeInc;
	szhints2.min_width = 21;
	szhints2.min_height = 31;
	szhints2.width_inc = -1;
	szhints2.height_inc = -1;
	szhints2.base_height = 20;
	szhints2.base_width = 30;

	hints = &szhints2;
	x_return = &xr;
	y_return = &yr;
	width_return = &wr;
	height_return = &hr;
	gravity_return = &gr;

	user_geom = "1x1";
	def_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(wr != 0) {
		report("Returned width was %d instead of 0", wr);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XWMGeometry-9.(A)
 * When neither the base_height nor min_height components of
 * the XSizeHints structure named by the hints argument is set,
 * then the value 0 is used.
 */
static void
t009()
{
int	rval;
int	xr, yr, wr, hr, gr;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	bwidth = 10;

	szhints2.flags = PResizeInc;
	szhints2.min_width = 21;
	szhints2.min_height = 31;
	szhints2.width_inc = -1;
	szhints2.height_inc = -1;
	szhints2.base_height = 20;
	szhints2.base_width = 30;

	hints = &szhints2;
	x_return = &xr;
	y_return = &yr;
	width_return = &wr;
	height_return = &hr;
	gravity_return = &gr;

	user_geom = "1x1";
	def_geom = NULL;
	startcall(display);
	if (isdeleted())
		return;
	rval = XWMGeometry(display, screen, user_geom, def_geom, bwidth, hints, x_return, y_return, width_return, height_return, gravity_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	trace("user_geom = %s, def_geom = %s", user_geom == NULL ? "<NULL>": user_geom, def_geom == NULL ? "<NULL>": def_geom);

	if(hr != 0) {
		report("Returned height was %d instead of 0", hr);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	t006, 6,
	t007, 7,
	t008, 8,
	t009, 9,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
