/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: crtrgn.m,v 1.8 92/06/11 17:33:40 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Region
 *   XCreateRegion()
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XCreateRegion	1
char    *TestName = "XCreateRegion";



int 	tet_thistest;

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XCreateRegion-1.(A)
 * A call to XCreateRegion returns a new, empty region.
 */
static void
t001()
{
static	XRectangle	rect = { -99, 666, 198, 3 };
XRectangle		bbox;
Region			reg;
int 	pass = 0, fail = 0;

	tpstartup();

	startcall(Dsp);
	if (isdeleted())
		return;
	reg = XCreateRegion();
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(reg == (Region) NULL) {
		delete("%s() returned NULL.", TestName);
		return;
	} else
		CHECK;

	if( XEmptyRegion(reg) != True) {
		report("XEmptyRegion() returned True.");
		FAIL;
	} else
		CHECK;

	XClipBox(reg, &bbox);
	if( (bbox.width != 0) || (bbox.height != 0)) {
		report("%s() returned a region of width %d and height %d instead of 0.", TestName, bbox.width, bbox.height);
		FAIL;
	} else
		CHECK;

	XUnionRectWithRegion(&rect, reg, reg);
	XClipBox(reg, &bbox);	
	
	if((rect.x != bbox.x) || (rect.y != bbox.y) || (rect.width != bbox.width) || (rect.height != bbox.height) ) {
		report("%s() returned the rectangle x %d, y %d, height %d, width %d, instead of x %d, y %d, height %d, width %d.",
                        "XClipBox",
			rect.x, rect.y, rect.width, rect.height,
			bbox.x, bbox.y, bbox.width, bbox.height );
		FAIL;
	} else
		CHECK;

	XDestroyRegion(reg);
	CHECKPASS(4);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XCreateRegion-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When sufficient storage cannot be allocated, then a call to
 * XCreateRegion returns NULL.
 */
static void
t002()
{
	report("There is no known portable test method for this assertion");
	UNTESTED;
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
