/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: fndcntxt.m,v 1.5 92/06/11 17:34:26 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XFindContext(display, w, context, data_return)
 *   Display *display;
 *   Window w;
 *   XContext context;
 *   caddr_t *data_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XFindContext	1
char    *TestName = "XFindContext";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_WINDOW w
#define A_DRAWABLE w


/*
 * Arguments to the XFindContext function
 */
static Display *display;
static Window w;
static XContext context;
static caddr_t *data_return;


static int 	ValueReturn;

static caddr_t xfc_data;

static char *xfc_ctxt = "another context";

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	w = defwin(display);
	context = XUniqueContext();
	data_return = &xfc_data;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XFindContext-1.(A)
 * A call to XFindContext returns the context data for display
 * display, window w and context type context in data_return,
 * and returns zero.
 */
static void
t001()
{
int a;
int ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a test window. */
/* Save some context information using XSaveContext. */
	a = XSaveContext(display, w, context, (caddr_t)xfc_ctxt);
	if (a != 0) {
		delete("Could not save test context.");
		report("Returned error: %s", contexterrorname(a));
		return;
	} else
		CHECK;

/* Find the context information using xname. */
	xfc_data = (caddr_t)NULL;
	startcall(display);
	if (isdeleted())
		return;
	ret = XFindContext(display, w, context, data_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify the context was returned as expected. */
#ifdef TEST_RPT
	xfc_data++;
#endif
	if (ret != 0) {
		FAIL;
		report("%s returned an error when expected to succeed.",
			TestName);
		report("Returned error: %s", contexterrorname(ret));
	} else {
		CHECK;
		if (xfc_data != (caddr_t)xfc_ctxt) {
			FAIL;
			report("%s did not return the expected context data.",
				TestName);
			report("Expected context: %0x", (unsigned int)xfc_ctxt);
			report("Returned context: %0x", (unsigned int)xfc_data);
		} else
			CHECK;
	}

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XFindContext-2.(A)
 * When there is no previously saved context data for display
 * display, window w and context type context, then a call to
 * XFindContext returns XCNOENT.
 */
static void
t002()
{
int ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a test window with no context. */

/* Call xname to find the non-existent context. */
	startcall(display);
	if (isdeleted())
		return;
	ret = XFindContext(display, w, context, data_return);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

#ifdef TEST_RPT
	ret = 0;
#endif

/* Verify that XCNOENT was returned. */
	if (ret != XCNOENT) {
		FAIL;
		report("%s did not return XCNOENT when",
			TestName);
		report("finding a non-existent context.");
		report("Return code: %s", contexterrorname(ret));
	} else
		CHECK;

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
