/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: gtdflt.m,v 1.17 92/07/01 11:37:17 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   char *
 *   XGetDefault(display, program, option)
 *   Display	*display;
 *   char	*program;
 *   char	*option;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XGetDefault	1
char    *TestName = "XGetDefault";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XGetDefault function
 */
static Display	*display;
static char	*program;
static char	*option;


#include	"Xatom.h"

extern char **environ;

static void
t003exec()
{
char	*argv[2];
char	*envp;

	argv[0] = "./Test3";
	argv[1] = (char *) NULL;
	envp = "HOME=./";
	if (xtest_putenv( envp )) {
		delete("xtest_putenv failed to add HOME to the environment");
		return;
	}
	tet_exec("./Test3", argv, environ);
	delete("tet_exec() of \"./Test3\" failed.");
}


static void
t004exec()
{
char	*argv[2];
char	*envp;

	argv[0] = "./Test4";
	argv[1] = (char *) NULL;
	envp = "XENVIRONMENT=./EnvXdefaults";
	if (xtest_putenv(envp)) {
		delete("xtest_putenv failed to add XENVIRONMENT to the environment");
	}

	tet_exec("./Test4", argv, environ);
	delete("tet_exec() of \"./Test4\" failed.");
}



static void
t005exec()
{
char	*argv[2];
char	*envp;

	argv[0] = "./Test5";
	argv[1] = (char *) NULL;
	envp = "HOME=.";
	if (xtest_putenv(envp)) {
		delete("xtest_putenv failed to add HOME to the environment");
		return;
	}

	if (getenv("XENVIRONMENT") != (char *)NULL) {
		char **newenv = environ; /* Remove XENVIRONMENT */
	
		trace("Removing XENVIRONMENT from environment");

		while( strncmp("XENVIRONMENT=", *newenv, 13)
			&& *newenv != NULL )
			newenv++;

		if (*newenv == NULL) {
			report("could not remove XENVIRONMENT from the environment");
			UNRESOLVED;
			return;
		}

		do {
			*newenv = *(newenv+1);
			newenv++;
		} while ( *newenv != NULL );
	}
	tet_exec("./Test5", argv, environ);
	delete("tet_exec() of \"./Test5\" failed.");
}

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	program = 0;
	option = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XGetDefault-1.(A)
 * A call to XGetDefault reads and returns from the resource
 * manager database the entry specified by the program and
 * option arguments.
 */
static void
t001()
{
unsigned char 	*pval = (unsigned char *) "VAL*VAl\nXT.LEO:CAL\nXT.OPT:VAL\nXT.Bezoomny:Cal\n";
char		*valstr;
char		*res = (char *) NULL;
Display		*client;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	valstr = "VAL";

	XChangeProperty (Dsp, RootWindow(Dsp, 0), XA_RESOURCE_MANAGER, XA_STRING, 8, PropModeReplace, pval, 1+strlen(pval));
	XSync(Dsp, False);

	display = opendisplay();
	program = "XT";
	option  = "OPT";
	startcall(display);
	if (isdeleted())
		return;
	res = XGetDefault(display, program, option);	
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res == (char *) NULL) {
		report("%s() returned NULL.", TestName);
		FAIL;
	} else {
		CHECK;

		if(strcmp(res, valstr) != 0) {
			report("%s() retured \"%s\" instead of \"%s\"", TestName, res, valstr);
			FAIL;
		} else
			CHECK;
	}

	CHECKPASS(2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetDefault-2.(A)
 * When the option specified by the program and option
 * arguments does not exist in the resource manager database,
 * then a call to XGetDefault returns NULL.
 */
static void
t002()
{
unsigned char 	*pval = (unsigned char *) "XT.LEO:CAL\nXT.OPT:VAL\nXT.Bezoomny:Cal\n";
char		*nullstr = "<NULL>";
char		*res = (char *) NULL;
Display		*client;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();


	XChangeProperty (Dsp, RootWindow(Dsp, 0), XA_RESOURCE_MANAGER, XA_STRING, 8, PropModeReplace, pval, 1+strlen(pval));
	XSync(Dsp, False);

	display = opendisplay();
	program = "XT";
	option  = "Cal";
	startcall(display);
	if (isdeleted())
		return;
	res = XGetDefault(display, program, option);	
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(res != (char *) NULL) {
		report("%s() retured \"%s\" instead of \"%s\"", TestName, res, nullstr);
		FAIL;
	} else
		PASS;

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetDefault-3.(C)
 * If the system is POSIX compliant: When the RESOURCE_MANAGER
 * property was defined at the time display was opened, then
 * the default resource manager database is taken from the
 * value of that property, otherwise the default resource
 * manager database is taken from the file $HOME/.Xdefaults in
 * the client's home directory.
 */
static void
t003()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(config.posix_system == 0)
		unsupported("This assertion can only be tested on a POSIX system.");
	else
		tet_fork(t003exec, TET_NULLFP, 0, ~0);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetDefault-4.(C)
 * If the system is POSIX compliant: When the XENVIRONMENT
 * environment variable is defined, then the file named by the
 * XENVIRONMENT environment variable is merged to update the
 * default resource database.
 */
static void
t004()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(config.posix_system == 0)
		unsupported("This assertion can only be tested on a POSIX system.");
	else
		tet_fork(t004exec, TET_NULLFP, 0, ~0);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XGetDefault-5.(C)
 * If the system is POSIX compliant: When the XENVIRONMENT
 * environment variable is not defined, then the file
 * $HOME/.Xdefaults-<name> is merged to update the default
 * resource database, where <name> specifies the name of the
 * machine on which the application is running.
 */
static void
t005()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	if(config.posix_system == 0)
		unsupported("This assertion can only be tested on a POSIX system.");
	else
		tet_fork(t005exec, TET_NULLFP, 0, ~0);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
