/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: pntinrgn.m,v 1.7 92/06/11 17:38:06 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Bool
 *   XPointInRegion(r, x, y)
 *   Region	r;
 *   int	x;
 *   int	y;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XPointInRegion	1
char    *TestName = "XPointInRegion";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XPointInRegion function
 */
static Region	r;
static int	x;
static int	y;


static	XPoint	lines[] = { {20,20}, {35,10}, {55,10}, {80,20}, {90,35}, {85,50}, {75,65}, {50,70}, {30,65}, {20,60}, {15,40}, {20,20} };
static	XPoint	ply1[] = { {20,20}, {35,10}, {55,10}, {80,20}, {90,35}, {85,50}, {75,65}, {50,70}, {30,65}, {20,60}, {15,40} };


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	r = 0;
	x = 0;
	y = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XPointInRegion-1.(A)
 * When the point specified by the x and y arguments is
 * contained in the region r, then a call to XPointInRegion
 * returns True.
 */
static void
t001()
{
Region		R;
Region		Rp;
Bool		res;
XRectangle	rrect;
int		maxx;
int		maxy;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R = makeregion();
	if(isdeleted()) return;
	R = XPolygonRegion(ply1, NELEM(ply1), WindingRule);
	
	XClipBox(R, &rrect);

	rrect.height = 1;
	rrect.width  = 1;
	r = R;
	maxx = 10 + rrect.x + rrect.width;
	maxy = 10 + rrect.y + rrect.height;
	for(x = rrect.x - 10; x <= maxx; x++) {
		rrect.x = x;
		for(y = rrect.y - 10; y <= maxy; y++) {
			rrect.y = y;
			startcall(Dsp);
			if (isdeleted())
				return;
			res = XPointInRegion(r, x, y);
			endcall(Dsp);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			Rp = XCreateRegion();
			XUnionRectWithRegion(&rrect, Rp, Rp);
			XUnionRegion(R, Rp, Rp);

			if(XEqualRegion(R, Rp) == True) {
				if(res != True) {
					report("%s() did not return True",
						TestName);
					report("when point %d %d was in the specified region",
						x, y);
					FAIL;
				} 
			} 
			if((x == maxx) && (y == maxy))
				CHECK;
			XDestroyRegion(Rp);
		}
	}

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XPointInRegion-2.(A)
 * When the point specified by the x and y arguments is not
 * contained in the region r, then a call to XPointInRegion
 * returns False.
 */
static void
t002()
{
Region		R;
Region		Rp;
Bool		res;
XRectangle	rrect;
int		maxx;
int		maxy;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R = makeregion();
	if(isdeleted()) return;
	R = XPolygonRegion(ply1, NELEM(ply1), WindingRule);
	
	XClipBox(R, &rrect);

	rrect.height = 1;
	rrect.width  = 1;
	r = R;
	maxx = 10 + rrect.x + rrect.width;
	maxy = 10 + rrect.y + rrect.height;
	for(x = rrect.x - 10; x <= maxx; x++) {
		rrect.x = x;
		for(y = rrect.y - 10; y <= maxy; y++) {
			rrect.y = y;
			startcall(Dsp);
			if (isdeleted())
				return;
			res = XPointInRegion(r, x, y);
			endcall(Dsp);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			Rp = XCreateRegion();
			XUnionRectWithRegion(&rrect, Rp, Rp);
			XUnionRegion(R, Rp, Rp);

			if(XEqualRegion(R, Rp) != True) {
				if(res != False) {
					report("%s() did not return False",
						TestName);
					report("when point %d %d was not in the specified region",
						x, y);
					FAIL;
				}
			}
			if((x == maxx) && (y == maxy))
					CHECK;
			XDestroyRegion(Rp);
		}
	}

	CHECKPASS(1);

	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
