/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: ptpxl.m,v 1.4 92/06/11 17:38:36 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   int
 *   XPutPixel(ximage, x, y, pixel)
 *   XImage		*ximage;
 *   int		x;
 *   int		y;
 *   unsigned long	pixel;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XPutPixel	1
char    *TestName = "XPutPixel";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XPutPixel function
 */
static XImage		*ximage;
static int		x;
static int		y;
static unsigned long	pixel;


static int 	ValueReturn;

int bltimage(im, w, h, dep)
XImage		*im;
unsigned int	w;
unsigned int	h;
int		dep;
{
int		i;
unsigned long	mask;
unsigned long	rpixel;

	ximage = im;
	mask = (1<<dep) - 1;
	for(y=0; y<h; y+=10)
		for(x=0; x<w; x+=10) {
			for(i = 0; i <= (15 & mask); i++) {
				pixel =  mask & (i | i<<4 | i<<12 | i<<20);
				startcall(Dsp);
				if (isdeleted())
					return(0);
				ValueReturn = XPutPixel(ximage, x, y, pixel);
				if (ValueReturn == 0) {
					report("%s() returned 0", TestName);
					return(0);
				}
				endcall(Dsp);
				if (geterr() != Success) {
					report("Got %s, Expecting Success", errorname(geterr()));
					return(0);
				}
				if((rpixel = XGetPixel(ximage, x, y)) != pixel) {
					report("XGetPixel() returned %lx instead of %lx.", rpixel, pixel);
					return(0);
				}
			}
		}

	return(1);
}		

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	ximage = 0;
	x = 0;
	y = 0;
	pixel = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XPutPixel-1.(A)
 * When the image ximage contains the coordinate x,y, and the
 * pixel argument is in normalised format, then a call to
 * XPutPixel overwrites x,y with the value pixel.
 */
static void
t001()
{
XVisualInfo	*vi;
int		npix;
unsigned int	width;
unsigned int	height;
Pixmap		pm;
Window		win;
int		i;
static int	fmats[2] = { XYPixmap, ZPixmap };
	
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(resetvinf(VI_PIX); nextvinf(&vi);) {

		pm = makepixm(Dsp, vi);
		getsize(Dsp, pm, &width, &height);
		width = width > 17 ? 17 : width;
		height = height > 19  ? 19 : height;

		for(i=0; i<2; i++) {
			ximage = XGetImage(Dsp, pm, 0,0, width, height, AllPlanes, fmats[i]);
			if( ximage == (XImage *) NULL ) {
				delete("XGetImage() returned NULL.");
			} else {
				if(bltimage(ximage, width, height, vi->depth) == 0) 
					FAIL;
				else
					CHECK;

				XDestroyImage(ximage);
			}

		}
	}
	npix = nvinf();

	for(resetvinf(VI_WIN); nextvinf(&vi);) {

		win = makewin(Dsp, vi);
		getsize(Dsp, win, &width, &height);
		width = width > 17 ? 17 : width;
		height = height > 19  ? 19 : height;

		for(i=0; i<2; i++) {
			ximage = XGetImage(Dsp, win, 0,0, width, height, AllPlanes, fmats[i]);
			if(ximage == (XImage *) NULL) {
				delete("XGetImage() returned NULL.");
			} else {
				if(bltimage(ximage, width, height, vi->depth) == 0) 
					FAIL;
				else
					CHECK;

				XDestroyImage(ximage);
			}
		}

	}

	CHECKPASS(2 * (npix + nvinf()));
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
