/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rmqgtsrchl.m,v 1.6 92/06/11 17:41:12 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   Bool
 *   XrmQGetSearchList(database, names, classes, list_return, list_length)
 *   XrmDatabase database;
 *   XrmNameList names;
 *   XrmClassList classes;
 *   XrmHashTable *list_return;
 *   int list_length;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XrmQGetSearchList	1
char    *TestName = "XrmQGetSearchList";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XrmQGetSearchList function
 */
static XrmDatabase database;
static XrmNameList names;
static XrmClassList classes;
static XrmHashTable *list_return;
static int list_length;



#define XQGSL_LENGTH 3
static	XrmHashTable	sl[XQGSL_LENGTH];

#define XQGSL_N_QUARK 10

static	XrmName	nl[XQGSL_N_QUARK];
static	XrmClass	cl[XQGSL_N_QUARK];

#define XQGSL_T3_ENTRIES 2
static char *t3_data[XQGSL_T3_ENTRIES] = {
	"a:one",
	"b:two" };

#define XQGSL_T4_ENTRIES 10
static char *t4_data[XQGSL_T4_ENTRIES] = {
	"a*z:one",
	"a.b*z:two",
	"a.b.c*z:three",
	"a.b.c.d*z:four",
	"a.b.c.d.e*z:five",
	"A*Z:six",
	"A.B*Z:seven",
	"A.B.C*Z:eight",
	"A.B.C.D*Z:nine",
	"A.B.C.D.E*Z:ten" };


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	database = (XrmDatabase)NULL;
	names = nl;
	classes = cl;
	list_return = sl; 	/* This is a XrmSearchList */
	list_length = XQGSL_LENGTH;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XrmQGetSearchList-1.(B)
 *   Reason: The statement in the X11 specification is not specific enough to write a test
 * A call to XrmQGetSearchList returns a list of database
 * levels in list_return where a match for the specified names
 * and classes in the database could occur.
 */
static void
t001()
{
	report("The statement in the X11 specification is not specific enough to write a test");
	UNTESTED;
}

/*
 * Assertion XrmQGetSearchList-2.(B)
 *   Reason: The statement in the X11 specification is not specific enough to write a test
 * On a call to XrmQGetSearchList, the list returned in
 * list_return is in best-to-worst match order.
 */
static void
t002()
{
	report("The statement in the X11 specification is not specific enough to write a test");
	UNTESTED;
}

/*
 * Assertion XrmQGetSearchList-3.(A)
 * When list_length is greater than or equal to the number of
 * database levels where a match could occur, then a call to
 * XrmQGetSearchList returns True.
 */
static void
t003()
{
int a;
Bool ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a test database with insufficient database levels to fill the list. */
	for(a=0; a<XQGSL_T3_ENTRIES; a++) {
		XrmPutLineResource(&database, t3_data[a]);
	}

/* Call xname to obtain search list. */
	for(a=0; a<XQGSL_N_QUARK; a++) {
		nl[a]=(XrmName)0;
		cl[a]=(XrmClass)0;
	}
	XrmStringToNameList( "a", nl );
	XrmStringToClassList("A", cl );
	startcall(Dsp);
	if (isdeleted())
		return;
	ret = XrmQGetSearchList(database, names, classes, list_return, list_length);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that True was returned. */
	if (ret != True) {
		FAIL;
		report("%s did not return True when the list_length was",
			TestName);
		report("greater than or equal to the number of");
		report("possible match database levels.");
		report("Returned value: %s", boolname(ret));
	} else
		CHECK;

	CHECKPASS(1);

	XrmDestroyDatabase(database);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmQGetSearchList-4.(A)
 * When list_length is less than the number of database levels
 * where a match could occur, then a call to XrmQGetSearchList
 * returns False.
 */
static void
t004()
{
int a;
Bool ret;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Create a test database with sufficient database levels to fill the list. */
	for(a=0; a<XQGSL_T4_ENTRIES; a++) {
		XrmPutLineResource(&database, t4_data[a]);
	}

/* Call xname to obtain search list. */
	for(a=0; a<XQGSL_N_QUARK; a++) {
		nl[a]=(XrmName)0;
		cl[a]=(XrmClass)0;
	}
	XrmStringToNameList( "a.b.c.d.e.f.g.h.z", nl );
	XrmStringToClassList( "A.B.C.D.E.F.G.H.Z", cl );
	startcall(Dsp);
	if (isdeleted())
		return;
	ret = XrmQGetSearchList(database, names, classes, list_return, list_length);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that False was returned. */
	if (ret != False) {
		FAIL;
		report("%s did not return False when the list_length was",
			TestName);
		report("less than the number of possible match database levels.");
		report("Returned value: %s", boolname(ret));
	} else
		CHECK;

	CHECKPASS(1);

	XrmDestroyDatabase(database);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = rmstartup;
void	(*tet_cleanup)() = cleanup;
