/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: rmstrtbndn.m,v 1.5 92/06/11 17:41:51 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   void
 *   XrmStringToBindingQuarkList(string, bindings_return, quarks_return)
 *   char *string;
 *   XrmBindingList bindings_return;
 *   XrmQuarkList quarks_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XrmStringToBindingQuarkList	1
char    *TestName = "XrmStringToBindingQuarkList";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XrmStringToBindingQuarkList function
 */
static char *string;
static XrmBindingList bindings_return;
static XrmQuarkList quarks_return;


#define XSTBQL_MAX_RETURN 10
static XrmBinding xstbql_bret[XSTBQL_MAX_RETURN];
static XrmQuark xstbql_qret[XSTBQL_MAX_RETURN];

static char *xstbql_s1[]={
	"xstbql_test",
	"einse",
	"zwei",
	"drei",
	"vier",
	"funf",};

static char *xstbql_s1prefix[]={
	"*",
	".",
	"*",
	".",
	"*",
	"."};

static int xstbql_ns1=NELEM(xstbql_s1);

static char *xstbql_s2[]={
	"xstbql_test2",
	"Binkleys",
	"Dads",
	"1972",
	"Ford",
	"Pinto"};

static char *xstbql_s2sep[]={
	"*",
	".",
	"*",
	"*",
	".",
	""};

static int xstbql_ns2=NELEM(xstbql_s2);

static char *xstbql_s4[]={
	"xstbql_test2",
	"hazel",
	"wal",
	"coco",
	"pea"};

static char *xstbql_s4prefix[]={
	".",
	"*",
	".",
	"*",
	"."};

static int xstbql_ns4=NELEM(xstbql_s4);


static char *xstbql_s5[]={
	"xstbql_test5.",
	"Calvin*",
	"Hobbes",
	};
static int xstbql_ns5=NELEM(xstbql_s5);

static void
xstbqlclear(value)
XrmQuark value;
{
	int i;

/* Clear the return arrays, to avoid rogue results. */
	for(i=0; i<XSTBQL_MAX_RETURN; i++) 
	{
		xstbql_qret[i]=value;
		xstbql_bret[i]=XrmBindTightly; /* Have to use a value. */
	}

}


static
xstbqlname(value, ptr)
XrmBinding value;
char *ptr;
{
	if(value==XrmBindTightly) {
		strcpy(ptr, "XrmBindTightly");
	} else
	if(value==XrmBindLoosely) {
		strcpy(ptr, "XrmBindLoosely");
	} else
		sprintf(ptr, "UNKNOWN(%d)", (int)value);
}
		

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	string = 0;
	bindings_return = (XrmBindingList) xstbql_bret;
	quarks_return = (XrmQuarkList) xstbql_qret; 
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XrmStringToBindingQuarkList-1.(A)
 * When string is a valid null-terminated resource name format
 * string, then a call to XrmStringToBindingQuarkList returns a
 * quark list in quarks_return corresponding to the components
 * of string.
 */
static void
t001()
{
int  t;
char *buffer1;
char *buffer2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	buffer1 = (char *)malloc( (size_t)256);
	if (buffer1==(char *)NULL) {
		delete("Could not malloc 256 bytes for buffer1.");
		return;
	} else
		CHECK;

	buffer2 = (char *)malloc( (size_t)4096);
	if (buffer2==(char *)NULL) {
		delete("Could not malloc 4096 bytes for buffer2.");
		return;
	} else
		CHECK;

	*buffer1='\0'; /* Make the buffers empty strings. */
	*buffer2='\0';

/* Create the test string. */
	for(t=0; t<xstbql_ns1; t++) {
		(void) strcat(buffer1, xstbql_s1prefix[t]);
		(void) strcat(buffer1, xstbql_s1[t]);
	}
	trace("Testing with '%s'", buffer1);

	xstbqlclear((XrmQuark)0);

/* Call xname to obtain the quark and binding lists. */
	string= buffer1;
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmStringToBindingQuarkList(string, bindings_return, quarks_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

#ifdef TESTING
	xstbql_qret[0]=0;
	xstbql_qret[1]=XrmStringToQuark("TESTING");
#endif

/* Verify that the quark list returned was as expected. */
	for(t=0; t<xstbql_ns1; t++) {
		char *ts;

		ts = XrmQuarkToString( xstbql_qret[t] );
		if (ts == (char *)NULL) {
			FAIL;
			report("Quark array[%d]=%d. Represents: NO STRING (expecting '%s')",
				t, xstbql_qret[t], xstbql_s1[t]);
			strcat(buffer2,"<NO STRING>");
		} else
		{
			if(strcmp(xstbql_s1[t], ts)) {
				FAIL;
				report("Quark array[%d]=%d. Represents: '%s' (expecting '%s')",
					t, xstbql_qret[t], ts, xstbql_s1[t]);
			} else {
				CHECK;
				trace("Quark array[%d]=%d. Represents: '%s' as expected.",
					t, xstbql_qret[t], ts );
			}
			(void) strcat(buffer2, ts);
		}
		(void) strcat(buffer2, " ");
	}

	if (fail) {
		report("%s did not split the string into the quarks as expected",
			TestName);
		report("String passed to %s: '%s'", TestName, buffer1);
		report("Quark array represents: %s", buffer2);
	} else
		CHECK;

	CHECKPASS(2+xstbql_ns1+1);
	free(buffer1);
	free(buffer2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmStringToBindingQuarkList-2.(A)
 * On a call to XrmStringToBindingQuarkList, the string is
 * separated into components according to the positions of
 * periods and asterisks.
 */
static void
t002()
{
int  t;
char *buffer1;
char *buffer2;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	buffer1 = (char *)malloc( (size_t)256);
	if (buffer1==(char *)NULL) {
		delete("Could not malloc 256 bytes for buffer1.");
		return;
	} else
		CHECK;

	buffer2 = (char *)malloc( (size_t)4096);
	if (buffer2==(char *)NULL) {
		delete("Could not malloc 4096 bytes for buffer2.");
		return;
	} else
		CHECK;

	*buffer1='\0'; /* Make the buffers empty strings. */
	*buffer2='\0';

/* Create the test string. */
	for(t=0; t<xstbql_ns2; t++) {
		(void) strcat(buffer1, xstbql_s2[t]);
		(void) strcat(buffer1, xstbql_s2sep[t]);
	}
	trace("Testing with '%s'", buffer1);

	xstbqlclear((XrmQuark)0);

/* Call xname to obtain the quark and binding list. */
	string= buffer1;
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmStringToBindingQuarkList(string, bindings_return, quarks_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

#ifdef TESTING
	xstbql_qret[0]=0;
	xstbql_qret[1]=XrmStringToQuark("TESTING");
#endif

/* Verify that the quark list returned was as expected. */
	for(t=0; t<xstbql_ns2; t++) {
		char *ts;

		ts = XrmQuarkToString( xstbql_qret[t] );
		if (ts == (char *)NULL) {
			FAIL;
			report("Quark array[%d]=%d. Represents: NO STRING (expecting '%s')",
				t, xstbql_qret[t], xstbql_s2[t]);
			strcat(buffer2,"<NO STRING>");
		} else
		{
			if(strcmp(xstbql_s2[t], ts)) {
				FAIL;
				report("Quark array[%d]=%d. Represents: '%s' (expecting '%s')",
					t, xstbql_qret[t], ts, xstbql_s2[t]);
			} else {
				CHECK;
				trace("Quark array[%d]=%d. Represents: '%s' as expected.",
					t, xstbql_qret[t], ts );
			}
			(void) strcat(buffer2, ts);
		}
		(void) strcat(buffer2, " ");
	}

	if (fail) {
		report("%s did not split the string into the quarks as expected",
			TestName);
		report("String passed to %s: '%s'", TestName, buffer1);
		report("Quark array represents: '%s'", buffer2);
	} else
		CHECK;

	CHECKPASS(2+xstbql_ns2+1);
	free(buffer1);
	free(buffer2);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmStringToBindingQuarkList-3.(A)
 * When string does not start with a period or asterisk, then
 * a leading period is assumed.
 */
static void
t003()
{
char ts[64];
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

/* Call xname to obtain quark and binding lists. */
	string = "xstbql_test3";
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmStringToBindingQuarkList(string, bindings_return, quarks_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that the binding was correct. */
	if ( xstbql_bret[0] != XrmBindTightly ) {
		FAIL;
		xstbqlname(xstbql_bret[0], ts);
		report("%s did not assume a leading period", TestName);
		report("Returned %s", ts);
		report("Expected XrmBindTightly");
	} else
		CHECK;

	CHECKPASS(1);
	
	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmStringToBindingQuarkList-4.(A)
 * When string is a valid resource name format string, then a
 * call to XrmStringToBindingQuarkList returns a binding list
 * in bindings_return corresponding to the components of string,
 * with components of string starting with an asterisk set to
 * XrmBindLoosely and other components set to XrmBindTightly.
 */
static void
t004()
{
int  t;
char *buffer1;
char *buffer2, *buffer3;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	buffer1 = (char *)malloc( (size_t)256);
	if (buffer1==(char *)NULL) {
		delete("Could not malloc 256 bytes for buffer1.");
		return;
	} else
		CHECK;

	buffer2 = (char *)malloc( (size_t)1024);
	if (buffer2==(char *)NULL) {
		delete("Could not malloc 1024 bytes for buffer2.");
		return;
	} else
		CHECK;

	buffer3 = (char *)malloc( (size_t)1024);
	if (buffer3==(char *)NULL) {
		delete("Could not malloc 1024 bytes for buffer3.");
		return;
	} else
		CHECK;

	*buffer1='\0'; /* Make the buffers empty strings. */
	*buffer2='\0';
	*buffer3='\0';

/* Create the test string. */
	for(t=0; t<xstbql_ns4; t++) {
		(void) strcat(buffer1, xstbql_s4prefix[t]);
		(void) strcat(buffer1, xstbql_s4[t]);
	}
	trace("Testing with '%s'", buffer1);

	xstbqlclear((XrmQuark)0);

/* Call xname to obtain the quark and binding lists. */
	string= buffer1;
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmStringToBindingQuarkList(string, bindings_return, quarks_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

#ifdef TESTING
	xstbql_bret[0]=(XrmBinding) -256;
#endif

/* Verify that the binding list returned was as expected. */
	for(t=0; t<xstbql_ns4; t++) {
		char retstr[64], expstr[64];
		XrmBinding expect;

		expect = *(xstbql_s4prefix[t])=='*'? XrmBindLoosely:XrmBindTightly;

		xstbqlname(expect, expstr);
		xstbqlname(xstbql_bret[t], retstr);

		if (xstbql_bret[t] != expect) {
			FAIL;
			report("Binding array[%d]=%s. Expecting %s",
				t, retstr, expstr);
		} else {
			CHECK;
			trace("Binding array[%d]=%s as expected.",
				t, retstr);
		}

		(void) strcat(buffer2, expstr);
		(void) strcat(buffer2, " ");

		(void) strcat(buffer3, retstr);
		(void) strcat(buffer3, " ");
	}

	if (fail) {
		report("%s did not split the string into the bindings as expected",
			TestName);
		report("String passed to %s: '%s'", TestName, buffer1);
		report("Expected bindings: %s", buffer2);
		report("Returned bindings: %s", buffer3);
	} else
		CHECK;

	CHECKPASS(3+xstbql_ns4+1);
	free(buffer1);
	free(buffer2);
	free(buffer3);

	tpcleanup();
	pfcount(pass, fail);
}

/*
 * Assertion XrmStringToBindingQuarkList-5.(A)
 * On a call to XrmStringToBindingQuarkList, the
 * quarks_return list is terminated with a zero.
 */
static void
t005()
{
char *buffer1;
XrmQuark unq;
int t;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	buffer1 = (char *)malloc( (size_t)256);
	if (buffer1==(char *)NULL) {
		delete("Could not malloc 256 bytes for buffer1.");
		return;
	} else
		CHECK;

	*buffer1='\0';

/* Create the test string. */
	for(t=0; t<xstbql_ns5; t++) {
		(void) strcat(buffer1, xstbql_s5[t]);
	}
	trace("Testing with '%s'", buffer1);

/* Set return buffer with unique quark value. */
	unq=XrmUniqueQuark();
	xstbqlclear(unq);

/* Call xname to obtain the quark and binding lists. */
	string= buffer1;
	startcall(Dsp);
	if (isdeleted())
		return;
	XrmStringToBindingQuarkList(string, bindings_return, quarks_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

/* Verify that the quark list was zero terminated. */
	t=0;
	while(t<XSTBQL_MAX_RETURN
		&& xstbql_qret[t] != unq
		&& xstbql_qret[t] != (XrmQuark)0) {
		t++;
	}

	if (t==XSTBQL_MAX_RETURN) {
		int i;
		FAIL;
		report("%s did not return the quark array as expected.",
			TestName);
		report("At least %d quarks returned.", XSTBQL_MAX_RETURN);
		for (i=0; i<XSTBQL_MAX_RETURN; i++)
			report("Quark array[%d]=%d", i, xstbql_qret[i]);
	} else {
		if (xstbql_qret[t] == unq) {
			FAIL;
			report("%s did not terminate the array with a zero.",
				TestName);
		} else
			CHECK;
	}

	CHECKPASS(2);
	free(buffer1);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	t003, 3,
	t004, 4,
	t005, 5,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = rmstartup;
void	(*tet_cleanup)() = cleanup;
