/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: strbyts.m,v 1.7 92/06/11 17:43:06 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XStoreBytes(display, bytes, nbytes)
 *   Display	*display;
 *   char	*bytes;
 *   int	nbytes;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XStoreBytes	1
char    *TestName = "XStoreBytes";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XStoreBytes function
 */
static Display	*display;
static char	*bytes;
static int	nbytes;


static char	buff[1217];

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	bytes = buff;
	nbytes = NELEM(buff);
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XStoreBytes-1.(A)
 * A call to XStoreBytes stores nbytes bytes from the bytes
 * argument into cut buffer zero.
 */
static void
t001()
{
char	*bp;
char	*bpr = (char *) NULL;
int	i;
int	br;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	for(i=NELEM(buff), bp = buff; i>0; *bp++ = (char) (i & 255), i--);

	startcall(display);
	if (isdeleted())
		return;
	XStoreBytes(display, bytes, nbytes);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	bp = XFetchBuffer(display, &br, 0);

	if(bp == (char *) NULL) {
		report("%s() did not set buffer 0 to contain any data.", TestName);
		FAIL;
	} else {
		CHECK;

		if(br != NELEM(buff)) {
			report("%s() set buffer 0 to contain %d bytes instead of %d.", TestName, br, NELEM(buff));
			FAIL;
		} else {
			CHECK;

			if(memcmp(bp, buff, NELEM(buff)) != 0) {
				report("%s() set buffer 0 to contain the wrong data.", TestName);
				FAIL;
			} else
				CHECK;
		}

		XFree(bp);
	}

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

/* Including from file error/EAll.mc */
/*
 * Assertion XStoreBytes-2.(B)
 *   Reason: There is no known portable test method for this assertion
 * When the server fails to allocate a required resource, then
 * a BadAlloc error occurs.
 */
/* End of included file error/EAll.mc */

/* Including from file error/EAll.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	report("There is no reliable method of testing that a BadAlloc error is generated");
	tet_result(TET_UNTESTED);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of included file error/EAll.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
