/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: strgn.m,v 1.6 92/06/11 17:43:14 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XSetRegion(display, gc, r)
 *   Display	*display;
 *   GC	gc;
 *   Region	r;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XSetRegion	1
char    *TestName = "XSetRegion";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display
#define A_GC gc


/*
 * Arguments to the XSetRegion function
 */
static Display	*display;
static GC	gc;
static Region	r;


static void
strgncleanup()
{
	XDestroyRegion(r);
	tpcleanup();
}


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	gc = 0;
	r = XCreateRegion();
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
	gc = ErrdefGC;
}

/*
 * Assertion XSetRegion-1.(A)
 * A call to XSetRegion sets the clip_mask component of the gc
 * argument to the region r.
 */
static void
t001()
{
XVisualInfo		*vi;
Window			win;
GC			gc;
unsigned int    	width;
unsigned int    	height;
static XRectangle	cliprect2 = { 23, 13, 53, 63 };
static XRectangle	cliprect = { 0,0, 1,1};
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	resetvinf(VI_WIN);
	nextvinf(&vi);
	win = makewin(display, vi); /* Makes a window with bg W_BG. */

	gc = makegc(display, win);
	XSetFillStyle(display, gc, FillSolid);
	XSetFunction(display, gc, GXxor);
	XSetForeground(display, gc, W_BG^W_FG);

	XUnionRectWithRegion(&cliprect, r, r);
	XFillRectangle(display, win, gc, cliprect.x, cliprect.y, cliprect.width, cliprect.height);

	if( checkarea(display, win, (struct area *) NULL, W_BG, W_BG, CHECK_ALL|CHECK_DIFFER)){
		delete("XFillRectangle() did set any pixels to non-background values.");
		return;
	} else
		CHECK;

	startcall(display);
	if (isdeleted())
		return;
	XSetRegion(display, gc, r);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}
	getsize(display, win, &width, &height);
	XFillRectangle(display, win, gc, 0,0, width, height);

	if( checkarea(display, win, (struct area *) NULL, W_BG, W_BG, CHECK_ALL) == 0){
		FAIL;
	} else
		CHECK;
	
	CHECKPASS(2);

	strgncleanup();
	pfcount(pass, fail);
}

/* Including from file error/EGC.mc */
/*
 * Assertion XSetRegion-2.(A)
 * When the GC argument does not name a defined GC, then a
 * BadGC error occurs.
 */
/* End of included file error/EGC.mc */

/* Including from file error/EGC.mc */
static void
t002()
{
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	seterrdef();


	A_GC = badgc(A_DISPLAY);

	startcall(display);
	if (isdeleted())
		return;
	XSetRegion(display, gc, r);
	endcall(display);
	if (geterr() != BadGC) {
		report("Got %s, Expecting BadGC", errorname(geterr()));
		FAIL;
	}

	if (geterr() == BadGC)
		PASS;
	else
		FAIL;
	strgncleanup();
	pfcount(pass, fail);
}

/* End of included file error/EGC.mc */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	t002, 2,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
