/*
 * Copyright 1990, 1991 by the Massachusetts Institute of Technology and
 * UniSoft Group Limited.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the names of MIT and UniSoft not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  MIT and UniSoft
 * make no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * $XConsortium: unnrgn.m,v 1.6 92/06/11 17:43:45 rws Exp $
 * 
 */
/*
 * SYNOPSIS:
 *   
 *   XUnionRegion(sra, srb, dr_return)
 *   Region		sra;
 *   Region		srb;
 *   Region		dr_return;
 */

/* $XConsortium: mcinclude.mc,v 1.2 92/06/29 19:41:37 rws Exp $ */

#include	<stdlib.h>
#include	"xtest.h"
#include	"Xlib.h"
#include	"Xutil.h"
#include	"Xresource.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
/* $XConsortium: mcextern.mc,v 1.2 92/06/29 19:41:35 rws Exp $ */

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XUnionRegion	1
char    *TestName = "XUnionRegion";

/*
 * Defines for different argument types
 */


/*
 * Arguments to the XUnionRegion function
 */
static Region		sra;
static Region		srb;
static Region		dr_return;


int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	sra = 0;
	srb = 0;
	dr_return = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

/*
 * Assertion XUnionRegion-1.(A)
 * A call to XUnionRegion computes the union of the regions
 * sra and srb and places the result in the region dr_return.
 */
static void
t001()
{
int			i;
int			j;
Region			R1;
Region			R2;
Region			R3;
Region			Rt;
static XPoint		tri1[] = { {5,5}, {30,5}, {5,20} };
static XPoint		tri2[] = { {5,20}, {30,20}, {30,5} };
static XRectangle	rect1 = { 5,5, 25,15};
XRectangle		rect2;
XRectangle		rrect;
int 	pass = 0, fail = 0;

	tpstartup();
	setargs();

	R1 = makeregion();
	R2 = makeregion();
	R3 = makeregion();
	Rt = makeregion();

	if(isdeleted()) return;

	R1 = XPolygonRegion(tri1, NELEM(tri1), WindingRule);
	R2 = XPolygonRegion(tri2, NELEM(tri2), WindingRule);

	sra = R1;
	srb = R2;
	dr_return = R3;
	startcall(Dsp);
	if (isdeleted())
		return;
	XUnionRegion(sra, srb, dr_return);
	endcall(Dsp);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	XUnionRectWithRegion(&rect1, Rt, Rt);

	if( XEqualRegion(Rt, R3) != True) {
		report("%s() did not unify two triangular regions into the correct rectangular region.", TestName);
		FAIL;
	} else
		CHECK;


	R2 = makeregion();

	srb = R2;
	dr_return = R2;
	rect2.width=3;
	rect2.height=3;
	for(j=5; j<18; j++) {
		rect2.y=j;
		for(i=5; i<28; i++) {

			if((R1 = XCreateRegion()) == (Region) NULL) {
				delete("XCreateRegion() returned NULL.");
				return;
			} else
				if(i==27 && j==17) CHECK;

			rect2.x=i;
			XUnionRectWithRegion(&rect2, R1, R1);
			sra = R1;
			startcall(Dsp);
			if (isdeleted())
				return;
			XUnionRegion(sra, srb, dr_return);
			endcall(Dsp);
			if (geterr() != Success) {
				report("Got %s, Expecting Success", errorname(geterr()));
				FAIL;
			}
			XDestroyRegion(R1);
		}
	}
	
	if( XEqualRegion(Rt, R2) != True) {
		report("%s() did not unify a set of over-lapping rectangular regions into the correct rectangular region.", TestName);
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	t001, 1,
	NULL, 0
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
