/*
 * XeviQueryVersion.c  
 *
 * Copyright (c) 2005 Intel Corporation
 * All rights reserved except as granted by this License.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the "Artistic License" which comes with this
 * Kit, with the following modification:
 * a) "executable(s)" should be interpreted to include
 * "test case(s)"
 * b) if you wish to make changes as defined in clause 2 and 3, and
 * distribute a modified version of this package, then
 * clauses 3c and 4c are required
 * c) Clause 7 is rephrased as follows: "Subroutines supplied by you
 * and linked into this Package shall not be considered part of this
 * Package".
 *
 *
 * Intel Corporation DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL X/OPEN OR IBM CORP. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * You should have received a copy of the Artistic License with this
 * Kit, in the file named "Artistic".  If not, we'll be glad to provide one.
 */

>>TITLE	XeviQueryVersion	CHEVI
Bool
XeviQueryVersion (Display *dpy, int *major_version_return, int *minor_version_return);
>>EXTERN
#include <X11/Xlibint.h>
#include <X11/extensions/XEVI.h>
#include <X11/extensions/XEVIstr.h>
#include <X11/extensions/Xext.h>
#include <X11/extensions/extutil.h>
#include <X11/Xutil.h>
static XExtensionInfo *xevi_info;/* needs to move to globals.c */
static /* const */ char *xevi_extension_name = EVINAME;
#define XeviCheckExtension(dpy,i,val) \
  XextCheckExtension (dpy, i, xevi_extension_name, val)
/*****************************************************************************
 *                                                                           *
 *                         private utility routines                          *
 *                                                                           *
 *****************************************************************************/
static /* const */ XExtensionHooks xevi_extension_hooks = {
    NULL,                       /* create_gc */
    NULL,                       /* copy_gc */
    NULL,                       /* flush_gc */
    NULL,                       /* free_gc */
    NULL,                       /* create_font */
    NULL,                       /* free_font */
    NULL,                       /* close_display */
    NULL,                       /* wire_to_event */
    NULL,                       /* event_to_wire */
    NULL,                       /* error */
    NULL,                       /* error_string */
};
static XEXT_GENERATE_FIND_DISPLAY (find_display, xevi_info,
                                   xevi_extension_name,
                                   &xevi_extension_hooks, 0, NULL)

>>ASSERTION Good A
XeviQueryVersion returns True if the request succeeded; the values of the major and minor protocol version supported by the server are returned in major_version_return and minor_version_return. 
>>STRATEGY
If the display supports the EVI extension:
	- initiatiate the call
	- validate the major and minor numbers

>>CODE
	Display *client2;
	int major = -1;
	int minor = -1;

	client2 = XOpenDisplay( config.display );
        if( (Display *)NULL ==  client2 ) 
	{
                delete("could not create new client");
                return;
        }  else 
		CHECK;

	if ( !XeviQueryExtension( client2 ) )
	{
		delete( "Display does not support extensions." );
		XCloseDisplay( client2 );
		return;
	} else 
		CHECK;

	if ( !XeviQueryVersion( client2, &major, &minor ) ||
		0 == major  || -1 == major || -1 == minor )
	{
		/* Test has failed */
		report( "Failed to obtain EVI vesion or version numbers are invalid." );
		FAIL;
	} else 
		CHECK;

	XCloseDisplay( client2 );
	CHECKPASS(3);


>>ASSERTION Good A
With the exception of XeviQueryExtension, if any of these routines are called 
with a display that does not support the extension, the ExtensionErrorHandler 
(which can be set with XSetExtensionErrorHandler and functions the same way as 
XSetErrorHandler) will be called and the function will then return.
>>STRATEGY
If the display supports the EVI extension,
	- munge the Display structure so it appears to not support the extension
	- Call the QueryVerison function and verify proper error handling.
		- function should return FALSE
	
Else
	
	- Call the QueryVerison function and verify proper error handling.
		- function should return FALSE
End
>>CODE
	Display *client2;
	int major = -1;
	int minor = -1;
	XExtDisplayInfo *info;

	client2 = XOpenDisplay( config.display );
        if( (Display *)NULL ==  client2 ) 
	{
                delete("could not create new client");
                return;
        }  else 
		CHECK;

	if ( XeviQueryExtension( client2 ) )
	{
		client2->flags = 1;
		if ( XeviQueryVersion( client2, &major, &minor ) )
		{
			/* Test has failed */
			report( "XeviQueryVersion should have returned FALSE." );
			FAIL;
		} else 
			CHECK;
	} else if ( XeviQueryVersion( client2, &major, &minor ) )
	{
		/* Test has failed */
		report( "XeviQueryVersion should have returned FALSE." );
		FAIL;
	} else 
		CHECK;

	XCloseDisplay( client2 );
	CHECKPASS(2);

